@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-file-style: "bsd" -*- */
 /* armor.c - Armor filters
  *        Copyright (C) 2001, 2002, 2003 Timo Schulz
- *        Copyright (C) 1998-2001 Free Software Foundation, Inc.
+ *        Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of OpenCDK.
  *
@@ -25,6 +24,7 @@
  * Heavily modified by Brendan Cully <brendan@kublai.com> (Mutt)
  * Modify the code for generic use by Timo Schulz <twoaday@freakmail.de>
  */
+/* X-TODO-STATUS: OK */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -109,7 +109,7 @@ static const char * valid_headers[] = {
     NULL
 };
 
-static char b64chars[64] =
+static char b64chars[] =
 "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
 static int index64[128] = {
@@ -126,12 +126,12 @@ static int index64[128] = {
 
 /* encode a raw binary buffer to a null-terminated base64 strings */
 static int
-base64_encode (byte * out, const byte * in, size_t len, size_t olen)
+base64_encode( byte * out, const byte * in, size_t len, size_t olen )
 {
-    if (!out || !in)
+    if( !out || !in )
         return CDK_Inv_Value;
 
-    while (len >= 3 && olen > 10) {
+    while( len >= 3 && olen > 10 ) {
         *out++ = b64chars[in[0] >> 2];
         *out++ = b64chars[((in[0] << 4) & 0x30) | (in[1] >> 4)];
         *out++ = b64chars[((in[1] << 2) & 0x3c) | (in[2] >> 6)];
@@ -142,11 +142,11 @@ base64_encode (byte * out, const byte * in, size_t len, size_t olen)
     }
 
     /* clean up remainder */
-    if (len > 0 && olen > 4) {
+    if( len > 0 && olen > 4 ) {
         byte fragment = 0;
         *out++ = b64chars[in[0] >> 2];
         fragment = (in[0] << 4) & 0x30;
-        if (len > 1)
+        if( len > 1 )
             fragment |= in[1] >> 4;
         *out++ = b64chars[fragment];
         *out++ = (len < 2) ? '=' : b64chars[(in[1] << 2) & 0x3c];
@@ -160,66 +160,68 @@ base64_encode (byte * out, const byte * in, size_t len, size_t olen)
 /* Convert '\0'-terminated base64 string to raw byte buffer.
    Returns length of returned buffer, or -1 on error. */
 static int
-base64_decode (char * out, const char * in)
+base64_decode( char * out, const char * in )
 {
     int len = 0;
     byte digit1, digit2, digit3, digit4;
 
-    if (!out || !in)
+    if( !out || !in )
         return -1;
 
     do {
         digit1 = in[0];
-        if (digit1 > 127 || b64val (digit1) == BAD)
+        if( digit1 > 127 || b64val (digit1) == BAD )
             return -1;
         digit2 = in[1];
-        if (digit2 > 127 || b64val (digit2) == BAD)
+        if( digit2 > 127 || b64val (digit2) == BAD )
             return -1;
         digit3 = in[2];
-        if (digit3 > 127 || ((digit3 != '=') && (b64val (digit3) == BAD)))
+        if( digit3 > 127 || ((digit3 != '=') && (b64val (digit3) == BAD)) )
             return -1;
         digit4 = in[3];
-        if (digit4 > 127 || ((digit4 != '=') && (b64val (digit4) == BAD)))
+        if( digit4 > 127 || ((digit4 != '=') && (b64val (digit4) == BAD)) )
             return -1;
         in += 4;
 
         /* digits are already sanity-checked */
         *out++ = (b64val (digit1) << 2) | (b64val (digit2) >> 4);
         len++;
-        if (digit3 != '=') {
+        if( digit3 != '=' ) {
             *out++ = ((b64val (digit2) << 4) & 0xf0) | (b64val (digit3) >> 2);
             len++;
-            if (digit4 != '=') {
+            if( digit4 != '=' ) {
                 *out++ = ((b64val (digit3) << 6) & 0xc0) | b64val (digit4);
                 len++;
 	    }
 	}
-    }
-    while (*in && digit4 != '=');
-
+    } while( *in && digit4 != '=' );
     return len;
 }
 
 
 static int
-is_compressed (CDK_STREAM inp)
+is_compressed( cdk_stream_t inp, int *r_zipalgo )
 {
     char buf[128], plain[256];
     int nread, check = 0, pkttype;
 
-    cdk_stream_seek (inp, 0);
-    while (!cdk_stream_eof (inp)) {
-        nread = _cdk_stream_gets (inp, buf, sizeof buf-1);
-        if (!nread)
+    if( r_zipalgo )
+        *r_zipalgo = 0;
+    cdk_stream_seek( inp, 0 );
+    while( !cdk_stream_eof( inp ) ) {
+        nread = _cdk_stream_gets( inp, buf, sizeof buf-1 );
+        if( !nread )
             break;
-        if (nread == 1 && !cdk_stream_eof (inp)
-            && (nread = _cdk_stream_gets (inp, buf, sizeof buf-1))) {
-            base64_decode (plain, buf);
-            if (!(*plain & 0x80))
+        if( nread == 1 && !cdk_stream_eof( inp )
+            && (nread = _cdk_stream_gets( inp, buf, sizeof buf-1)) ) {
+            base64_decode( plain, buf );
+            if( !(*plain & 0x80) )
                 break;
             pkttype = *plain & 0x40 ? (*plain & 0x3f) : ((*plain >> 2) & 0xf);
-            if (pkttype == CDK_PKT_COMPRESSED)
-                check = *(plain + 1);
+            if( pkttype == CDK_PKT_COMPRESSED && r_zipalgo ) {
+                _cdk_log_debug( "armor compressed (algo=%d)\n", *(plain+1) );
+                *r_zipalgo = *(plain + 1);
+            }
             break;
         }
     }
@@ -227,68 +229,52 @@ is_compressed (CDK_STREAM inp)
 }
 
 
-static u16
-check_armor (CDK_STREAM inp)
+static int
+check_armor( cdk_stream_t inp, int *r_zipalgo )
 {
     char buf[4096];
     size_t nread;
-    int check = 0, alg;
+    int check = 0;
 
-    nread = cdk_stream_read (inp, buf, sizeof buf-1);
-    if (nread > 0) {
+    nread = cdk_stream_read( inp, buf, sizeof buf-1 );
+    if( nread > 0 ) {
         buf[nread] = '\0';
-        if (strstr (buf, "-----BEGIN PGP"))
-            check = 1 << 8;
-        if (check && (alg = is_compressed (inp)))
-            check |= alg;
-        cdk_stream_seek (inp, 0);
+        if( strstr( buf, "-----BEGIN PGP" ) ) {
+            is_compressed( inp, r_zipalgo );
+            check = 1;
+        }
+        cdk_stream_seek( inp, 0 );
     }
     return check;
 }
 
 
 static int
-is_armored (int ctb)
+is_armored( int ctb )
 {
     int pkttype = 0;
 
-    if (!(ctb & 0x80))
+    if( !(ctb & 0x80) )
         return 1; /* invalid packet: assume it is armored */
     pkttype = ctb & 0x40 ? (ctb & 0x3f) : ((ctb >> 2) & 0xf);
 
-    switch (pkttype) {
-    case CDK_PKT_MARKER:
-    case CDK_PKT_SYMKEY_ENC:
-    case CDK_PKT_ONEPASS_SIG:
-    case CDK_PKT_PUBLIC_KEY:
-    case CDK_PKT_SECRET_KEY:
-    case CDK_PKT_PUBKEY_ENC:
-    case CDK_PKT_SIGNATURE:
-    case CDK_PKT_OLD_COMMENT:
-    case CDK_PKT_PLAINTEXT:
-    case CDK_PKT_COMPRESSED:
-    case CDK_PKT_ENCRYPTED:
-        return 0; /* seems to be a regular packet: not armored */
+    switch( pkttype ) {
+      case CDK_PKT_MARKER:
+      case CDK_PKT_SYMKEY_ENC:
+      case CDK_PKT_ONEPASS_SIG:
+      case CDK_PKT_PUBLIC_KEY:
+      case CDK_PKT_SECRET_KEY:
+      case CDK_PKT_PUBKEY_ENC:
+      case CDK_PKT_SIGNATURE:
+      case CDK_PKT_LITERAL:
+      case CDK_PKT_COMPRESSED:
+      case CDK_PKT_ENCRYPTED:
+          return 0; /* seems to be a regular packet: not armored */
     }
     return 1;
 }
 
 
-static void
-encode_crc (unsigned crc, char *crcbuf)
-{
-    byte crcbuf2[3];
-  
-    crcbuf2[0] = crc >> 16;
-    crcbuf2[1] = crc >> 8;
-    crcbuf2[2] = crc;
-    crcbuf[0] = b64chars[crcbuf2[0] >> 2];
-    crcbuf[1] = b64chars[((crcbuf2[0] << 4) & 0x30) | (crcbuf2[1] >> 4)];
-    crcbuf[2] = b64chars[((crcbuf2[1] << 2) & 0x3c) | (crcbuf2[2] >> 6)];
-    crcbuf[3] = b64chars[crcbuf2[2] & 0x3f];
-}
-
-
 static unsigned
 update_crc( unsigned crc, const byte *buf, size_t count )
 {
@@ -303,34 +289,35 @@ update_crc( unsigned crc, const byte *buf, size_t count )
 }
 
 
-static int
+static cdk_error_t
 armor_encode( void * opaque, FILE * in, FILE * out )
 {
-    armor_filter_s * afx = opaque;
+    armor_filter_t * afx = opaque;
     struct stat statbuf;
     char crcbuf[5], buf[128], raw[49];
+    byte crcbuf2[3];
     size_t nread = 0;
     const char * lf;
-    int rc = 0;
+    cdk_error_t rc = 0;
 
-    if (!afx)
+    if( !afx )
         return CDK_Inv_Value;
-    if (afx->idx < 0 || afx->idx > DIM (armor_begin)
-        || afx->idx2 < 0 || afx->idx2 > DIM (armor_end))
+    if( afx->idx < 0 || afx->idx > DIM (armor_begin)
+        || afx->idx2 < 0 || afx->idx2 > DIM (armor_end) )
         return CDK_Inv_Value;
 
-    _cdk_log_debug ("armor filter: encode\n");
+    _cdk_log_debug( "armor filter: encode\n" );
   
-    memset (crcbuf, 0, sizeof crcbuf);
+    memset( crcbuf, 0, sizeof crcbuf );
 
     lf = afx->le ? afx->le : LF;
-    fprintf (out, "-----%s-----%s", armor_begin[afx->idx], lf);
-    fprintf (out, "Version: OpenPrivacy "VERSION"%s", lf);
-    if (afx->hdrlines)
-        fwrite (afx->hdrlines, 1, strlen (afx->hdrlines), out);
-    fprintf (out, "%s", lf);
+    fprintf( out, "-----%s-----%s", armor_begin[afx->idx], lf );
+    fprintf( out, "Version: OpenPrivacy "VERSION"%s", lf );
+    if( afx->hdrlines)
+        fwrite( afx->hdrlines, 1, strlen( afx->hdrlines ), out );
+    fprintf( out, "%s", lf );
 
-    if (fstat (fileno (in), &statbuf))
+    if( fstat( fileno( in ), &statbuf ) )
         return CDK_General_Error;
 
     while( !feof( in ) ) {
@@ -347,7 +334,13 @@ armor_encode( void * opaque, FILE * in, FILE * out )
     }
 
     if( !rc ) {
-        encode_crc( afx->crc, crcbuf );
+        crcbuf2[0] = afx->crc >> 16;
+        crcbuf2[1] = afx->crc >> 8;
+        crcbuf2[2] = afx->crc;
+        crcbuf[0] = b64chars[crcbuf2[0] >> 2];
+        crcbuf[1] = b64chars[((crcbuf2[0] << 4) & 0x30) |( crcbuf2[1] >> 4)];
+        crcbuf[2] = b64chars[((crcbuf2[1] << 2) & 0x3c) |( crcbuf2[2] >> 6)];
+        crcbuf[3] = b64chars[crcbuf2[2] & 0x3f];
         fprintf( out, "=%s%s", crcbuf, lf );
         fprintf( out, "-----%s-----%s", armor_end[afx->idx2], lf );
     }
@@ -363,31 +356,35 @@ armor_encode( void * opaque, FILE * in, FILE * out )
  * Check if the stream contains armored data.
  **/
 int
-cdk_armor_filter_use (CDK_STREAM inp)
+cdk_armor_filter_use( cdk_stream_t inp )
 {
     int c = 0, check = 0;
+    int zipalgo;
 
-    c = cdk_stream_getc (inp);
-    if (c == EOF)
+    c = cdk_stream_getc( inp );
+    if( c == EOF )
         return 0; /* EOF, doesn't matter whether armored or not */
-    cdk_stream_seek (inp, 0);
-    check = is_armored (c);
-    if (check)
-        check = check_armor (inp);
+    cdk_stream_seek( inp, 0 );
+    check = is_armored( c );
+    if( check ) {
+        check = check_armor( inp, &zipalgo );
+        if( zipalgo )
+            cdk_stream_control( inp, CDK_STREAMCTL_COMPRESSED, zipalgo );
+    }
     return check;
 }
 
 
 static int
-search_header (const char *buf, const char **array)
+search_header( const char *buf, const char **array )
 {
     const char *s;
     int i;
 
-    if (strlen (buf) < 5 || strncmp (buf, "-----", 5))
+    if( strlen( buf ) < 5 || strncmp( buf, "-----", 5 ) )
         return -1;  
-    for (i = 0; (s = array[i]); i++) {
-        if (!strncmp (s, buf + 5, strlen (s)))
+    for( i = 0; (s = array[i]); i++ ) {
+        if( !strncmp( s, buf + 5, strlen( s ) ) )
             return i;
     }
     return -1;
@@ -395,68 +392,68 @@ search_header (const char *buf, const char **array)
 
 
 const char *
-_cdk_armor_get_lineend (void)
+_cdk_armor_get_lineend( void )
 {
-  return LF;
+    return LF;
 }
 
 
-static int
-armor_decode (void * opaque, FILE *in, FILE *out)
+static cdk_error_t
+armor_decode( void * opaque, FILE * in, FILE * out )
 {
-    armor_filter_s * afx = opaque;
+    armor_filter_t * afx = opaque;
     const char * s = NULL;
     char buf[127];
-    byte raw[128];
+    byte raw[128], crcbuf[4];
     u32 crc2 = 0;
     size_t nread = 0;
     int i, pgp_data = 0;
-    int rc = 0;
+    cdk_error_t rc = 0;
 
-    if (!afx)
+    if( !afx )
         return CDK_Inv_Value;
   
-    _cdk_log_debug ("armor filter: decode\n");
+    _cdk_log_debug( "armor filter: decode\n" );
   
-    fseek (in, 0, SEEK_SET);
+    fseek( in, 0, SEEK_SET );
 
     /* Search the begin of the message */
-    while (!feof (in) && !pgp_data) {
-        s = fgets (buf, sizeof buf-1, in);
-        if (!s)
+    while( !feof( in ) && !pgp_data ) {
+        s = fgets( buf, sizeof buf-1, in );
+        if( !s )
             break;
-        afx->idx = search_header (buf, armor_begin);
-        if (afx->idx >= 0)
+        afx->idx = search_header( buf, armor_begin );
+        if( afx->idx >= 0 )
             pgp_data = 1;
     }
 
-    if (feof (in) || !pgp_data) {
+    if( feof( in ) || !pgp_data ) {
         rc = CDK_Armor_Error; /* no data found */
         goto leave;
     }
 
     /* Parse header until the empty line is reached */
-    while (!feof (in)) {
-        s = fgets (buf, sizeof buf-1, in);
-        if (!s)
+    while( !feof( in ) ) {
+        s = fgets( buf, sizeof buf-1, in );
+        if( !s )
             goto leave;
-        if (strlen (s) == strlen (LF)) {
+        if( strlen( s ) == strlen( LF ) ) {
             rc = 0;
             break; /* empty line */
 	}
         /* From RFC2440: OpenPGP should consider improperly formatted Armor
            Headers to be corruption of the ASCII Armor. A colon and a single
            space separate the key and value. */
-        if (!strstr (buf, ": ")) {
-            printf( "CORRUPTED HEADER\n" );
+        if( !strstr( buf, ": " ) ) {
             rc = CDK_Armor_Error;
             goto leave;
 	}
-        for (rc = 1, i = 0; (s = valid_headers[i]); i++) {
-            if (!strncmp (s, buf, strlen (s)))
+        rc = CDK_General_Error;
+        for( i = 0; (s = valid_headers[i]); i++ ) {
+            if( !strncmp( s, buf, strlen( s ) ) )
                 rc = 0;
 	}
-        if (rc) {
+        if( rc ) {
             /* From RFC2440: Unknown keys should be reported to the user,
                but OpenPGP should continue to process the message. */
             _cdk_log_info ("unknown header: `%s'\n", buf);
@@ -465,47 +462,46 @@ armor_decode (void * opaque, FILE *in, FILE *out)
     }
 
     /* Read the data body */
-    while (!feof (in)) {
-        s = fgets (buf, sizeof buf-1, in);
-        if (!s)
+    while( !feof( in ) ) {
+        s = fgets( buf, sizeof buf-1, in );
+        if( !s )
             break;
         buf[strlen (buf) - strlen (LF)] = '\0';
-        if (buf[0] == '=' && strlen (s) == 5) { /* CRC */
-            byte crcbuf[4];
-            memset (crcbuf, 0, sizeof crcbuf);
-            base64_decode (crcbuf, buf + 1);
-            crc2 = crcbuf[0] << 16 | crcbuf[1] << 8 | crcbuf[2];
+        if( buf[0] == '=' && strlen( s ) == 5 ) { /* CRC */
+            memset( crcbuf, 0, sizeof crcbuf );
+            base64_decode( crcbuf, buf + 1 );
+            crc2 = (crcbuf[0] << 16) | (crcbuf[1] << 8) | crcbuf[2];
             break; /* stop here */
 	}
         else {
-            nread = base64_decode (raw, buf);
-            if (!nread)
+            nread = base64_decode( raw, buf );
+            if( !nread )
                 break;
-            afx->crc = update_crc (afx->crc, raw, nread);
-            fwrite (raw, 1, nread, out);
+            afx->crc = update_crc( afx->crc, raw, nread );
+            fwrite( raw, 1, nread, out );
 	}
     }
 
     /* Search the tail of the message */
-    s = fgets (buf, sizeof buf-1, in);
-    if (s) {
-        buf[strlen (buf) - strlen (LF)] = '\0';
+    s = fgets( buf, sizeof buf-1, in );
+    if( s ) {
+        buf[strlen( buf ) - strlen( LF )] = '\0';
         rc = CDK_General_Error;
-        afx->idx2 = search_header (buf, armor_end);
-        if (afx->idx2 >= 0)
+        afx->idx2 = search_header( buf, armor_end );
+        if( afx->idx2 >= 0 )
             rc = 0;
     }
 
     /* this catches error when no tail was found or the header is
        different then the tail line. */
-    if (rc || afx->idx != afx->idx2) {
-        printf( "index does not match\n" );
+    if( rc || afx->idx != afx->idx2 )
         rc = CDK_Armor_Error;
-    }
 
     afx->crc_okay = (afx->crc == crc2) ? 1 : 0;
-    if (!afx->crc_okay && !rc)
+    if( !afx->crc_okay && !rc ) {
+        _cdk_log_debug( "file crc=%X afx_crc=%X\n", crc2, afx->crc );
         rc = CDK_Armor_CRC_Error;
+    }
 
  leave:
     return rc;
@@ -520,36 +516,39 @@ armor_decode (void * opaque, FILE *in, FILE *out)
  *
  * Protect a file with ASCII armor.
  **/
-int
-cdk_file_armor (CDK_HD hd, const char * file, const char * output)
+cdk_error_t
+cdk_file_armor( cdk_ctx_t hd, const char * file, const char * output )
 {
-    CDK_STREAM inp, out;
+    cdk_stream_t inp, out;
     int rc;
 
     rc = _cdk_check_args( hd->opt.overwrite, file, output );
     if( rc )
         return rc;
     
-    rc = cdk_stream_open (file, &inp);
-    if (rc)
+    rc = cdk_stream_open( file, &inp );
+    if( rc )
         return rc;
     
-    rc = cdk_stream_new (output, &out);
-    if (rc) {
-        cdk_stream_close (inp);
+    rc = cdk_stream_new( output, &out );
+    if( rc ) {
+        cdk_stream_close( inp );
         return rc;
     }
 
-    cdk_stream_set_armor_flag (out, CDK_ARMOR_MESSAGE);
-    if (hd->opt.compress)
-        cdk_stream_set_compress_flag( out, hd->compress.algo, hd->compress.level );
-    cdk_stream_set_literal_flag (out, 0, file);
+    cdk_stream_set_armor_flag( out, CDK_ARMOR_MESSAGE );
+    if( hd->opt.compress )
+        rc = cdk_stream_set_compress_flag( out, hd->compress.algo,
+                                           hd->compress.level );
+    if( !rc )
+        rc = cdk_stream_set_literal_flag( out, 0, file );
+    if( !rc )
+        rc = cdk_stream_kick_off( inp, out );
+    if( !rc )
+        rc = _cdk_stream_get_errno( out );
 
-    rc = cdk_stream_kick_off (inp, out);
-
-    cdk_stream_close (out);
-    cdk_stream_close (inp);
-  
+    cdk_stream_close( out );
+    cdk_stream_close( inp );
     return rc;
 }
 
@@ -561,38 +560,41 @@ cdk_file_armor (CDK_HD hd, const char * file, const char * output)
  *
  * Remove ASCII armor from a file.
  **/
-int
+cdk_error_t
 cdk_file_dearmor (const char * file, const char * output)
 {
-    CDK_STREAM inp, out;
-    int rc = 0, isarmor = 0;
+    cdk_stream_t inp, out;
+    int rc = 0, zipalgo;
 
     rc = _cdk_check_args( 1, file, output );
     if( rc )
         return rc;
-  
-    rc = cdk_stream_open (file, &inp);
-    if (rc)
+
+    rc = cdk_stream_open( file, &inp );
+    if( rc )
         return rc;
 
-    rc = cdk_stream_create (output, &out);
-    if (rc) {
-        cdk_stream_close (inp);
+    rc = cdk_stream_create( output, &out );
+    if( rc ) {
+        cdk_stream_close( inp );
         return rc;
     }
 
-    isarmor = cdk_armor_filter_use (inp);
-    if (isarmor) {
-        cdk_stream_set_literal_flag (inp, 0, NULL);
-        if (isarmor & 3)
-            cdk_stream_set_compress_flag (inp, isarmor & 3, 0);
-        cdk_stream_set_armor_flag (inp, 0);
-        rc = cdk_stream_kick_off (inp, out);
+    if( cdk_armor_filter_use( inp ) ) {
+        rc = cdk_stream_set_literal_flag( inp, 0, NULL );
+        zipalgo = cdk_stream_control( inp, CDK_STREAMCTL_COMPRESSED, -1 );
+        if( zipalgo )
+            rc = cdk_stream_set_compress_flag( inp, zipalgo, 0 );
+        if( !rc )
+            rc = cdk_stream_set_armor_flag( inp, 0 );
+        if( !rc )
+            rc = cdk_stream_kick_off( inp, out );
+        if( !rc )
+            rc = _cdk_stream_get_errno( inp );
     }
 
-    cdk_stream_close (inp);
-    cdk_stream_close (out);
-  
+    cdk_stream_close( inp );
+    cdk_stream_close( out );
     return rc;
 }
 
@@ -605,7 +607,7 @@ _cdk_filter_armor( void * opaque, int ctl, FILE * in, FILE * out )
     else if( ctl == STREAMCTL_WRITE )
         return armor_encode( opaque, in, out );
     else if( ctl == STREAMCTL_FREE ) {
-        armor_filter_s * afx = opaque;
+        armor_filter_t * afx = opaque;
         if( afx ) {
             _cdk_log_debug( "free armor filter\n" );
             afx->idx = afx->idx2 = 0;
