@@ -1,5 +1,4 @@
-/* -*- Mode: C; c-file-style: "bsd" -*-
- * context.h
+/* context.h
  *       Copyright (C) 2002, 2003 Timo Schulz
  *
  * This file is part of OpenCDK.
@@ -45,20 +44,22 @@ struct cdk_verify_result_s {
     int sig_len;
     int sig_status;
     int sig_flags;
-    unsigned long keyid[2];
-    unsigned long created;
-    unsigned long expires;
+    u32 keyid[2];
+    u32 created;
+    u32 expires;
     int pubkey_algo;
     int digest_algo;
     char * notation;
-    unsigned char * sig_data;
+    char * user_id;
+    char * policy_url;
+    byte * sig_data;
 };
 
 struct cdk_s2k_s {
     int mode;
-    unsigned char hash_algo;
-    unsigned char salt[8];
-    unsigned long count;
+    byte hash_algo;
+    byte salt[8];
+    u32 count;
 };
 
 struct cdk_ctx_s {
@@ -75,6 +76,7 @@ struct cdk_ctx_s {
         int cipher_algo;
     } _s2k;
     struct {
+        unsigned blockmode:1;
         unsigned armor:1;
         unsigned textmode:1;
         unsigned compress:1;
@@ -104,43 +106,42 @@ struct cdk_ctx_s {
 };
 
 struct cdk_prefitem_s {
-    unsigned char type;
-    unsigned char value;
+    byte type;
+    byte value;
 };
 
 struct cdk_desig_revoker_s {
     struct cdk_desig_revoker_s * next;
-    unsigned char class;
-    unsigned char algid;
-    unsigned char fpr[20];
+    byte class;
+    byte algid;
+    byte fpr[20];
 };
 
 struct cdk_subpkt_s {
     struct cdk_subpkt_s * next;
-    unsigned long size;
-    unsigned char type;
-    unsigned char d[1];  
+    u32 size;
+    byte type;
+    byte d[1];  
 };
 
 struct cdk_mpi_s {
-    unsigned short bits;
-    unsigned short bytes;
-    unsigned char data[1];
+    u16 bits;
+    u16 bytes;
+    byte data[1];
 };
 
 struct key_idx_s {
-    unsigned long offset;
-    unsigned long keyid[2];
-    unsigned char fpr[20]; 
+    u32 offset;
+    u32 keyid[2];
+    byte fpr[20]; 
 };
 
 
-struct cdk_dbsearch_s 
-{
+struct cdk_dbsearch_s {
     union {
         char * pattern;
-        unsigned long keyid[2];
-        unsigned char fpr[20];
+        u32 keyid[2];
+        byte fpr[20];
     } u;
     int type;
 };
@@ -149,7 +150,7 @@ typedef struct cdk_dbsearch_s *cdk_dbsearch_t;
 
 struct key_table_s {
     struct key_table_s * next;
-    unsigned long offset;
+    u32 offset;
     cdk_dbsearch_t desc;   
 };
 
@@ -177,6 +178,7 @@ struct cdk_keylist_s {
         cdk_pkt_pubkey_t pk;
         cdk_pkt_seckey_t sk;
     } key;
+    int version;
     int type;  
 };
 
@@ -185,7 +187,7 @@ struct cdk_dek_s {
     int keylen;
     int use_mdc;
     unsigned rfc1991:1;
-    unsigned char key[32]; /* 256-bit */
+    byte key[32]; /* 256-bit */
 };
 
 struct cdk_strlist_s {
