@@ -1,4 +1,3 @@
-/* -*- Mode: C; c-file-style: "bsd" -*- */
 /* encrypt.c
  *       Copyright (C) 2002, 2003 Timo Schulz
  *
@@ -35,30 +34,29 @@
 
 
 struct mainproc_ctx_s {
-    CDK_DEK dek;
-    CDK_STREAM s;
-    CDK_KBNODE node;
-    CDK_STREAM tmpfp;
+    cdk_dek_t dek;
+    cdk_stream_t s;
+    cdk_kbnode_t node;
+    cdk_stream_t tmpfp;
     struct {
         unsigned present:1;
         unsigned one_pass:1;
-        GCRY_MD_HD md;
+        cdk_md_hd_t md;
         int digest_algo;
         int is_expired;
-        cdkPKT_public_key * pk;
+        cdk_pkt_pubkey_t pk;
         unsigned pt_offset;
     } sig;
-    unsigned reset:1;
     unsigned eof_seen:1;
     unsigned key_seen:1;
     char * file; /* for detached signatures */
     const char * output;
 };
-typedef struct mainproc_ctx_s *CTX;
+typedef struct mainproc_ctx_s * CTX;
 
 
 static void
-write_marker_packet( CDK_STREAM out )
+write_marker_packet( cdk_stream_t out )
 {
     byte buf[5];
 
@@ -72,24 +70,24 @@ write_marker_packet( CDK_STREAM out )
 
 
 static void
-plaintext_set_rfc1991( CDK_STREAM out )
+literal_set_rfc1991( cdk_stream_t out )
 {
-    plaintext_filter_s * pfx;
-    pfx = _cdk_stream_get_opaque( out, fPLAINTEXT );
+    literal_filter_t * pfx;
+    pfx = _cdk_stream_get_opaque( out, fLITERAL );
     if( pfx )
         pfx->rfc1991 = 1;
 }
 
 
 static int
-sym_stream_encrypt (CDK_HD hd, CDK_STREAM inp, CDK_STREAM out)
+sym_stream_encrypt( cdk_ctx_t hd, cdk_stream_t inp, cdk_stream_t out )
 {
-    CDK_PACKET pkt;
-    cdkPKT_symkey_enc * enc;
+    cdk_packet_t pkt = NULL;
+    cdk_pkt_symkey_enc_t enc;
     char * pw = NULL;
     int rc = 0;
 
-    if (!hd || !inp || !out)
+    if( !hd || !inp || !out )
         return CDK_Inv_Value;
 
     pw = _cdk_passphrase_get( hd, "Enter Passphrase: " );
@@ -110,8 +108,11 @@ sym_stream_encrypt (CDK_HD hd, CDK_STREAM inp, CDK_STREAM out)
         hd->dek->rfc1991 = 1;
         goto start; /* skip the pkt_symkey_enc packet. */
     }
+
+    if( hd->opt.blockmode )
+        _cdk_stream_set_blockmode( out, 8192 );
   
-    cdk_stream_set_cache( out, 1 );
+    cdk_stream_switch_cache( out, 1 );
     if( hd->opt.compat )
         write_marker_packet( out );
       
@@ -120,17 +121,21 @@ sym_stream_encrypt (CDK_HD hd, CDK_STREAM inp, CDK_STREAM out)
         rc = CDK_Out_Of_Core;
         goto fail;
     }
+    pkt = cdk_calloc( 1, sizeof * pkt );
+    if( !pkt ) {
+        rc = CDK_Out_Of_Core;
+        goto fail;
+    }
     enc->version = 4;
     enc->cipher_algo = hd->dek->algo;
     enc->s2k = hd->s2k;
-    cdk_pkt_init( &pkt );
-    pkt.pkttype = CDK_PKT_SYMKEY_ENC;
-    pkt.pkt.symkey_enc = enc;
-    rc = cdk_pkt_build( out, &pkt );
+    pkt->pkttype = CDK_PKT_SYMKEY_ENC;
+    pkt->pkt.symkey_enc = enc;
+    rc = cdk_pkt_write( out, pkt );
     cdk_free( enc );
     if( rc )
         goto fail;
-    cdk_stream_set_cache( out, 0 );
+    cdk_stream_switch_cache( out, 0 );
 
  start:
     if( hd->opt.armor )
@@ -141,25 +146,25 @@ sym_stream_encrypt (CDK_HD hd, CDK_STREAM inp, CDK_STREAM out)
                                       hd->compress.level );
     cdk_stream_set_literal_flag( out, 0, _cdk_stream_get_fname( inp ) );
     if( hd->opt.rfc1991 )
-        plaintext_set_rfc1991( out );
-
+        literal_set_rfc1991( out );
     rc = cdk_stream_kick_off( inp, out );
 
 fail:
     _cdk_passphrase_free( pw, pw? strlen( pw ) : 0 );
+    cdk_free( pkt );
     return rc;
 }
 
 
 static int
-use_rfc1991_format( CDK_HD hd, CDK_KEYLIST kl )
+use_rfc1991_format( cdk_ctx_t hd, cdk_keylist_t kl )
 {
-    CDK_KEYLIST l;
+    cdk_keylist_t l;
   
     if( hd->opt.rfc1991 )
         return 1;
     for( l = kl; l; l = l->next ) {
-        if( l->type == CDK_PKT_PUBLIC_KEY && l->key.pk->version == 3 )
+        if( l->version == 3 )
             return 1;
     }
     return 0;
@@ -167,11 +172,11 @@ use_rfc1991_format( CDK_HD hd, CDK_KEYLIST kl )
 
 
 static int
-select_cipher_algo( int rfc1991, CDK_KEYLIST kl )
+select_cipher_algo( int rfc1991, cdk_keylist_t kl )
 {
     int pgp2 = _cdk_is_idea_available( );
-    int def_cipher = pgp2 && rfc1991? GCRY_CIPHER_IDEA : GCRY_CIPHER_CAST5;
-    return rfc1991? def_cipher : cdk_pklist_select_algo( kl );
+    int def_cipher = pgp2 && rfc1991? CDK_CIPHER_IDEA : CDK_CIPHER_CAST5;
+    return rfc1991?def_cipher : cdk_pklist_select_algo( kl, CDK_PREFTYPE_SYM );
 } 
 
 
@@ -198,14 +203,14 @@ _cdk_check_args( int overwrite, const char * in, const char * out )
  * If remusr is NULL, then symmetric encryption is used. Via the
  * handle the caller can set or unset multiple options.
  */
-int
-cdk_stream_encrypt( CDK_HD hd, CDK_STRLIST remusr,
-                    CDK_STREAM inp, CDK_STREAM out )
+cdk_error_t
+cdk_stream_encrypt( cdk_ctx_t hd, cdk_strlist_t remusr,
+                    cdk_stream_t inp, cdk_stream_t out )
 {
-    CDK_KEYLIST pkl = NULL;
+    cdk_keylist_t pkl = NULL;
     int cipher_algo, compress_algo = 0;
     int use_rfc1991 = 0;
-    int rc = 0;
+    cdk_error_t rc = 0;
 
     if( !hd || !inp || !out )
         return CDK_Inv_Value;
@@ -213,7 +218,7 @@ cdk_stream_encrypt( CDK_HD hd, CDK_STRLIST remusr,
     if( !remusr )
         return sym_stream_encrypt( hd, inp, out );
 
-    rc = cdk_pklist_build( hd->db.pub, remusr, &pkl, GCRY_PK_USAGE_ENCR );
+    rc = cdk_pklist_build( &pkl, hd->db.pub, remusr, PK_USAGE_ENCR );
     if( rc )
         return rc;
 
@@ -235,7 +240,10 @@ cdk_stream_encrypt( CDK_HD hd, CDK_STRLIST remusr,
         cdk_dek_set_mdc_flag( hd->dek, cdk_pklist_use_mdc( pkl ) );
     hd->dek->rfc1991 = use_rfc1991;
 
-    cdk_stream_set_cache( out, 1 );
+    if( hd->opt.blockmode )
+        _cdk_stream_set_blockmode( out, 8192 );
+
+    cdk_stream_switch_cache( out, 1 );
     if( hd->opt.compat && !hd->opt.rfc1991 )
         write_marker_packet( out );
     
@@ -244,7 +252,7 @@ cdk_stream_encrypt( CDK_HD hd, CDK_STRLIST remusr,
     if( rc )
         return rc;
 
-    cdk_stream_set_cache( out, 0 );
+    cdk_stream_switch_cache( out, 0 );
 
     if( hd->opt.armor )
         cdk_stream_set_armor_flag( out, 0 );
@@ -253,7 +261,7 @@ cdk_stream_encrypt( CDK_HD hd, CDK_STRLIST remusr,
         cdk_stream_set_compress_flag( out, compress_algo, hd->compress.level );
     cdk_stream_set_literal_flag( out, 0, _cdk_stream_get_fname( inp ) );
     if( hd->dek->rfc1991 )
-        plaintext_set_rfc1991( out );
+        literal_set_rfc1991( out );
 
     return cdk_stream_kick_off( inp, out );
 }
@@ -267,12 +275,12 @@ cdk_stream_encrypt( CDK_HD hd, CDK_STRLIST remusr,
  * @output: Output file
  *
  **/
-int
-cdk_file_encrypt (CDK_HD hd, CDK_STRLIST remusr,
+cdk_error_t
+cdk_file_encrypt (cdk_ctx_t hd, cdk_strlist_t remusr,
                   const char * file, const char * output)
 {
-    CDK_STREAM inp = NULL, out = NULL;
-    int rc;
+    cdk_stream_t inp = NULL, out = NULL;
+    cdk_error_t rc;
 
     rc = _cdk_check_args( hd->opt.overwrite, file, output );
     if( !rc )
@@ -288,17 +296,7 @@ cdk_file_encrypt (CDK_HD hd, CDK_STRLIST remusr,
 
 
 static void
-set_reset( CTX c, CDK_STREAM inp )
-{
-    if( !c->reset ) {
-        cdk_stream_control( inp, CDK_STREAMCTL_RESET, 1 );
-        c->reset = 1;
-    }
-}
-
-
-static void
-write_status (CDK_HD hd, int type, const char * fmt, ...)
+write_status (cdk_ctx_t hd, int type, const char * fmt, ...)
 {
     va_list arg_ptr;
     char * buf;
@@ -311,7 +309,7 @@ write_status (CDK_HD hd, int type, const char * fmt, ...)
     n = vasprintf (&buf, fmt, arg_ptr);
     buf[n] = '\0';
     hd->callback (hd->callback_value, type, buf);
-    _cdk_vasprintf_free (buf);
+    free (buf);
     va_end (arg_ptr);
 }
 
@@ -319,21 +317,16 @@ write_status (CDK_HD hd, int type, const char * fmt, ...)
 static int
 is_openpgp_ext (const char * file)
 {
-    int yes = 0;
-
-    if (strstr (file, ".asc") || strstr (file, ".sig")
-        || strstr (file, ".gpg") || strstr (file, ".pgp"))
-        yes = 1;
-
-    return yes;
+    return (strstr (file, ".asc") || strstr (file, ".sig")
+            || strstr (file, ".gpg") || strstr (file, ".pgp")) ? 1 : 0;
 }
   
 
 static int
-hash_data_file( char * file, int digest_algo, GCRY_MD_HD * r_md )
+hash_data_file( char * file, int digest_algo, cdk_md_hd_t * r_md )
 {
-    md_filter_s * mfx;
-    CDK_STREAM s;
+    md_filter_t * mfx;
+    cdk_stream_t s;
     int rc;
 
     if( file && is_openpgp_ext( file ) )
@@ -349,16 +342,16 @@ hash_data_file( char * file, int digest_algo, GCRY_MD_HD * r_md )
     cdk_stream_read( s, NULL, 0 );
     mfx = _cdk_stream_get_opaque( s, fHASH );
     if( mfx && mfx->md )
-        *r_md = gcry_md_copy( mfx->md );
+        rc = cdk_md_copy( r_md, mfx->md );
     cdk_stream_close( s );
-    return 0;
+    return rc;
 }
 
 
 static int
-handle_symkey_enc (CTX c, CDK_HD hd, CDK_PACKET * pkt)
+handle_symkey_enc (CTX c, cdk_ctx_t hd, cdk_packet_t pkt)
 {
-    cdkPKT_symkey_enc * key;
+    cdk_pkt_symkey_enc_t key;
     char * pw = NULL;
     int rc = 0;
 
@@ -380,16 +373,18 @@ handle_symkey_enc (CTX c, CDK_HD hd, CDK_PACKET * pkt)
 
 
 static int
-get_seckey( CDK_HD hd, CDK_KEYDB_HD db, u32 * keyid, cdkPKT_secret_key **r_sk )
+get_seckey( cdk_ctx_t hd, cdk_keydb_hd_t db, u32 * keyid,
+            cdk_pkt_seckey_t*r_sk )
 {
     int rc;
 
+    if( !r_sk )
+        return CDK_Inv_Value;
     if( hd->cache.on && hd->cache.sk ) {
-        cdkPKT_secret_key * sk = hd->cache.sk;
+        cdk_pkt_seckey_t sk = hd->cache.sk;
         cdk_sk_get_keyid( sk, NULL );
         if( sk->keyid[0] == keyid[0] && sk->keyid[1] == keyid[1] ) {
-            if( r_sk )
-                *r_sk = sk;
+            *r_sk = sk;
             return 0;
         }
     }
@@ -401,10 +396,10 @@ get_seckey( CDK_HD hd, CDK_KEYDB_HD db, u32 * keyid, cdkPKT_secret_key **r_sk )
     
 
 static int
-handle_pubkey_enc( CTX c, CDK_HD hd, CDK_PACKET * pkt )
+handle_pubkey_enc( CTX c, cdk_ctx_t hd, cdk_packet_t pkt )
 {
-    cdkPKT_pubkey_enc * enc;
-    cdkPKT_secret_key * sk = NULL;
+    cdk_pkt_pubkey_enc_t enc;
+    cdk_pkt_seckey_t sk = NULL;
     int rc = 0;
 
     assert( pkt->pkttype == CDK_PKT_PUBKEY_ENC );
@@ -432,9 +427,9 @@ handle_pubkey_enc( CTX c, CDK_HD hd, CDK_PACKET * pkt )
 
 
 static int
-rfc1991_get_sesskey (CDK_DEK * r_dek, CDK_HD hd)
+rfc1991_get_sesskey (cdk_dek_t * r_dek, cdk_ctx_t hd)
 {
-    CDK_S2K_HD s2k;
+    cdk_s2k_t s2k;
     char * pw;
     int rc;
 
@@ -444,12 +439,12 @@ rfc1991_get_sesskey (CDK_DEK * r_dek, CDK_HD hd)
     pw = _cdk_passphrase_get (hd, "Enter Passphrase: ");
     if (!pw)
         return CDK_Out_Of_Core;
-    rc = cdk_s2k_new( &s2k, 0, GCRY_MD_MD5, NULL );
+    rc = cdk_s2k_new( &s2k, 0, CDK_MD_MD5, NULL );
     if( rc ) {
         _cdk_passphrase_free( pw, pw? strlen( pw ) : 0 );
         return CDK_Out_Of_Core;
     }
-    rc = cdk_dek_from_passphrase( r_dek, GCRY_CIPHER_IDEA, s2k, 0, pw );
+    rc = cdk_dek_from_passphrase( r_dek, CDK_CIPHER_IDEA, s2k, 0, pw );
     _cdk_passphrase_free( pw, pw? strlen( pw ) : 0 );
     cdk_free( s2k );
     return rc;
@@ -457,9 +452,9 @@ rfc1991_get_sesskey (CDK_DEK * r_dek, CDK_HD hd)
 
   
 static int
-handle_encrypted (CTX c, CDK_HD hd, CDK_PACKET * pkt, int use_mdc)
+handle_encrypted (CTX c, cdk_ctx_t hd, cdk_packet_t pkt, int use_mdc)
 {
-    cdkPKT_encrypted * enc;
+    cdk_pkt_encrypted_t enc;
     int pgp2_compat = _cdk_is_idea_available ();
     int rc = 0, pkttype = pkt->pkttype;
 
@@ -479,7 +474,6 @@ handle_encrypted (CTX c, CDK_HD hd, CDK_PACKET * pkt, int use_mdc)
     }
   
     enc = pkt->pkt.encrypted;
-    set_reset (c, enc->buf);
     cdk_stream_set_cipher_flag (enc->buf, c->dek, use_mdc);
     rc = cdk_stream_read (enc->buf, NULL, 0);
     if (!rc)
@@ -491,9 +485,9 @@ handle_encrypted (CTX c, CDK_HD hd, CDK_PACKET * pkt, int use_mdc)
 
 
 static int
-handle_compressed( CTX c, CDK_PACKET * pkt )
+handle_compressed( CTX c, cdk_packet_t pkt )
 {
-    cdkPKT_compressed * zip;
+    cdk_pkt_compressed_t zip;
     int rc;
 
     assert( pkt->pkttype == CDK_PKT_COMPRESSED );
@@ -508,7 +502,7 @@ handle_compressed( CTX c, CDK_PACKET * pkt )
 
 
 static int
-handle_onepass_sig( CTX c, CDK_PACKET * pkt )
+handle_onepass_sig( CTX c, cdk_packet_t pkt )
 {
     int rc = 0;
     
@@ -517,32 +511,30 @@ handle_onepass_sig( CTX c, CDK_PACKET * pkt )
     if( c->sig.md )
         return 0; /* already open */       
     c->sig.digest_algo = pkt->pkt.onepass_sig->digest_algo;
-    if( _cdk_md_test_algo( c->sig.digest_algo ) )
+    if( cdk_md_test_algo( c->sig.digest_algo ) )
         return CDK_Inv_Algo;
-    c->sig.md = gcry_md_open( c->sig.digest_algo, 0 );
-    if( !c->sig.md )
-        rc = CDK_Gcry_Error;
+    rc = cdk_md_open( &c->sig.md, c->sig.digest_algo, 0 );
     return rc;
 }
 
 
 static int
-handle_plaintext( CTX c, CDK_PACKET * pkt, CDK_STREAM * ret_out )
+handle_literal( CTX c, cdk_packet_t pkt, cdk_stream_t * ret_out )
 {
-    plaintext_filter_s * pfx;
-    cdkPKT_plaintext * pt;
-    CDK_STREAM out;
+    literal_filter_t * pfx;
+    cdk_pkt_literal_t pt;
+    cdk_stream_t out;
     const char * s;
     int rc = 0;
 
-    assert( pkt->pkttype == CDK_PKT_PLAINTEXT );
+    assert( pkt->pkttype == CDK_PKT_LITERAL );
 
     if( !ret_out )
         return CDK_Inv_Value;
 
     if( !c->tmpfp ) {
         /* fixme: handle _CONSOLE */
-        s = c->output? c->output : pkt->pkt.plaintext->name;
+        s = c->output? c->output : pkt->pkt.literal->name;
         rc = cdk_stream_create( s, ret_out );
         if( rc )
             return rc;
@@ -550,12 +542,12 @@ handle_plaintext( CTX c, CDK_PACKET * pkt, CDK_STREAM * ret_out )
     else
         *ret_out = c->tmpfp;
     out = *ret_out;
-    pt = pkt->pkt.plaintext;
+    pt = pkt->pkt.literal;
     cdk_stream_seek( c->s, c->sig.present? c->sig.pt_offset : 0 );
     cdk_stream_set_literal_flag( c->s, 0, NULL );
     if( c->sig.present ) {
         _cdk_log_debug( "enable hash filter (algo=%d)\n", c->sig.digest_algo );
-        pfx = _cdk_stream_get_opaque( c->s, fPLAINTEXT );
+        pfx = _cdk_stream_get_opaque( c->s, fLITERAL );
         if( pfx )
             pfx->md = c->sig.md;
     }
@@ -564,9 +556,9 @@ handle_plaintext( CTX c, CDK_PACKET * pkt, CDK_STREAM * ret_out )
 
 
 static byte *
-mpi_encode( cdkPKT_signature * sig )
+mpi_encode( cdk_pkt_signature_t sig )
 {
-    CDK_MPI a;
+    cdk_mpi_t a;
     byte * p;
     size_t len, i, nsig = 0, pos = 0;
     
@@ -588,8 +580,13 @@ mpi_encode( cdkPKT_signature * sig )
 
 
 static void
-set_verify_result( cdkPKT_signature * sig, Verify_Result res )
+store_verify_result( cdk_ctx_t hd, cdk_pkt_signature_t sig,
+                     _cdk_verify_result_t res )
 {
+    cdk_subpkt_t node;
+    const byte * d;
+    size_t n, n2, n3;
+    
     res->sig_len = sig->mpi[0]->bits;
     res->sig_ver = sig->version;
     res->keyid[0] = sig->keyid[0];
@@ -600,14 +597,60 @@ set_verify_result( cdkPKT_signature * sig, Verify_Result res )
     if( sig->flags.expired )
         res->sig_flags |= CDK_FLAG_SIG_EXPIRED;
     res->sig_data = mpi_encode( sig );
+    node = cdk_subpkt_find( sig->hashed, CDK_SIGSUBPKT_SIGNERS_UID );
+    if( !node )
+        node = cdk_subpkt_find( sig->unhashed, CDK_SIGSUBPKT_SIGNERS_UID );
+    if( node ) {
+        d = cdk_subpkt_get_data( node, NULL, &n );
+        if( !d )
+            return;
+        if( !_cdk_keydb_check_userid( hd->db.pub, sig->keyid, d ) ) {
+            /* check if the signer's user-id is really part of the
+               key which was used for signing. this basic check avoid
+               forged signatures by mallory. */
+            _cdk_log_debug( "user-id could not found on the issuser key.");
+            return;
+        }
+        res->user_id = cdk_calloc( 1, n+1 );
+        if( !res->user_id )
+            return;
+        memcpy( res->user_id, d, n );
+    }
+    node = cdk_subpkt_find( sig->hashed, CDK_SIGSUBPKT_NOTATION );
+    if( node ) {
+        d = cdk_subpkt_get_data( node, NULL, &n );
+        if( n < 8 )
+            return; /* invalid notation data */
+        if( !(*d & 0x80) )
+            return; /* not human readable */
+        n2 = d[4] << 8 | d[5];
+        n3 = d[6] << 8 | d[7];
+        d += 8;
+        res->notation = cdk_calloc( 1, n2+n3+1+1 );
+        if( !res->notation )
+            return;
+        memcpy( res->notation, d, n2 ); /* notation name */
+        res->notation[n2] = '=';
+        memcpy( res->notation+n2+1, d+n2, n3 ); /* notation value */
+    }
+    node = cdk_subpkt_find( sig->hashed, CDK_SIGSUBPKT_POLICY );
+    if( node ) {
+        d = cdk_subpkt_get_data( node, NULL, &n );
+        if( !d )
+            return;
+        res->policy_url = cdk_calloc( 1, n+1 );
+        if( !res->policy_url )
+            return;
+        memcpy( res->policy_url, d, n );
+    }
 }
 
     
 static int
-handle_signature (CDK_HD hd, CTX c, CDK_PACKET * pkt)
+handle_signature( cdk_ctx_t hd, CTX c, cdk_packet_t pkt )
 {
-    Verify_Result res;
-    cdkPKT_signature * sig;
+    _cdk_verify_result_t res;
+    cdk_pkt_signature_t sig;
     u32 keyid[2];
     int rc;
 
@@ -623,7 +666,7 @@ handle_signature (CDK_HD hd, CTX c, CDK_PACKET * pkt)
   
     sig = pkt->pkt.signature;
     if( !c->sig.one_pass && !c->sig.md ) {
-        if( _cdk_md_test_algo( sig->digest_algo ) )
+        if( cdk_md_test_algo( sig->digest_algo ) )
             return CDK_Inv_Algo;
         rc = hash_data_file( c->file, sig->digest_algo, &c->sig.md );
         if( rc )
@@ -631,7 +674,7 @@ handle_signature (CDK_HD hd, CTX c, CDK_PACKET * pkt)
     }
 
     cdk_sig_get_keyid( sig, keyid );
-    set_verify_result( sig, res );
+    store_verify_result( hd, sig, res );
   
     rc = cdk_keydb_get_pk( hd->db.pub, keyid, &c->sig.pk );
     if( rc ) {
@@ -646,7 +689,7 @@ handle_signature (CDK_HD hd, CTX c, CDK_PACKET * pkt)
     rc = _cdk_sig_check( c->sig.pk, sig, c->sig.md, &c->sig.is_expired );
     res->sig_status = !rc? CDK_SIGSTAT_GOOD : CDK_SIGSTAT_BAD;
     if( !rc )
-        _cdk_log_debug("Good Signature from %08lX%08lX (expired %d)\n",
+        _cdk_log_debug("good signature from %08lX%08lX (expired %d)\n",
                        keyid[0], keyid[1], c->sig.is_expired );
     return rc;
 }
@@ -660,7 +703,7 @@ free_mainproc( CTX c )
     cdk_kbnode_release( c->node );
     c->node = NULL;
     if( c->sig.present ) {
-        gcry_md_close( c->sig.md );
+        cdk_md_close( c->sig.md );
         c->sig.md = NULL;
         _cdk_free_pubkey( c->sig.pk );
         c->sig.pk = NULL;
@@ -674,10 +717,11 @@ free_mainproc( CTX c )
 
 
 static int
-do_proc_packets( CDK_HD hd, CTX c, CDK_STREAM inp, CDK_STREAM * ret_out )
+do_proc_packets( cdk_ctx_t hd, CTX c, cdk_stream_t inp,
+                 cdk_stream_t * ret_out )
 {
-    CDK_PACKET * pkt = NULL;
-    CDK_KBNODE n = NULL, node;
+    cdk_packet_t pkt = NULL;
+    cdk_kbnode_t n = NULL, node;
     const char * s;
     int rc = 0, npos, with_mdc = 0;
 
@@ -695,7 +739,7 @@ do_proc_packets( CDK_HD hd, CTX c, CDK_STREAM inp, CDK_STREAM * ret_out )
         pkt = cdk_calloc (1, sizeof *pkt);
         if (!pkt)
             return CDK_Out_Of_Core;
-        rc = cdk_pkt_parse (inp, pkt);
+        rc = cdk_pkt_read (inp, pkt);
         _cdk_log_debug ("type=%d old_ctb=%d len=%d (%d)\n",
                         pkt->pkttype, pkt->old_ctb, pkt->pktlen, rc);
         if (rc == CDK_EOF)
@@ -710,71 +754,69 @@ do_proc_packets( CDK_HD hd, CTX c, CDK_STREAM inp, CDK_STREAM * ret_out )
             _cdk_kbnode_add (c->node, n);
 
         switch (pkt->pkttype) {
-        case CDK_PKT_SYMKEY_ENC:
-            rc = handle_symkey_enc (c, hd, pkt);
-            _cdk_log_debug (" handle_symkey_enc (%d)\n", rc);
-            break;
+          case CDK_PKT_SYMKEY_ENC:
+              rc = handle_symkey_enc (c, hd, pkt);
+              _cdk_log_debug (" handle_symkey_enc (%d)\n", rc);
+              break;
           
-        case CDK_PKT_PUBKEY_ENC:
-            rc = handle_pubkey_enc (c, hd, pkt);
-            _cdk_log_debug (" handle_pubkey_enc (%d)\n", rc); 
-            break;
+          case CDK_PKT_PUBKEY_ENC:
+              rc = handle_pubkey_enc (c, hd, pkt);
+              _cdk_log_debug (" handle_pubkey_enc (%d)\n", rc); 
+              break;
           
-        case CDK_PKT_ENCRYPTED_MDC: 
-        case CDK_PKT_ENCRYPTED:
-            with_mdc = pkt->pkttype == CDK_PKT_ENCRYPTED_MDC;
-            rc = handle_encrypted (c, hd, pkt, with_mdc);
-            _cdk_log_debug (" handle_encrypted (%d)\n", rc);
-            if (!rc)
-                inp = c->s;
-            break;
+          case CDK_PKT_ENCRYPTED_MDC: 
+          case CDK_PKT_ENCRYPTED:
+              with_mdc = pkt->pkttype == CDK_PKT_ENCRYPTED_MDC;
+              rc = handle_encrypted (c, hd, pkt, with_mdc);
+              _cdk_log_debug (" handle_encrypted (%d)\n", rc);
+              if (!rc)
+                  inp = c->s;
+              break;
           
-        case CDK_PKT_COMPRESSED:
-            if (!c->s)
-                c->s = inp;
-            set_reset (c, c->s);
-            rc = handle_compressed (c, pkt);
-            _cdk_log_debug (" handle_compressed (%d)\n", rc);
-            break;
+          case CDK_PKT_COMPRESSED:
+              if (!c->s)
+                  c->s = inp;
+              rc = handle_compressed (c, pkt);
+              _cdk_log_debug (" handle_compressed (%d)\n", rc);
+              break;
 
-        case CDK_PKT_ONEPASS_SIG:
-            if (!c->s)
-                c->s = inp;
-            _cdk_log_debug (" handle_onepass_sig (0)\n");
-            c->sig.present = 1;
-            c->sig.one_pass = 1;
-            c->sig.pt_offset = cdk_stream_tell (c->s);
-            break;
+          case CDK_PKT_ONEPASS_SIG:
+              if (!c->s)
+                  c->s = inp;
+              _cdk_log_debug (" handle_onepass_sig (0)\n");
+              c->sig.present = 1;
+              c->sig.one_pass = 1;
+              c->sig.pt_offset = cdk_stream_tell (c->s);
+              break;
 
-        case CDK_PKT_PLAINTEXT:
-            /* skip rest of the packet */
-            if (!c->s)
-                c->s = inp;
-            if( !_cdk_stream_get_blockmode( c->s ) ) {
-                npos = cdk_stream_tell (c->s) + pkt->pkt.plaintext->len;
-                cdk_stream_seek (c->s, npos);
-            }
-            else
-                cdk_stream_seek( c->s, cdk_stream_get_length( c->s ) );
-            set_reset( c, c->s );
-            break;
+          case CDK_PKT_LITERAL:
+              /* skip rest of the packet */
+              if (!c->s)
+                  c->s = inp;
+              if( !_cdk_stream_get_blockmode( c->s ) ) {
+                  npos = cdk_stream_tell (c->s) + pkt->pkt.literal->len;
+                  cdk_stream_seek (c->s, npos);
+              }
+              else
+                  cdk_stream_seek( c->s, cdk_stream_get_length( c->s ) );
+              break;
           
-        case CDK_PKT_SIGNATURE:
-            if (!c->sig.present)
-                c->sig.present = 1;
-            break; /* handle it later */
+          case CDK_PKT_SIGNATURE:
+              if (!c->sig.present)
+                  c->sig.present = 1;
+              break; /* handle it later */
 
-        case CDK_PKT_MDC:
-            _cdk_log_debug( "MDC packet detected.\n" );
-            break;
+          case CDK_PKT_MDC:
+              _cdk_log_debug( "MDC packet detected.\n" );
+              break;
 
-        case CDK_PKT_MARKER:
-            _cdk_log_debug( "marker packet detected.\n" );
-            break;
+          case CDK_PKT_MARKER:
+              _cdk_log_debug( "marker packet detected.\n" );
+              break;
 
-        default:
-            rc = CDK_Inv_Packet;
-            break; 
+          default:
+              rc = CDK_Inv_Packet;
+              break; 
         }
         if (rc)
             break;
@@ -784,24 +826,24 @@ do_proc_packets( CDK_HD hd, CTX c, CDK_STREAM inp, CDK_STREAM * ret_out )
     for( node = c->node; !rc && node; node = node->next ) {
         pkt = node->pkt;
         switch (pkt->pkttype) {
-        case CDK_PKT_ONEPASS_SIG:
-            rc = handle_onepass_sig (c, pkt);
-            _cdk_log_debug (" _handle_onepass_sig (%d)\n", rc);
-            break;
+          case CDK_PKT_ONEPASS_SIG:
+              rc = handle_onepass_sig (c, pkt);
+              _cdk_log_debug (" _handle_onepass_sig (%d)\n", rc);
+              break;
           
-        case CDK_PKT_PLAINTEXT:
-            rc = handle_plaintext (c, pkt, ret_out);
-            _cdk_log_debug (" _handle_plaintext (%d)\n", rc);
-            break;
+          case CDK_PKT_LITERAL:
+              rc = handle_literal (c, pkt, ret_out);
+              _cdk_log_debug (" _handle_literal (%d)\n", rc);
+              break;
 
-        case CDK_PKT_SIGNATURE:
-            rc = handle_signature (hd, c, pkt);
-            _cdk_log_debug (" _handle_signature (%d)\n", rc);
-            break;
+          case CDK_PKT_SIGNATURE:
+              rc = handle_signature (hd, c, pkt);
+              _cdk_log_debug (" _handle_signature (%d)\n", rc);
+              break;
 
-        default:
-            _cdk_log_debug ("skip packet type %d\n", pkt->pkttype);
-            break;
+          default:
+              _cdk_log_debug ("skip packet type %d\n", pkt->pkttype);
+              break;
         }
         if (rc)
             break;
@@ -813,11 +855,11 @@ do_proc_packets( CDK_HD hd, CTX c, CDK_STREAM inp, CDK_STREAM * ret_out )
 
 
 int
-_cdk_proc_packets( CDK_HD hd, CDK_STREAM inp,
-                   const char * output, CDK_STREAM outstream,
-                   GCRY_MD_HD md )
+_cdk_proc_packets( cdk_ctx_t hd, cdk_stream_t inp,
+                   const char * output, cdk_stream_t outstream,
+                   cdk_md_hd_t md )
 {
-    CDK_STREAM out = NULL;
+    cdk_stream_t out = NULL;
     CTX c;
     int rc;
 
@@ -844,33 +886,33 @@ _cdk_proc_packets( CDK_HD hd, CDK_STREAM inp,
 
 
 static int
-check_pubkey_enc_list( CDK_STREAM in, CDK_KEYDB_HD hd )
+check_pubkey_enc_list( cdk_stream_t in, cdk_keydb_hd_t hd )
 {
-    CDK_PACKET pkt;
+    cdk_packet_t pkt;
     int n = 0, nenc = 0;
     
     if( !in || !hd )
         return CDK_Inv_Value;
 
-    if( cdk_armor_filter_use( in ) ) {
+    if( cdk_armor_filter_use( in ) )
         cdk_stream_set_armor_flag( in, 0 );
-        cdk_stream_control( in, CDK_STREAMCTL_RESET, 1 );
-    }
-    cdk_pkt_init( &pkt );
-    while( !cdk_pkt_parse( in, &pkt ) ) {
-        if( pkt.pkttype != CDK_PKT_PUBKEY_ENC ) {
-            if( CDK_PKT_IS_ENCRYPTED( pkt.pkttype ) )
-                cdk_free( pkt.pkt.encrypted );
+    pkt = cdk_calloc( 1, sizeof * pkt );
+    cdk_pkt_init( pkt );
+    while( !cdk_pkt_read( in, pkt ) ) {
+        if( pkt->pkttype != CDK_PKT_PUBKEY_ENC ) {
+            if( CDK_PKT_IS_ENCRYPTED( pkt->pkttype ) )
+                cdk_free( pkt->pkt.encrypted );
             else
-                cdk_pkt_free( &pkt );
+                cdk_pkt_free( pkt );
             break;
         }
         nenc++;
-        if( !cdk_keydb_check_sk( hd, pkt.pkt.pubkey_enc->keyid ) )
+        if( !cdk_keydb_check_sk( hd, pkt->pkt.pubkey_enc->keyid ) )
             n++;
-        cdk_pkt_free( &pkt );
-        cdk_pkt_init( &pkt );
+        cdk_pkt_free( pkt );
+        cdk_pkt_init( pkt );
     }
+    cdk_free( pkt );
     cdk_stream_seek( in, 0 );
     if( !nenc )
         return 0;
@@ -888,10 +930,10 @@ check_pubkey_enc_list( CDK_STREAM in, CDK_KEYDB_HD hd )
  * When the operation was successfull, hd can contain information about
  * the signature (when present) and more.
  **/
-int
-cdk_file_decrypt( CDK_HD hd, const char * file, const char * output )
+cdk_error_t
+cdk_file_decrypt( cdk_ctx_t hd, const char * file, const char * output )
 {
-    CDK_STREAM inp = NULL;
+    cdk_stream_t inp = NULL;
     int rc = 0;
 
     if( !file )
@@ -911,8 +953,8 @@ cdk_file_decrypt( CDK_HD hd, const char * file, const char * output )
 }
 
 
-int
-cdk_stream_decrypt( CDK_HD hd, CDK_STREAM inp, CDK_STREAM out )
+cdk_error_t
+cdk_stream_decrypt( cdk_ctx_t hd, cdk_stream_t inp, cdk_stream_t out )
 {
     int rc;
 
@@ -939,16 +981,16 @@ cdk_stream_decrypt( CDK_HD hd, CDK_STREAM inp, CDK_STREAM out )
  * exactly like the cdk_file_xxx functions with the exception that it can
  * be used with memory and not with streams or files.
  **/
-int
-cdk_data_transform( CDK_HD hd, int mode,
-                    CDK_STRLIST locusr, CDK_STRLIST remusr,
+cdk_error_t
+cdk_data_transform( cdk_ctx_t hd, enum cdk_crypto_mode_t mode,
+                    cdk_strlist_t locusr, cdk_strlist_t remusr,
                     const void * inbuf, size_t insize,
                     byte ** outbuf, size_t * outsize,
                     int modval )
 {
-    CDK_STREAM inp, out;
-    CDK_KEYDB_HD db;
-    CDK_KBNODE knode = NULL;
+    cdk_stream_t inp, out;
+    cdk_keydb_hd_t db;
+    cdk_kbnode_t knode = NULL;
     int rc, res[4];
     
     if( !hd )
@@ -973,47 +1015,47 @@ cdk_data_transform( CDK_HD hd, int mode,
     cdk_stream_tmp_set_mode( out, 1 );
         
     switch( mode ) {
-    case CDK_CRYPTYPE_ENCRYPT:
-        rc = cdk_stream_encrypt( hd, remusr, inp, out );
-        break;
+      case CDK_CRYPTYPE_ENCRYPT:
+          rc = cdk_stream_encrypt( hd, remusr, inp, out );
+          break;
 
-    case CDK_CRYPTYPE_DECRYPT:
-        rc = cdk_stream_decrypt( hd, inp, out );
-        break;
+      case CDK_CRYPTYPE_DECRYPT:
+          rc = cdk_stream_decrypt( hd, inp, out );
+          break;
 
-    case CDK_CRYPTYPE_SIGN:
-        rc = cdk_stream_sign( hd, inp, out, locusr, remusr, 0, modval );
-        break;
+      case CDK_CRYPTYPE_SIGN:
+          rc = cdk_stream_sign( hd, inp, out, locusr, remusr, 0, modval );
+          break;
 
-    case CDK_CRYPTYPE_VERIFY:
-        rc = cdk_stream_verify( hd, inp, out );
-        break;
+      case CDK_CRYPTYPE_VERIFY:
+          rc = cdk_stream_verify( hd, inp, out );
+          break;
 
-    case CDK_CRYPTYPE_EXPORT:
-        if( cdk_handle_control( hd, CDK_CTLF_GET, CDK_CTL_ARMOR ) )
-            cdk_stream_set_armor_flag( out, CDK_ARMOR_PUBKEY );
-        db = cdk_handle_get_keydb( hd, CDK_DBTYPE_PK_KEYRING );
-        rc = cdk_keydb_export( db, out, remusr );
-        break;
+      case CDK_CRYPTYPE_EXPORT:
+          if( cdk_handle_control( hd, CDK_CTLF_GET, CDK_CTL_ARMOR ) )
+              cdk_stream_set_armor_flag( out, CDK_ARMOR_PUBKEY );
+          db = cdk_handle_get_keydb( hd, CDK_DBTYPE_PK_KEYRING );
+          rc = cdk_keydb_export( db, out, remusr );
+          break;
 
-    case CDK_CRYPTYPE_IMPORT:
-        if( cdk_armor_filter_use( inp ) )
-            cdk_stream_set_armor_flag( inp, 0 );
-        rc = cdk_keydb_get_keyblock( inp, &knode );
-        if( knode ) {
-            db = cdk_handle_get_keydb( hd, CDK_DBTYPE_PK_KEYRING );
-            rc = cdk_keydb_import( db, knode, res );
-            if( !rc ) {
-                *outbuf = _cdk_keydb_get_importres_as_xml( res );
-                *outsize = strlen( *outbuf );
-            }
-            cdk_kbnode_release( knode );
-        }
-        break;
+      case CDK_CRYPTYPE_IMPORT:
+          if( cdk_armor_filter_use( inp ) )
+              cdk_stream_set_armor_flag( inp, 0 );
+          rc = cdk_keydb_get_keyblock( inp, &knode );
+          if( knode ) {
+              db = cdk_handle_get_keydb( hd, CDK_DBTYPE_PK_KEYRING );
+              rc = cdk_keydb_import( db, knode, res );
+              if( !rc ) {
+                  *outbuf = NULL; /* FIXME */
+                  *outsize = strlen( *outbuf );
+              }
+              cdk_kbnode_release( knode );
+          }
+          break;
 
-    default:
-        rc = CDK_Inv_Mode;
-        break;
+      default:
+          rc = CDK_Inv_Mode;
+          break;
     }
 
     cdk_stream_close( inp );
@@ -1022,7 +1064,7 @@ cdk_data_transform( CDK_HD hd, int mode,
         rc = cdk_stream_mmap( out, outbuf, outsize );
     }
     else if( !rc && mode == CDK_CRYPTYPE_VERIFY ) {
-        *outbuf = cdk_sig_get_info_as_xml( hd, modval );
+        *outbuf = NULL; /* FIXME */
         *outsize = *outbuf? strlen( *outbuf ) : 0;
     }
     cdk_stream_close( out );
