@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with OpenCDK; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+ * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #ifndef CDK_FILTERS_H
@@ -37,6 +37,7 @@ typedef struct {
     struct {
         int on;
         size_t size;
+        size_t nleft;
     } blkmode;
     cdk_stream_t s;
 } cipher_filter_t;
@@ -83,15 +84,17 @@ typedef struct {
 int _cdk_filter_armor( void * opaque, int ctl, FILE * in, FILE * out );
 
 /*-- cipher.c --*/
-int _cdk_filter_hash( void * opaque, int ctl, FILE * in, FILE * out );
-int _cdk_filter_cipher( void * opaque, int ctl, FILE * in, FILE * out );
+cdk_error_t _cdk_filter_hash( void * opaque, int ctl, FILE * in, FILE * out );
+cdk_error_t _cdk_filter_cipher( void * opaque, int ctl,
+                                FILE * in, FILE * out );
 
 /*-- plaintext.c --*/
 int _cdk_filter_literal( void * opaque, int ctl, FILE * in, FILE * out );
 int _cdk_filter_text( void * opaque, int ctl, FILE * in, FILE * out );
 
 /*-- compress.c --*/
-int _cdk_filter_compress( void * opaque, int ctl, FILE * in, FILE * out );
+cdk_error_t _cdk_filter_compress( void * opaque, int ctl,
+                                  FILE * in, FILE * out );
 
 #endif /* CDK_FILTERS_H */
 
