@@ -1,5 +1,4 @@
-/* -*- Mode: C; c-file-style: "bsd" -*-
- * keydb.c - Key database routines
+/* keydb.c - Key database routines
  *        Copyright (C) 2002, 2003 Timo Schulz
  *
  * This file is part of OpenCDK.
@@ -44,8 +43,9 @@ typedef struct key_idx_s * key_idx_t;
 
 static void keydb_cache_free( key_table_t cache );
 static int keydb_search_copy( cdk_dbsearch_t * r_dst, cdk_dbsearch_t src );
+static void keydb_search_free( cdk_dbsearch_t dbs );
 static int classify_data( const byte * buf, size_t len );
-void keydb_search_free( cdk_dbsearch_t dbs );
+static cdk_kbnode_t find_selfsig_node( cdk_kbnode_t key, cdk_pkt_pubkey_t pk );
 
      
 static char *
@@ -94,7 +94,7 @@ keydb_idx_build( const char * file )
         rc = CDK_Out_Of_Core;
         goto leave;
     }
-  
+    
     rc = cdk_stream_open( file, &inp );
     if( !rc )
         rc = cdk_stream_create( fname, &out );
@@ -146,6 +146,7 @@ cdk_keydb_idx_rebuild( cdk_keydb_hd_t hd )
         return 0;
   
     cdk_stream_close( hd->idx );
+    hd->idx = NULL;
     if( !hd->idx_name ) {
         hd->idx_name = keydb_idx_mkname( hd->name );
         if( !hd->idx_name )
@@ -247,26 +248,26 @@ cdk_keydb_new( cdk_keydb_hd_t * r_hd, int type, void * data, size_t count )
         return CDK_Out_Of_Core;
   
     switch( type ) {
-    case CDK_DBTYPE_PK_KEYRING:
-    case CDK_DBTYPE_SK_KEYRING:
-        hd->name = cdk_strdup( data );
-        if( !hd->name ) {
-            cdk_free( hd );
-            return CDK_Out_Of_Core;
-        }
-        break;
+      case CDK_DBTYPE_PK_KEYRING:
+      case CDK_DBTYPE_SK_KEYRING:
+          hd->name = cdk_strdup( data );
+          if( !hd->name ) {
+              cdk_free( hd );
+              return CDK_Out_Of_Core;
+          }
+          break;
       
-    case CDK_DBTYPE_DATA:
-        hd->buf = cdk_stream_tmp_from_mem( data, count );
-        if( !hd->buf ) {
-            cdk_free( hd );
-            return CDK_Out_Of_Core;
-        }
-        break;
+      case CDK_DBTYPE_DATA:
+          hd->buf = cdk_stream_tmp_from_mem( data, count );
+          if( !hd->buf ) {
+              cdk_free( hd );
+              return CDK_Out_Of_Core;
+          }
+          break;
 
-    default:
-        cdk_free( hd );
-        return CDK_Inv_Mode;
+      default:
+          cdk_free( hd );
+          return CDK_Inv_Mode;
     }
     hd->type = type;
     if( type == CDK_DBTYPE_SK_KEYRING )
@@ -393,22 +394,22 @@ find_by_keyid( cdk_kbnode_t knode, cdk_dbsearch_t ks )
             || node->pkt->pkttype == CDK_PKT_SECRET_SUBKEY ) {
             _cdk_pkt_get_keyid( node->pkt, keyid );
             switch( ks->type ) {
-            case CDK_DBSEARCH_SHORT_KEYID:
-                if( keyid[1] == ks->u.keyid[1] ) {
-                    found = 1;
-                    break;
-                }
-                break;
+              case CDK_DBSEARCH_SHORT_KEYID:
+                  if( keyid[1] == ks->u.keyid[1] ) {
+                      found = 1;
+                      break;
+                  }
+                  break;
 
-            case CDK_DBSEARCH_KEYID:
-                if( KEYID_CMP( keyid, ks->u.keyid ) ) {
-                    found = 1;
-                    break;
-                }
-                break;
+              case CDK_DBSEARCH_KEYID:
+                  if( KEYID_CMP( keyid, ks->u.keyid ) ) {
+                      found = 1;
+                      break;
+                  }
+                  break;
 
-            default: /* invalid mode */
-                return 0;
+              default: /* invalid mode */
+                  return 0;
             }
         }
     }
@@ -456,34 +457,34 @@ find_by_pattern( cdk_kbnode_t knode, cdk_dbsearch_t ks )
         uidlen = node->pkt->pkt.user_id->len;
         name = node->pkt->pkt.user_id->name;
         switch( ks->type ) {
-        case CDK_DBSEARCH_EXACT:
-            if( name && (strlen( ks->u.pattern ) == uidlen
-                         && !strncmp( ks->u.pattern, name, uidlen ) ) ) {
-                found = 1;
-                break;
-            }
-            break;
+          case CDK_DBSEARCH_EXACT:
+              if( name && (strlen( ks->u.pattern ) == uidlen
+                           && !strncmp( ks->u.pattern, name, uidlen ) ) ) {
+                  found = 1;
+                  break;
+              }
+              break;
 
-        case CDK_DBSEARCH_SUBSTR:
-            if( uidlen > 65536 )
-                break;
-            if( name && strlen( ks->u.pattern ) > uidlen )
-                break;
-            if( name && _cdk_memistr( name, uidlen, ks->u.pattern ) ) {
-                found = 1;
-                break;
-            }
-            break;
+          case CDK_DBSEARCH_SUBSTR:
+              if( uidlen > 65536 )
+                  break;
+              if( name && strlen( ks->u.pattern ) > uidlen )
+                  break;
+              if( name && _cdk_memistr( name, uidlen, ks->u.pattern ) ) {
+                  found = 1;
+                  break;
+              }
+              break;
 
-        default: /* invalid mode */
-            return 0;
+          default: /* invalid mode */
+              return 0;
         }
     }
     return found;
 }
 
 
-void
+static void
 keydb_search_free( cdk_dbsearch_t dbs )
 {
     if( !dbs )
@@ -519,27 +520,27 @@ keydb_cache_find( key_table_t cache, cdk_dbsearch_t desc )
         if( t->desc->type != desc->type )
             continue;
         switch( t->desc->type ) {
-        case CDK_DBSEARCH_SHORT_KEYID:
-        case CDK_DBSEARCH_KEYID:
-            if( KEYID_CMP( t->desc->u.keyid, desc->u.keyid ) )
-                return t;
-            break;
+          case CDK_DBSEARCH_SHORT_KEYID:
+          case CDK_DBSEARCH_KEYID:
+              if( KEYID_CMP( t->desc->u.keyid, desc->u.keyid ) )
+                  return t;
+              break;
 
-        case CDK_DBSEARCH_EXACT:
-            if( strlen( t->desc->u.pattern ) == strlen( desc->u.pattern )
-                && !strcmp( t->desc->u.pattern, desc->u.pattern ) )
-                return t;
-            break;
+          case CDK_DBSEARCH_EXACT:
+              if( strlen( t->desc->u.pattern ) == strlen( desc->u.pattern )
+                  && !strcmp( t->desc->u.pattern, desc->u.pattern ) )
+                  return t;
+              break;
 
-        case CDK_DBSEARCH_SUBSTR:
-            if( strstr( t->desc->u.pattern, desc->u.pattern ) )
-                return t;
-            break;
+          case CDK_DBSEARCH_SUBSTR:
+              if( strstr( t->desc->u.pattern, desc->u.pattern ) )
+                  return t;
+              break;
 
-        case CDK_DBSEARCH_FPR:
-            if( !memcmp( t->desc->u.fpr, desc->u.fpr, 20 ) )
-                return t;
-            break;
+          case CDK_DBSEARCH_FPR:
+              if( !memcmp( t->desc->u.fpr, desc->u.fpr, 20 ) )
+                  return t;
+              break;
         }
     }
     return NULL;
@@ -547,7 +548,7 @@ keydb_cache_find( key_table_t cache, cdk_dbsearch_t desc )
   
 
 static int
-keydb_cache_add (cdk_keydb_hd_t hd, cdk_dbsearch_t dbs, u32 offset)
+keydb_cache_add( cdk_keydb_hd_t hd, cdk_dbsearch_t dbs, u32 offset )
 {
     key_table_t k;
 
@@ -569,11 +570,11 @@ keydb_cache_add (cdk_keydb_hd_t hd, cdk_dbsearch_t dbs, u32 offset)
 
       
 static int
-keydb_search_copy (cdk_dbsearch_t * r_dst, cdk_dbsearch_t src)
+keydb_search_copy( cdk_dbsearch_t * r_dst, cdk_dbsearch_t src )
 {
     cdk_dbsearch_t dst;
   
-    if (!r_dst || !src)
+    if( !r_dst || !src )
         return CDK_Inv_Value;
     
     dst = cdk_calloc( 1, sizeof *dst );
@@ -581,22 +582,22 @@ keydb_search_copy (cdk_dbsearch_t * r_dst, cdk_dbsearch_t src)
         return CDK_Out_Of_Core;
     dst->type = src->type;
     switch( src->type ) {
-    case CDK_DBSEARCH_EXACT:
-    case CDK_DBSEARCH_SUBSTR:
-        dst->u.pattern = cdk_strdup( src->u.pattern );
-        if( !dst->u.pattern )
-            return CDK_Out_Of_Core;
-        break;
+      case CDK_DBSEARCH_EXACT:
+      case CDK_DBSEARCH_SUBSTR:
+          dst->u.pattern = cdk_strdup( src->u.pattern );
+          if( !dst->u.pattern )
+              return CDK_Out_Of_Core;
+          break;
 
-    case CDK_DBSEARCH_SHORT_KEYID:
-    case CDK_DBSEARCH_KEYID:
-        dst->u.keyid[0] = src->u.keyid[0];
-        dst->u.keyid[1] = src->u.keyid[1];
-        break;
+      case CDK_DBSEARCH_SHORT_KEYID:
+      case CDK_DBSEARCH_KEYID:
+          dst->u.keyid[0] = src->u.keyid[0];
+          dst->u.keyid[1] = src->u.keyid[1];
+          break;
 
-    case CDK_DBSEARCH_FPR:
-        memcpy( dst->u.fpr, src->u.fpr, 20 );
-        break;
+      case CDK_DBSEARCH_FPR:
+          memcpy( dst->u.fpr, src->u.fpr, 20 );
+          break;
     }
     *r_dst = dst;
     return 0;
@@ -629,86 +630,86 @@ cdk_keydb_search_start( cdk_keydb_hd_t db, int type, void * desc )
         return CDK_Out_Of_Core;
     dbs->type = type;
     switch( type ) {
-    case CDK_DBSEARCH_EXACT:
-    case CDK_DBSEARCH_SUBSTR:
-        cdk_free( dbs->u.pattern );
-        dbs->u.pattern = cdk_strdup( desc );
-        if( !dbs->u.pattern ) {
-            cdk_free( dbs );
-            return CDK_Out_Of_Core;
-        }
-        break;
+      case CDK_DBSEARCH_EXACT:
+      case CDK_DBSEARCH_SUBSTR:
+          cdk_free( dbs->u.pattern );
+          dbs->u.pattern = cdk_strdup( desc );
+          if( !dbs->u.pattern ) {
+              cdk_free( dbs );
+              return CDK_Out_Of_Core;
+          }
+          break;
 
-    case CDK_DBSEARCH_SHORT_KEYID:
-        keyid = desc;
-        dbs->u.keyid[1] = keyid[0];
-        break;
+      case CDK_DBSEARCH_SHORT_KEYID:
+          keyid = desc;
+          dbs->u.keyid[1] = keyid[0];
+          break;
       
-    case CDK_DBSEARCH_KEYID:
-        keyid = desc;
-        dbs->u.keyid[0] = keyid[0];
-        dbs->u.keyid[1] = keyid[1];
-        break;
+      case CDK_DBSEARCH_KEYID:
+          keyid = desc;
+          dbs->u.keyid[0] = keyid[0];
+          dbs->u.keyid[1] = keyid[1];
+          break;
 
-    case CDK_DBSEARCH_FPR:
-        memcpy( dbs->u.fpr, desc, 20 );
-        break;
+      case CDK_DBSEARCH_FPR:
+          memcpy( dbs->u.fpr, desc, 20 );
+          break;
 
-    case CDK_DBSEARCH_NEXT:
-        break;
+      case CDK_DBSEARCH_NEXT:
+          break;
 
-    case CDK_DBSEARCH_AUTO:
-        /* override the type with the actual db search type. */
-        dbs->type = classify_data( desc, strlen( desc ) );
-        switch( dbs->type ) {
-        case CDK_DBSEARCH_SUBSTR:
-        case CDK_DBSEARCH_EXACT:
-            cdk_free( dbs->u.pattern );
-            p = dbs->u.pattern = cdk_strdup( desc );
-            if( !p ) {
-                cdk_free( dbs );
-                return CDK_Out_Of_Core;
-            }
-            break;
+      case CDK_DBSEARCH_AUTO:
+          /* override the type with the actual db search type. */
+          dbs->type = classify_data( desc, strlen( desc ) );
+          switch( dbs->type ) {
+            case CDK_DBSEARCH_SUBSTR:
+            case CDK_DBSEARCH_EXACT:
+                cdk_free( dbs->u.pattern );
+                p = dbs->u.pattern = cdk_strdup( desc );
+                if( !p ) {
+                    cdk_free( dbs );
+                    return CDK_Out_Of_Core;
+                }
+                break;
 
-        case CDK_DBSEARCH_SHORT_KEYID:
-        case CDK_DBSEARCH_KEYID:
-            p = desc;
-            if( !strncmp( p, "0x", 2 ) )
-                p += 2;
-            if( strlen( p ) == 8 ) {
-                dbs->u.keyid[0] = 0;
-                dbs->u.keyid[1] = strtoul( p, NULL, 16 );
-            }
-            else if( strlen( p ) == 16 ) {
-                dbs->u.keyid[0] = strtoul( p    , NULL, 16 );
-                dbs->u.keyid[1] = strtoul( p + 8, NULL, 16 );
-            }
-            else { /* should never happen */
-                cdk_free( dbs );
-                return CDK_Inv_Mode;
-            }
-            break;
+            case CDK_DBSEARCH_SHORT_KEYID:
+            case CDK_DBSEARCH_KEYID:
+                p = desc;
+                if( !strncmp( p, "0x", 2 ) )
+                    p += 2;
+                if( strlen( p ) == 8 ) {
+                    dbs->u.keyid[0] = 0;
+                    dbs->u.keyid[1] = strtoul( p, NULL, 16 );
+                }
+                else if( strlen( p ) == 16 ) {
+                    dbs->u.keyid[0] = strtoul( p    , NULL, 16 );
+                    dbs->u.keyid[1] = strtoul( p + 8, NULL, 16 );
+                }
+                else { /* should never happen */
+                    cdk_free( dbs );
+                    return CDK_Inv_Mode;
+                }
+                break;
 
-        case CDK_DBSEARCH_FPR:
-            p = desc;
-            if( strlen( p ) != 40 ) {
-                cdk_free( dbs );
-                return CDK_Inv_Mode;
-            }
-            for( i = 0; i < 20; i++ ) {
-                tmp[0] = p[2*i];
-                tmp[1] = p[2*i+1];
-                tmp[2] = 0x00;
-                dbs->u.fpr[i] = strtoul( tmp, NULL, 16 );
-            }
-            break;
-        }
-        break;
+            case CDK_DBSEARCH_FPR:
+                p = desc;
+                if( strlen( p ) != 40 ) {
+                    cdk_free( dbs );
+                    return CDK_Inv_Mode;
+                }
+                for( i = 0; i < 20; i++ ) {
+                    tmp[0] = p[2*i];
+                    tmp[1] = p[2*i+1];
+                    tmp[2] = 0x00;
+                    dbs->u.fpr[i] = strtoul( tmp, NULL, 16 );
+                }
+                break;
+          }
+          break;
 
-    default:
-        cdk_free( dbs );
-        return CDK_Inv_Mode;
+      default:
+          cdk_free( dbs );
+          return CDK_Inv_Mode;
     }
     keydb_search_free( db->dbs );
     db->dbs = dbs;
@@ -793,32 +794,30 @@ cdk_keydb_search( cdk_keydb_hd_t hd, cdk_kbnode_t * ret_key )
         pos = cdk_stream_tell( kr );
         rc = cdk_keydb_get_keyblock( kr, &knode );
         if( rc ) {
-            if( rc == CDK_EOF && knode )
-                rc = 0;
-            if( !knode && rc == CDK_EOF )
-                rc = CDK_Error_No_Key;
+            if( rc == CDK_EOF )
+                rc = knode? 0 : CDK_Error_No_Key;
             if( rc )
                 break;
         }
 
         switch( ks->type ) {
-        case CDK_DBSEARCH_SHORT_KEYID:
-        case CDK_DBSEARCH_KEYID:
-            key_found = find_by_keyid( knode, ks );
-            break;
+          case CDK_DBSEARCH_SHORT_KEYID:
+          case CDK_DBSEARCH_KEYID:
+              key_found = find_by_keyid( knode, ks );
+              break;
 
-        case CDK_DBSEARCH_FPR:
-            key_found = find_by_fpr( knode, ks );
-            break;
+          case CDK_DBSEARCH_FPR:
+              key_found = find_by_fpr( knode, ks );
+              break;
           
-	case CDK_DBSEARCH_EXACT:
-	case CDK_DBSEARCH_SUBSTR:
-            key_found = find_by_pattern( knode, ks );
-            break;
+          case CDK_DBSEARCH_EXACT:
+          case CDK_DBSEARCH_SUBSTR:
+              key_found = find_by_pattern( knode, ks );
+              break;
 
-        case CDK_DBSEARCH_NEXT:
-            key_found = knode? 1 : 0;
-            break;
+          case CDK_DBSEARCH_NEXT:
+              key_found = knode? 1 : 0;
+              break;
 	}
 
         if( key_found ) {
@@ -840,7 +839,7 @@ cdk_keydb_search( cdk_keydb_hd_t hd, cdk_kbnode_t * ret_key )
 cdk_error_t
 cdk_keydb_get_bykeyid( cdk_keydb_hd_t hd, u32 * keyid, cdk_kbnode_t * ret_pk )
 {
-    int rc;
+    cdk_error_t rc;
 
     if( !hd || !keyid || !ret_pk )
         return CDK_Inv_Value;
@@ -855,7 +854,7 @@ cdk_keydb_get_bykeyid( cdk_keydb_hd_t hd, u32 * keyid, cdk_kbnode_t * ret_pk )
 cdk_error_t
 cdk_keydb_get_byfpr( cdk_keydb_hd_t hd, const byte * fpr, cdk_kbnode_t * r_pk )
 {
-    int rc;
+    cdk_error_t rc;
 
     if( !hd || !fpr || !r_pk )
         return CDK_Inv_Value;
@@ -871,7 +870,7 @@ cdk_error_t
 cdk_keydb_get_bypattern( cdk_keydb_hd_t hd, const char * patt,
 			 cdk_kbnode_t * ret_pk )
 {
-    int rc;
+    cdk_error_t rc;
 
     if( !hd || !patt || !ret_pk )
         return CDK_Inv_Value;
@@ -909,8 +908,9 @@ keydb_check_key( cdk_packet_t pkt )
 static cdk_kbnode_t
 keydb_find_byusage( cdk_kbnode_t root, int req_usage, int is_pk )
 {
-    cdk_kbnode_t node;
+    cdk_kbnode_t node, n, keys = NULL;
     int pkttype = 0, req_type = 0;
+    long timestamp = 0;
 
     req_type = is_pk? CDK_PKT_PUBLIC_KEY : CDK_PKT_SECRET_KEY;
     if( !req_usage )
@@ -920,22 +920,50 @@ keydb_find_byusage( cdk_kbnode_t root, int req_usage, int is_pk )
     if( node && !keydb_check_key( node->pkt ) )
         return NULL;
 
-    /* xxx: if there are more subkeys, use the one with the requested
-            usage and the newest timestamp. */
+    /* check all keys and only use non-revoked and non-expired keys.
+       then add all keys with the requested usage to a kbnode. */
     for( node = root; node; node = node->next ) {
         pkttype = node->pkt->pkttype;
         if( is_pk && (node->pkt->pkttype == CDK_PKT_PUBLIC_KEY
                       || node->pkt->pkttype == CDK_PKT_PUBLIC_SUBKEY)
             && keydb_check_key( node->pkt )
-            && (node->pkt->pkt.public_key->pubkey_usage & req_usage) )
-            return node;
+            && (node->pkt->pkt.public_key->pubkey_usage & req_usage) ) {
+            n = cdk_kbnode_new( node->pkt );
+            _cdk_kbnode_clone( n );
+            if( !keys )
+                keys = n;
+            else
+                _cdk_kbnode_add( keys, n );
+        }
         if( !is_pk && (node->pkt->pkttype == CDK_PKT_SECRET_KEY
                        || node->pkt->pkttype == CDK_PKT_SECRET_SUBKEY)
             && keydb_check_key( node->pkt )
-            && (node->pkt->pkt.secret_key->pk->pubkey_usage & req_usage) )
-            return node;
+            && (node->pkt->pkt.secret_key->pk->pubkey_usage & req_usage) ) {
+            n = cdk_kbnode_new( node->pkt );
+            _cdk_kbnode_clone( n );
+            if( !keys )
+                keys = n;
+            else
+                _cdk_kbnode_add( keys, n );
+        }
     }
-    return NULL;
+    if( !keys )
+        return NULL;
+    /* now walk through the key array, which only contain valid keys,
+       and return the one with the newest time stamp. */
+    for( n=NULL, node = keys; node; node = node->next ) {
+        if( is_pk && (node->pkt->pkttype == CDK_PKT_PUBLIC_KEY
+            || node->pkt->pkttype == CDK_PKT_PUBLIC_SUBKEY) ) {
+            if( node->pkt->pkt.public_key->timestamp > timestamp )
+                n = node;
+        }
+        else if( !is_pk && (node->pkt->pkttype == CDK_PKT_SECRET_KEY
+                 || node->pkt->pkttype == CDK_PKT_SECRET_SUBKEY) ) {
+            if( node->pkt->pkt.secret_key->pk->timestamp > timestamp )
+                n = node;
+        }
+    }
+    return n;
 }
 
 
@@ -960,6 +988,7 @@ _cdk_keydb_get_sk_byusage( cdk_keydb_hd_t hd, const char * name,
 {
     cdk_kbnode_t knode = NULL, node = NULL;
     cdk_pkt_seckey_t sk = NULL;
+    const char * s;
     int rc = 0;
 
     if( !ret_sk || !usage )
@@ -980,9 +1009,24 @@ _cdk_keydb_get_sk_byusage( cdk_keydb_hd_t hd, const char * name,
 
     sk = node->pkt->pkt.secret_key;
     _cdk_kbnode_clone( node );
+    for( node = knode; node; node = node->next ) {
+        if( node->pkt->pkttype == CDK_PKT_USER_ID ) {
+            s = node->pkt->pkt.user_id->name;
+            if( sk && !sk->pk->uid && _cdk_memistr( s, strlen( s ), name ) ) {
+                _cdk_copy_userid( &sk->pk->uid, node->pkt->pkt.user_id );
+                break;
+            }
+        }
+    }
+    if( sk->pk->uid ) {
+        node = find_selfsig_node( knode, sk->pk );
+        if( node )
+            _cdk_copy_signature( &sk->pk->uid->selfsig,
+                                 node->pkt->pkt.signature );
+    }
     cdk_kbnode_release( knode );
-
-leave:
+    
+ leave:
     *ret_sk = sk;
     return rc;
 }
@@ -990,7 +1034,7 @@ leave:
 
 int
 _cdk_keydb_get_pk_byusage( cdk_keydb_hd_t hd, const char * name,
-                           cdk_pkt_pubkey_t* ret_pk, int usage )
+                           cdk_pkt_pubkey_t * ret_pk, int usage )
 {
     cdk_kbnode_t knode, node = NULL;
     cdk_pkt_pubkey_t pk = NULL;
@@ -1023,6 +1067,11 @@ _cdk_keydb_get_pk_byusage( cdk_keydb_hd_t hd, const char * name,
 	    }
 	}
     }
+    if( pk->uid ) {
+        node = find_selfsig_node( knode, pk );
+        if( node )
+            _cdk_copy_signature( &pk->uid->selfsig, node->pkt->pkt.signature );
+    }
     cdk_kbnode_release( knode );
 
 leave:
@@ -1110,6 +1159,22 @@ is_selfsig( cdk_kbnode_t node, u32 * keyid )
     return 0;
 }
 
+
+static cdk_kbnode_t
+find_selfsig_node( cdk_kbnode_t key, cdk_pkt_pubkey_t pk )
+{
+    cdk_kbnode_t n;
+    u32 keyid[2];
+    
+    cdk_pk_get_keyid( pk, keyid );
+    for( n = key; n; n = n->next ) {
+        if( is_selfsig( n, keyid ) )
+            break;
+    }
+    return n;
+}
+
+
     
 static int
 keydb_merge_selfsig( cdk_kbnode_t key, u32 * keyid )
@@ -1459,9 +1524,9 @@ classify_data( const byte * buf, size_t len )
         }
         if( i == len ) {
             switch( len ) {
-            case 8: type = CDK_DBSEARCH_SHORT_KEYID; break;
-            case 16: type = CDK_DBSEARCH_KEYID; break;
-            case 40: type = CDK_DBSEARCH_FPR; break;
+              case  8: type = CDK_DBSEARCH_SHORT_KEYID; break;
+              case 16: type = CDK_DBSEARCH_KEYID; break;
+              case 40: type = CDK_DBSEARCH_FPR; break;
             }
         }
     }
@@ -1563,6 +1628,11 @@ cdk_keydb_import( cdk_keydb_hd_t hd, cdk_kbnode_t knode, int *result )
     for( node = knode; node; node = node->next ) {
         if( node->pkt->pkttype == CDK_PKT_RING_TRUST )
             continue; /* No uniformed syntax for this packet */
+        if( node->pkt->pkttype == CDK_PKT_SIGNATURE
+            && !node->pkt->pkt.signature->flags.exportable ) {
+            _cdk_log_debug( "found non-exportable signature, skipped\n" );
+            continue;
+        }
         rc = cdk_pkt_write( out, node->pkt );
         if( rc )
             break;
@@ -1576,6 +1646,36 @@ cdk_keydb_import( cdk_keydb_hd_t hd, cdk_kbnode_t knode, int *result )
 }
 
 
+int
+_cdk_keydb_check_userid( cdk_keydb_hd_t hd, u32 * keyid, const char *id )
+{
+    cdk_kbnode_t knode = NULL, unode = NULL;
+    int rc, check = 0;
+    
+    if( !hd )
+        return CDK_Inv_Value;
+
+    rc = cdk_keydb_search_start( hd, CDK_DBSEARCH_KEYID, keyid );
+    if( !rc )
+        rc = cdk_keydb_search( hd, &knode );
+    if( !rc )
+        rc = cdk_keydb_search_start( hd, CDK_DBSEARCH_EXACT, (char *)id );
+    if( !rc )
+        rc = cdk_keydb_search( hd, &unode );
+    if( !rc ) {
+        cdk_keydb_search_start( hd, CDK_DBSEARCH_KEYID, keyid );
+        if( unode && find_by_keyid( unode, hd->dbs ) )
+            check++;
+        cdk_keydb_search_start( hd, CDK_DBSEARCH_EXACT, (char *)id );
+        if( knode && find_by_pattern( knode, hd->dbs ) )
+            check++;
+    }   
+    cdk_kbnode_release( unode );
+    cdk_kbnode_release( knode );
+    return check==2 ? 1 : 0;
+}
+
+    
 int
 cdk_keydb_check_sk( cdk_keydb_hd_t hd, u32 * keyid )
 {
