@@ -1,6 +1,5 @@
-/* -*- Mode: C; c-file-style: "bsd" -*- */
-/* keyserver.c - Keyserver support
- *        Copyright (C) 2002 Timo Schulz
+/* keyserver.c - Simple keyserver support
+ *        Copyright (C) 2002, 2003 Timo Schulz
  *
  * This file is part of OpenCDK.
  *
@@ -48,25 +47,23 @@ init_sockets( void )
 {
 #ifdef __MINGW32__
     WSADATA wsdata;
-    int rc;
   
     if( initialized )
         return;
-    rc = WSAStartup( 0x0101, &wsdata );
-    if( rc )
+    if( WSAStartup( 0x0101, &wsdata ) )
         _cdk_log_debug( "winsock init failed.\n" );
-    initialized = 1;
 #endif
+    initialized = 1;
 }
 
   
 static int
-keyserver_hkp( const char * host, int port, u32 keyid, CDK_KBNODE * ret_key )
+keyserver_hkp( const char * host, int port, u32 keyid, cdk_kbnode_t * ret_key )
 {
     const char * fmt;
     struct hostent * hp;
     struct sockaddr_in saddr;
-    CDK_STREAM a;
+    cdk_stream_t a;
     char * buf, buffer[256];
     int nwritten, nread, state = 0;
     int rc = 0, fd;
@@ -120,7 +117,7 @@ keyserver_hkp( const char * host, int port, u32 keyid, CDK_KBNODE * ret_key )
         buffer[nread] = '\0';
         /*_cdk_log_debug( "%s", buffer );*/
         cdk_stream_write( a, buffer, nread );
-        if( strstr( buffer, "<pre>") || strstr( buffer, "</pre>" ) )
+        if( strstr( buffer, "<pre>" ) || strstr( buffer, "</pre>" ) )
             state++;
     }
     cdk_free( buf );
@@ -152,13 +149,14 @@ keyserver_hkp( const char * host, int port, u32 keyid, CDK_KBNODE * ret_key )
  *
  * Receive a key from a keyserver.
  **/
-int cdk_keyserver_recv_key( const char * host, int port,
-                            const byte * keyid, int kid_type,
-                            CDK_KBNODE * ret_key )
+cdk_error_t
+cdk_keyserver_recv_key( const char * host, int port,
+                        const byte * keyid, int kid_type,
+                        cdk_kbnode_t * ret_key )
 {
     u32 kid = 0;
 
-    if( !host || !keyid )
+    if( !host || !keyid || !ret_key )
         return CDK_Inv_Value;
     
     if( !initialized )
@@ -175,10 +173,10 @@ int cdk_keyserver_recv_key( const char * host, int port,
         host += 8;
 
     switch( kid_type ) {
-    case CDK_DBSEARCH_SHORT_KEYID: kid = _cdk_buftou32( keyid ); break;
-    case CDK_DBSEARCH_KEYID      : kid = _cdk_buftou32( keyid + 4 ); break;
-    case CDK_DBSEARCH_FPR        : kid = _cdk_buftou32( keyid + 16 ); break;
-    default                      : return CDK_Inv_Mode;
+      case CDK_DBSEARCH_SHORT_KEYID: kid = _cdk_buftou32( keyid ); break;
+      case CDK_DBSEARCH_KEYID      : kid = _cdk_buftou32( keyid + 4 ); break;
+      case CDK_DBSEARCH_FPR        : kid = _cdk_buftou32( keyid + 16 ); break;
+      default                      : return CDK_Inv_Mode;
     }
     
     return keyserver_hkp( host, port, kid, ret_key );
