@@ -1,6 +1,5 @@
-/* -*- Mode: C; c-file-style: "bsd" -*- */
 /* plaintext.c - Literal packet filters
- *       Copyright (C) 2002 Timo Schulz
+ *       Copyright (C) 2002, 2003 Timo Schulz
  *
  * This file is part of OpenCDK.
  *
@@ -30,37 +29,37 @@
 
 
 static int
-plaintext_decode( void * opaque, FILE * in, FILE * out )
+literal_decode( void * opaque, FILE * in, FILE * out )
 {
-    plaintext_filter_s * pfx = opaque;
-    CDK_STREAM si, so;
+    literal_filter_t * pfx = opaque;
+    cdk_stream_t si, so;
     CDK_PACKET pkt;
-    cdkPKT_plaintext * pt;
+    cdk_pkt_literal_t pt;
     byte buf[8192];
     size_t nread;
     int rc, bufsize;
 
-    _cdk_log_debug( "plaintext filter: decode\n" );
+    _cdk_log_debug( "literal filter: decode\n" );
   
     if (!pfx || !in || !out)
         return CDK_Inv_Value;
 
-    si = cdk_stream_fpopen( in, STREAMCTL_READ );
+    si = _cdk_stream_fpopen( in, STREAMCTL_READ );
     if (!si)
         return CDK_Out_Of_Core;
-    so = cdk_stream_fpopen( out, STREAMCTL_WRITE );
+    so = _cdk_stream_fpopen( out, STREAMCTL_WRITE );
     if( !so ) {
         cdk_stream_close( si );
         return CDK_Out_Of_Core;
     }
     cdk_pkt_init( &pkt );
-    rc = cdk_pkt_parse( si, &pkt );
-    if( pkt.pkttype != CDK_PKT_PLAINTEXT ) {
+    rc = cdk_pkt_read( si, &pkt );
+    if( pkt.pkttype != CDK_PKT_LITERAL ) {
         if( pkt.pkttype )
             cdk_pkt_free( &pkt );
         return rc;
     }
-    pt = pkt.pkt.plaintext;
+    pt = pkt.pkt.literal;
     pfx->mode = pt->mode;
     pfx->filename = cdk_strdup( pt->name? pt->name : " " );
     if( !pfx->filename ) {
@@ -68,8 +67,8 @@ plaintext_decode( void * opaque, FILE * in, FILE * out )
         return CDK_Out_Of_Core;
     }
     while( !feof( in ) ) {
-        _cdk_log_debug( "partial on=%d size=%lu\n",
-                        pfx->blkmode.on, pfx->blkmode.size );
+        /*_cdk_log_debug( "partial on=%d size=%lu\n",
+          pfx->blkmode.on, pfx->blkmode.size );*/
         if( pfx->blkmode.on )
             bufsize = pfx->blkmode.size;
         else
@@ -80,7 +79,7 @@ plaintext_decode( void * opaque, FILE * in, FILE * out )
             break;
         }
         if( pfx->md )
-            gcry_md_write (pfx->md, buf, nread);
+            cdk_md_write (pfx->md, buf, nread);
         cdk_stream_write( so, buf, nread );
         pt->len -= nread;
         if( pfx->blkmode.on ) {
@@ -99,31 +98,30 @@ plaintext_decode( void * opaque, FILE * in, FILE * out )
 
 
 static int
-plaintext_encode (void * opaque, FILE * in, FILE * out)
+literal_encode (void * opaque, FILE * in, FILE * out)
 {
-    plaintext_filter_s * pfx = opaque;
-    cdkPKT_plaintext * pt;
-    CDK_STREAM si, so;
+    literal_filter_t * pfx = opaque;
+    cdk_pkt_literal_t pt;
+    cdk_stream_t si;
     CDK_PACKET pkt;
     size_t filelen;
     int rc;
 
-    _cdk_log_debug ("plaintext filter: encode\n");
+    _cdk_log_debug ("literal filter: encode\n");
   
     if (!pfx || !in || !out)
         return CDK_Inv_Value;
   
-    if (!pfx->filename)
+    if (!pfx->filename) {
         pfx->filename = cdk_strdup ("_CONSOLE");
+        if( !pfx->filename )
+            return CDK_Out_Of_Core;
+    }
 
-    si = cdk_stream_fpopen (in, STREAMCTL_READ);
+    si = _cdk_stream_fpopen (in, STREAMCTL_READ);
     if (!si)
         return CDK_Out_Of_Core;
-    so = cdk_stream_fpopen (out, STREAMCTL_WRITE);
-    if (!so) {
-        cdk_stream_close (si);
-        return CDK_Out_Of_Core;
-    }
+
     filelen = strlen (pfx->filename);
     pt = cdk_calloc (1, sizeof *pt + filelen - 1);
     if (!pt)
@@ -137,28 +135,27 @@ plaintext_encode (void * opaque, FILE * in, FILE * out)
     pt->buf = si;
     cdk_pkt_init (&pkt);
     pkt.old_ctb = pfx->rfc1991? 1 : 0;
-    pkt.pkttype = CDK_PKT_PLAINTEXT;
-    pkt.pkt.plaintext = pt;
-    rc = cdk_pkt_build (so, &pkt);
+    pkt.pkttype = CDK_PKT_LITERAL;
+    pkt.pkt.literal = pt;
+    rc = _cdk_pkt_write_fp (out, &pkt);
 
     cdk_free (pt);
     cdk_stream_close (si);
-    cdk_stream_close (so);
-
     return rc;
 }
 
 
 int
-_cdk_filter_plaintext( void * opaque, int ctl, FILE * in, FILE * out )
+_cdk_filter_literal( void * opaque, int ctl, FILE * in, FILE * out )
 {
     if( ctl == STREAMCTL_READ )
-        return plaintext_decode( opaque, in, out );
+        return literal_decode( opaque, in, out );
     else if( ctl == STREAMCTL_WRITE )
-        return plaintext_encode( opaque, in, out );
+        return literal_encode( opaque, in, out );
     else if( ctl == STREAMCTL_FREE ) {
-        plaintext_filter_s * pfx = opaque;
+        literal_filter_t * pfx = opaque;
         if( pfx ) {
+            _cdk_log_debug( "free literal filter\n" );
             cdk_free( pfx->filename );
             pfx->filename = NULL;
         }
@@ -191,7 +188,7 @@ text_encode( void * opaque, FILE * in, FILE * out )
 static int
 text_decode( void * opaque, FILE * in, FILE * out )
 {
-    text_filter_s * tfx = opaque;
+    text_filter_t * tfx = opaque;
     const char * s;
     char buf[1024];
 
@@ -219,7 +216,7 @@ _cdk_filter_text( void * opaque, int ctl, FILE * in, FILE * out )
     else if( ctl == STREAMCTL_WRITE )
         return text_decode( opaque, in, out );
     else if( ctl == STREAMCTL_FREE ) {
-        text_filter_s * tfx = opaque;
+        text_filter_t * tfx = opaque;
         if( tfx ) {
             _cdk_log_debug( "free text filter\n" );
             tfx->lf = NULL;
