@@ -1,5 +1,4 @@
-/* -*- Mode: C; c-file-style: "bsd" -*-
- * pubkey.c - Public key API
+/* pubkey.c - Public key API
  *        Copyright (C) 2002, 2003 Timo Schulz
  *
  * This file is part of OpenCDK.
@@ -44,7 +43,7 @@ convert_to_gcrympi( cdk_mpi_t m[4], int ncount )
         return NULL;
     for( i = 0; i < ncount; i++ ) {
         nbytes = m[i]->bytes + 2;
-        if( gcry_mpi_scan( &d[i], GCRYMPI_FMT_PGP, m[i]->data, nbytes, &nbytes ) ) {
+        if( gcry_mpi_scan( &d[i], GCRYMPI_FMT_PGP, m[i]->data, &nbytes ) ) {
             rc = CDK_Gcry_Error;
             break;
 	}
@@ -174,7 +173,7 @@ digest_to_sexp( gcry_sexp_t * r_md, int algo, const byte * md, size_t mdlen )
     nbytes = mdlen ? mdlen : cdk_md_get_algo_dlen( algo );
     if( !nbytes )
         return CDK_Inv_Algo;
-    if( gcry_mpi_scan( &m, GCRYMPI_FMT_USG, md, nbytes, &nbytes ) )
+    if( gcry_mpi_scan( &m, GCRYMPI_FMT_USG, md, &nbytes ) )
         return CDK_Gcry_Error;
     if( gcry_sexp_build( &sexp, NULL, "%m", m ) )
         rc = CDK_Gcry_Error;
@@ -214,10 +213,10 @@ sexp_to_bitmpi( gcry_sexp_t sexp, const char * val, cdk_mpi_t * ret_buf )
     }
     buf->data[0] = nbits >> 8;
     buf->data[1] = nbits;
-    if( gcry_mpi_print( GCRYMPI_FMT_USG, NULL, nbytes, &nbytes, m ) )
+    if( gcry_mpi_print( GCRYMPI_FMT_USG, NULL, &nbytes, m ) )
         rc = CDK_Gcry_Error;
     else
-        if( gcry_mpi_print( GCRYMPI_FMT_USG, buf->data + 2, nbytes, &nbytes, m ) )
+        if( gcry_mpi_print( GCRYMPI_FMT_USG, buf->data + 2, &nbytes, m ) )
             rc = CDK_Gcry_Error;
     if( !rc ) {
         buf->bytes = nbytes;
@@ -528,47 +527,33 @@ cdk_pk_get_nbits( cdk_pkt_pubkey_t pk )
 int
 cdk_pk_get_npkey( int algo )
 {
-size_t bytes;
-
-    if (gcry_pk_algo_info( algo, GCRYCTL_GET_ALGO_NPKEY, NULL, &bytes))
-    	return 0;
-    
-    return bytes;
+    int n = gcry_pk_algo_info( algo, GCRYCTL_GET_ALGO_NPKEY, NULL, 0 );
+    return n > 0 ? n : 0;
 }
 
 
 int
 cdk_pk_get_nskey( int algo )
 {  
-size_t bytes;
-
-    if (gcry_pk_algo_info( algo, GCRYCTL_GET_ALGO_NSKEY, NULL, &bytes))
-    	return 0;
-    
-    bytes -= cdk_pk_get_npkey( algo );
-    return bytes;
+    int n = gcry_pk_algo_info( algo, GCRYCTL_GET_ALGO_NSKEY, NULL, 0 );
+    n -= cdk_pk_get_npkey( algo );
+    return n > 0 ? n : 0;
 }
 
 
 int
 cdk_pk_get_nsig( int algo )
 {
-size_t bytes;
-
-    if (gcry_pk_algo_info( algo, GCRYCTL_GET_ALGO_NSIGN, NULL, &bytes))
-    	return 0;
-    return bytes;
+    int n = gcry_pk_algo_info( algo, GCRYCTL_GET_ALGO_NSIGN, NULL, 0 );
+    return n > 0 ? n : 0;
 }
 
 
 int
 cdk_pk_get_nenc( int algo )
 {
-size_t bytes;
-
-    if (gcry_pk_algo_info( algo, GCRYCTL_GET_ALGO_NENCR, NULL, &bytes))
-    	return 0;
-    return bytes;
+    int n = gcry_pk_algo_info( algo, GCRYCTL_GET_ALGO_NENCR, NULL, 0 );
+    return n > 0 ? n : 0;
 }
 
 
@@ -578,12 +563,12 @@ _cdk_pk_algo_usage( int algo )
     int usage = 0;
 
     switch( algo ) {
-    case CDK_PK_RSA  : usage = PK_USAGE_SIGN | PK_USAGE_ENCR; break;
-    case CDK_PK_RSA_E: usage = PK_USAGE_ENCR; break;
-    case CDK_PK_RSA_S: usage = PK_USAGE_SIGN; break;
-    case CDK_PK_ELG  : usage = PK_USAGE_SIGN | PK_USAGE_ENCR; break;
-    case CDK_PK_ELG_E: usage = PK_USAGE_ENCR; break;
-    case CDK_PK_DSA  : usage = PK_USAGE_SIGN; break; 
+      case CDK_PK_RSA  : usage = PK_USAGE_SIGN | PK_USAGE_ENCR; break;
+      case CDK_PK_RSA_E: usage = PK_USAGE_ENCR; break;
+      case CDK_PK_RSA_S: usage = PK_USAGE_SIGN; break;
+      case CDK_PK_ELG  : usage = PK_USAGE_SIGN | PK_USAGE_ENCR; break;
+      case CDK_PK_ELG_E: usage = PK_USAGE_ENCR; break;
+      case CDK_PK_DSA  : usage = PK_USAGE_SIGN; break; 
     }
     return usage;  
 }
@@ -595,7 +580,7 @@ _cdk_pk_test_algo( int algo, unsigned int usage_flags )
     size_t n = usage_flags;
   
     if( algo < 0 || algo > 110 )
-        return GPG_ERR_INV_PARAMETER;
+        return GCRYERR_INV_PK_ALGO;
     return gcry_pk_algo_info( algo, GCRYCTL_TEST_ALGO, NULL, &n );    
 }
 
@@ -675,12 +660,12 @@ cdk_sk_unprotect( cdk_pkt_seckey_t sk, const char * pw )
                                       sk->protect.s2k, 0, pw );
         if( rc )
             return rc;
-        hd = cdk_cipher_open( sk->protect.algo, 1,
+        rc = cdk_cipher_open( &hd, sk->protect.algo, 1,
                               dek->key, dek->keylen,
                               sk->protect.iv, sk->protect.ivlen );
-        if( !hd ) {
+        if( rc ) {
             cdk_free( dek );
-            return CDK_Inv_Algo;
+            return rc;
         }
         wipemem( dek, sizeof dek );
         cdk_dek_free( dek );
@@ -702,9 +687,10 @@ cdk_sk_unprotect( cdk_pkt_seckey_t sk, const char * pw )
                     return CDK_Inv_Packet;
                 }
                 else {
-                    cdk_md_hd_t md = cdk_md_open( CDK_MD_SHA1, 1 );
-                    if( !md )
-                        return CDK_Gcry_Error;
+                    cdk_md_hd_t md;
+                    rc = cdk_md_open( &md, CDK_MD_SHA1, 1 );
+                    if( rc )
+                        return rc;
                     cdk_md_write( md, data, ndata - dlen );
                     cdk_md_final( md );
                     if( !memcmp( cdk_md_read( md, CDK_MD_SHA1 ),
@@ -762,23 +748,28 @@ cdk_sk_unprotect( cdk_pkt_seckey_t sk, const char * pw )
 cdk_error_t
 cdk_sk_protect( cdk_pkt_seckey_t sk, const char * pass )
 {
-    cdk_cipher_hd_t hd;
-    cdk_md_hd_t md;
+    cdk_cipher_hd_t hd = NULL;
+    cdk_md_hd_t md = NULL;
     cdk_mpi_t a;
-    cdk_dek_t dek;
+    cdk_dek_t dek = NULL;
     cdk_s2k_t s2k;
-    byte * p;
+    byte * p = NULL;
     size_t enclen = 0, nskey, i;
     int rc;
-
-    rc = cdk_s2k_new( &s2k, 3, CDK_MD_SHA1, NULL );
-    if( rc )
-        return rc;
-    rc = cdk_dek_from_passphrase( &dek, CDK_CIPHER_3DES, s2k, 2, pass );
-    if( rc )
-        return rc;
     
     nskey = cdk_pk_get_nskey( sk->pubkey_algo  );
+    if( !nskey )
+        return CDK_Inv_Algo;
+    
+    _cdk_random_initialize( );
+    rc = cdk_s2k_new( &s2k, CDK_S2K_ITERSALTED, CDK_MD_SHA1, NULL );
+    if( !rc )
+        rc = cdk_dek_from_passphrase( &dek, CDK_CIPHER_CAST5, s2k, 2, pass );
+    if( rc ) {
+        cdk_s2k_free( s2k );
+        return rc;
+    }
+    
     for( i = 0; i < nskey; i++ ) {
         enclen += 2;
         enclen += sk->mpi[i]->bytes;
@@ -797,23 +788,18 @@ cdk_sk_protect( cdk_pkt_seckey_t sk, const char * pass )
     enclen += 20;
     sk->enclen = enclen;
     sk->protect.s2k = s2k;
-    sk->protect.algo = CDK_CIPHER_3DES;
+    sk->protect.algo = CDK_CIPHER_CAST5;
     sk->protect.ivlen = cdk_cipher_get_algo_blklen( sk->protect.algo  );
-    gcry_randomize( sk->protect.iv, sk->protect.ivlen, GCRY_STRONG_RANDOM  );
-    hd = cdk_cipher_open( sk->protect.algo, 1,
-                          dek->key, dek->keylen,
+    cdk_randomize( sk->protect.iv, sk->protect.ivlen, CDK_RANDOM_STRONG );
+    rc = cdk_cipher_open( &hd, sk->protect.algo, 1, dek->key, dek->keylen,
                           sk->protect.iv, sk->protect.ivlen );
-    if( !hd ) {
-        cdk_free( p );
-        return CDK_Gcry_Error;
-    }
+    if( rc )
+        goto leave;
+
+    rc = cdk_md_open( &md, CDK_MD_SHA1, 1 );
+    if( rc )
+        goto leave;
 
-    md = cdk_md_open( CDK_MD_SHA1, GCRY_CIPHER_SECURE  );
-    if( !md ) {
-        cdk_cipher_close( hd  );
-        cdk_free( p  );
-        return CDK_Gcry_Error;
-    }
     sk->protect.sha1chk = 1;
     sk->is_protected = 1;
     sk->csum = 0;
@@ -822,6 +808,10 @@ cdk_sk_protect( cdk_pkt_seckey_t sk, const char * pass )
     memcpy( p + enclen - 20, cdk_md_read( md, 0 ), 20 );
     cdk_md_close( md );
     rc = cdk_cipher_encrypt( hd, p, p, enclen );
+
+leave:
+    cdk_free( p );
+    cdk_md_close( md );
     cdk_cipher_close( hd );
     cdk_dek_free( dek );
     return rc;
@@ -862,6 +852,7 @@ cdk_pk_revoke_create( cdk_pkt_seckey_t sk, int code, const char * inf,
     cdk_pkt_signature_t sig;
     char * p = NULL, * dat;
     int n;
+    int rc;
     
     if( !sk || !ret_revcert )
         return CDK_Inv_Value;
@@ -894,10 +885,10 @@ cdk_pk_revoke_create( cdk_pkt_seckey_t sk, int code, const char * inf,
     }
     cdk_free( dat );
 
-    md = cdk_md_open( CDK_MD_SHA1, 0 );
-    if( !md ) {
+    rc = cdk_md_open( &md, CDK_MD_SHA1, 0 );
+    if( rc ) {
         _cdk_free_signature( sig );
-        return CDK_Gcry_Error;
+        return rc;
     }
     _cdk_hash_pubkey( sk->pk, md, 0 );
     _cdk_free_signature( sig );
@@ -924,6 +915,7 @@ cdk_pk_get_fingerprint( cdk_pkt_pubkey_t pk, byte * fpr )
     cdk_md_hd_t hd;
     int md_algo;
     int dlen = 0;
+    int rc;
 
     if( !pk || !fpr )
         return CDK_Inv_Value;
@@ -933,9 +925,9 @@ cdk_pk_get_fingerprint( cdk_pkt_pubkey_t pk, byte * fpr )
     else
         md_algo = pk->version < 4 ? CDK_MD_RMD160 : CDK_MD_SHA1;
     dlen = cdk_md_get_algo_dlen( md_algo );
-    hd = cdk_md_open( md_algo, 0 );
-    if( !hd )
-        return CDK_Gcry_Error;
+    rc = cdk_md_open( &hd, md_algo, 0 );
+    if( rc )
+        return rc;
     _cdk_hash_pubkey( pk, hd, 1 );
     cdk_md_final( hd );
     memcpy( fpr, cdk_md_read( hd, md_algo ), dlen );
@@ -1030,22 +1022,22 @@ _cdk_pkt_get_keyid( cdk_packet_t pkt, u32 * keyid )
         return 0;
     
     switch( pkt->pkttype ) {
-    case CDK_PKT_PUBLIC_KEY:
-    case CDK_PKT_PUBLIC_SUBKEY:
-        lowbits = cdk_pk_get_keyid( pkt->pkt.public_key, keyid );
-        break;
+      case CDK_PKT_PUBLIC_KEY:
+      case CDK_PKT_PUBLIC_SUBKEY:
+          lowbits = cdk_pk_get_keyid( pkt->pkt.public_key, keyid );
+          break;
       
-    case CDK_PKT_SECRET_KEY:
-    case CDK_PKT_SECRET_SUBKEY:
-        lowbits = cdk_sk_get_keyid( pkt->pkt.secret_key, keyid );
-        break;
+      case CDK_PKT_SECRET_KEY:
+      case CDK_PKT_SECRET_SUBKEY:
+          lowbits = cdk_sk_get_keyid( pkt->pkt.secret_key, keyid );
+          break;
 
-    case CDK_PKT_SIGNATURE:
-        lowbits = cdk_sig_get_keyid( pkt->pkt.signature, keyid );
-        break;
+      case CDK_PKT_SIGNATURE:
+          lowbits = cdk_sig_get_keyid( pkt->pkt.signature, keyid );
+          break;
       
-    default:
-        lowbits = 0;
+      default:
+          lowbits = 0;
     }
     return lowbits;
 }
@@ -1058,16 +1050,16 @@ _cdk_pkt_get_fingerprint( cdk_packet_t pkt, byte * fpr )
         return CDK_Inv_Value;
     
     switch( pkt->pkttype ) {
-    case CDK_PKT_PUBLIC_KEY:
-    case CDK_PKT_PUBLIC_SUBKEY:
-        return cdk_pk_get_fingerprint( pkt->pkt.public_key, fpr );
+      case CDK_PKT_PUBLIC_KEY:
+      case CDK_PKT_PUBLIC_SUBKEY:
+          return cdk_pk_get_fingerprint( pkt->pkt.public_key, fpr );
 
-    case CDK_PKT_SECRET_KEY:
-    case CDK_PKT_SECRET_SUBKEY:
-        return cdk_pk_get_fingerprint( pkt->pkt.secret_key->pk, fpr );
+      case CDK_PKT_SECRET_KEY:
+      case CDK_PKT_SECRET_SUBKEY:
+          return cdk_pk_get_fingerprint( pkt->pkt.secret_key->pk, fpr );
 
-    default:
-        return CDK_Inv_Packet;
+      default:
+          return CDK_Inv_Packet;
     }
     return 0;
 }
