@@ -1,5 +1,4 @@
-/* -*- Mode: C; c-file-style: "bsd" -*-
- * read-packet.c - Read OpenPGP packets
+/* read-packet.c - Read OpenPGP packets
  *        Copyright (C) 2001, 2002, 2003 Timo Schulz 
  *
  * This file is part of OpenCDK.
@@ -206,7 +205,7 @@ _cdk_pkt_read_len( FILE * inp, int * ret_partial )
 
 static int
 read_encrypted( cdk_stream_t inp, size_t pktlen, cdk_pkt_encrypted_t enc,
-		int partial, int mdc )
+		int partial, int is_mdc )
 {
     int rc = 0, version;
 
@@ -216,7 +215,7 @@ read_encrypted( cdk_stream_t inp, size_t pktlen, cdk_pkt_encrypted_t enc,
     if( DEBUG_PKT )
         _cdk_log_debug( "** read encrypted packet %d bytes\n", pktlen );
 
-    if( mdc ) {
+    if( is_mdc ) {
         version = stream_getc( inp );
         if( version != 1 )
             return CDK_Inv_Packet;
@@ -257,24 +256,24 @@ read_symkey_enc( cdk_stream_t inp, size_t pktlen, cdk_pkt_symkey_enc_t ske )
     ske->cipher_algo = stream_getc( inp );
     s2k->mode = stream_getc( inp );
     switch( s2k->mode ) {
-    case 0: minlen = 0; break;
-    case 1: minlen = 8; break;
-    case 3: minlen = 9; break;
+      case CDK_S2K_SIMPLE    : minlen = 0; break;
+      case CDK_S2K_SALTED    : minlen = 8; break;
+      case CDK_S2K_ITERSALTED: minlen = 9; break;
     }
     s2k->hash_algo = stream_getc( inp );
-    if( s2k->mode == 0 )
+    if( s2k->mode == CDK_S2K_SIMPLE )
         ; /* nothing to do */
-    else if( s2k->mode == 1 || s2k->mode == 3 ) {
+    else if( s2k->mode == CDK_S2K_SALTED || s2k->mode == CDK_S2K_ITERSALTED ) {
         rc = stream_read( inp, s2k->salt, DIM (s2k->salt), &nread);
         if( !rc && nread != DIM( s2k->salt ) )
             return CDK_Inv_Packet;
-        if( !rc && s2k->mode == 3 )
+        if( !rc && s2k->mode == CDK_S2K_ITERSALTED )
             s2k->count = stream_getc( inp );
     }
     else
         return CDK_Inv_Packet;
     ske->seskeylen = pktlen - 4 - minlen;
-    if( ske->seskeylen > sizeof ske->seskey )
+    if( ske->seskeylen > DIM( ske->seskey ) )
         return CDK_Inv_Packet;
     for( nread = 0; nread < ske->seskeylen; nread++ ) {
         ske->seskey[nread] = stream_getc( inp );
@@ -356,6 +355,7 @@ read_compressed( cdk_stream_t inp, size_t pktlen, cdk_pkt_compressed_t c )
         c->len = 0;
         c->buf = inp;
     }
+    /* fixme: support for partial bodies. */
 
     return 0;
 }
@@ -543,12 +543,12 @@ read_attribute (cdk_stream_t inp, size_t pktlen, cdk_pkt_userid_t attr)
     }
     p = buf;
     len = *p++;
-    if (len == 255) {
+    if( len == 255 ) {
         len = _cdk_buftou32( p );
         p += 4;
         pktlen -= 4;
     }
-    else if (len >= 192) {
+    else if( len >= 192 ) {
         if( pktlen < 2 ) {
             cdk_free( buf );
             return CDK_Inv_Packet;
@@ -773,8 +773,7 @@ read_signature( cdk_stream_t inp, size_t pktlen, cdk_pkt_signature_t sig )
                 sig->flags.notation = 1;
             else if( node->type == CDK_SIGSUBPKT_REVOCABLE && node->d[0] == 0 )
                 sig->flags.revocable = 0;
-            else if( node->type == CDK_SIGSUBPKT_EXPORTABLE
-                     && node->d[0] == 0 )
+            else if( node->type == CDK_SIGSUBPKT_EXPORTABLE && node->d[0]== 0 )
                 sig->flags.exportable = 0;
 	}
         if( sig->sig_class == 0x1F ) {
@@ -905,13 +904,11 @@ skip_packet( cdk_stream_t inp, size_t pktlen )
 {
     byte buf[4096];
     size_t nread;
-  
-    while( pktlen > 4095 ) {
-        stream_read( inp, buf, sizeof buf-1, &nread );
+
+    while( pktlen > 0 ) {
+        stream_read( inp, buf, pktlen>4095? sizeof buf-1 : pktlen, &nread );
         pktlen -= nread;
     }
-    stream_read( inp, buf, pktlen, &nread );
-    pktlen -= nread;
     assert( pktlen == 0 );
 }
 
@@ -969,123 +966,124 @@ cdk_pkt_read( cdk_stream_t inp, cdk_packet_t pkt )
     pkt->old_ctb = is_newctb? 0 : 1;
 
     switch( pkt->pkttype ) {
-    case CDK_PKT_ATTRIBUTE:
-        pkt->pkt.user_id = cdk_calloc (1,
-                                       sizeof *pkt->pkt.user_id + pkt->pktlen);
-        if (!pkt->pkt.user_id)
-            return CDK_Out_Of_Core;
-        rc = read_attribute (inp, pktlen, pkt->pkt.user_id);
-        pkt->pkttype = CDK_PKT_USER_ID; /* treated as an user id */
-        break;
+      case CDK_PKT_ATTRIBUTE:
+          pkt->pkt.user_id = cdk_calloc( 1, sizeof *pkt->pkt.user_id
+                                         + pkt->pktlen );
+          if( !pkt->pkt.user_id )
+              return CDK_Out_Of_Core;
+          rc = read_attribute( inp, pktlen, pkt->pkt.user_id );
+          pkt->pkttype = CDK_PKT_USER_ID; /* treated as an user id (??) */
+          break;
 
-    case CDK_PKT_USER_ID:
-        pkt->pkt.user_id = cdk_calloc (1,
-                                       sizeof *pkt->pkt.user_id + pkt->pktlen);
-        if (!pkt->pkt.user_id)
-            return CDK_Out_Of_Core;
-        rc = read_user_id (inp, pktlen, pkt->pkt.user_id);
-        break;
+      case CDK_PKT_USER_ID:
+          pkt->pkt.user_id = cdk_calloc( 1, sizeof *pkt->pkt.user_id
+                                         + pkt->pktlen );
+          if( !pkt->pkt.user_id )
+              return CDK_Out_Of_Core;
+          rc = read_user_id( inp, pktlen, pkt->pkt.user_id );
+          break;
 
-    case CDK_PKT_PUBLIC_KEY:
-        pkt->pkt.public_key = cdk_calloc (1, sizeof *pkt->pkt.public_key);
-        if (!pkt->pkt.public_key)
-            return CDK_Out_Of_Core;
-        rc = read_public_key (inp, pkt->pkt.public_key);
-        break;
+      case CDK_PKT_PUBLIC_KEY:
+          pkt->pkt.public_key = cdk_calloc( 1, sizeof *pkt->pkt.public_key );
+          if( !pkt->pkt.public_key )
+              return CDK_Out_Of_Core;
+          rc = read_public_key( inp, pkt->pkt.public_key );
+          break;
 
-    case CDK_PKT_PUBLIC_SUBKEY:
-        pkt->pkt.public_key = cdk_calloc (1, sizeof *pkt->pkt.public_key);
-        if (!pkt->pkt.public_key)
-            return CDK_Out_Of_Core;
-        rc = read_public_subkey (inp, pkt->pkt.public_key);
-        break;
+      case CDK_PKT_PUBLIC_SUBKEY:
+          pkt->pkt.public_key = cdk_calloc( 1, sizeof *pkt->pkt.public_key );
+          if( !pkt->pkt.public_key )
+              return CDK_Out_Of_Core;
+          rc = read_public_subkey( inp, pkt->pkt.public_key );
+          break;
 
-    case CDK_PKT_SECRET_KEY:
-        pkt->pkt.secret_key = cdk_calloc (1, sizeof *pkt->pkt.secret_key);
-        if (!pkt->pkt.secret_key)
-            return CDK_Out_Of_Core;
-        pkt->pkt.secret_key->pk =cdk_calloc (1,
-                                             sizeof *pkt->pkt.secret_key->pk);
-        if (!pkt->pkt.secret_key->pk)
-            return CDK_Out_Of_Core;
-        rc = read_secret_key (inp, pktlen, pkt->pkt.secret_key);
-        break;
+      case CDK_PKT_SECRET_KEY:
+          pkt->pkt.secret_key = cdk_calloc( 1, sizeof *pkt->pkt.secret_key );
+          if( !pkt->pkt.secret_key )
+              return CDK_Out_Of_Core;
+          pkt->pkt.secret_key->pk =cdk_calloc( 1, sizeof
+                                               *pkt->pkt.secret_key->pk );
+          if( !pkt->pkt.secret_key->pk )
+              return CDK_Out_Of_Core;
+          rc = read_secret_key( inp, pktlen, pkt->pkt.secret_key );
+          break;
 
-    case CDK_PKT_SECRET_SUBKEY:
-        pkt->pkt.secret_key = cdk_calloc (1, sizeof *pkt->pkt.secret_key);
-        if (!pkt->pkt.secret_key)
-            return CDK_Out_Of_Core;
-        pkt->pkt.secret_key->pk =
-            cdk_calloc (1, sizeof *pkt->pkt.secret_key->pk);
-        if (!pkt->pkt.secret_key->pk)
-            return CDK_Out_Of_Core;
-        rc = read_secret_subkey (inp, pktlen, pkt->pkt.secret_key);
-        break;
+      case CDK_PKT_SECRET_SUBKEY:
+          pkt->pkt.secret_key = cdk_calloc( 1, sizeof *pkt->pkt.secret_key );
+          if( !pkt->pkt.secret_key )
+              return CDK_Out_Of_Core;
+          pkt->pkt.secret_key->pk =
+              cdk_calloc( 1, sizeof *pkt->pkt.secret_key->pk );
+          if( !pkt->pkt.secret_key->pk )
+              return CDK_Out_Of_Core;
+          rc = read_secret_subkey( inp, pktlen, pkt->pkt.secret_key );
+          break;
 
-    case CDK_PKT_LITERAL:
-        pkt->pkt.literal = cdk_calloc( 1, sizeof *pkt->pkt.literal );
-        if (!pkt->pkt.literal)
-            return CDK_Out_Of_Core;
-        rc = read_literal( inp, pktlen, &pkt->pkt.literal, is_partial);
-        break;
+      case CDK_PKT_LITERAL:
+          pkt->pkt.literal = cdk_calloc( 1, sizeof *pkt->pkt.literal  );
+          if( !pkt->pkt.literal )
+              return CDK_Out_Of_Core;
+          rc = read_literal( inp, pktlen, &pkt->pkt.literal, is_partial );
+          break;
 
-    case CDK_PKT_ONEPASS_SIG:
-        pkt->pkt.onepass_sig = cdk_calloc (1, sizeof *pkt->pkt.onepass_sig);
-        if (!pkt->pkt.onepass_sig)
-            return CDK_Out_Of_Core;
-        rc = read_onepass_sig (inp, pktlen, pkt->pkt.onepass_sig);
-        break;
+      case CDK_PKT_ONEPASS_SIG:
+          pkt->pkt.onepass_sig = cdk_calloc( 1, sizeof *pkt->pkt.onepass_sig );
+          if( !pkt->pkt.onepass_sig )
+              return CDK_Out_Of_Core;
+          rc = read_onepass_sig( inp, pktlen, pkt->pkt.onepass_sig );
+          break;
 
-    case CDK_PKT_SIGNATURE:
-        pkt->pkt.signature = cdk_calloc (1, sizeof *pkt->pkt.signature);
-        if (!pkt->pkt.signature)
-            return CDK_Out_Of_Core;
-        rc = read_signature (inp, pktlen, pkt->pkt.signature);
-        break;
+      case CDK_PKT_SIGNATURE:
+          pkt->pkt.signature = cdk_calloc( 1, sizeof *pkt->pkt.signature );
+          if( !pkt->pkt.signature )
+              return CDK_Out_Of_Core;
+          rc = read_signature( inp, pktlen, pkt->pkt.signature );
+          break;
 
-    case CDK_PKT_ENCRYPTED_MDC:
-    case CDK_PKT_ENCRYPTED:
-        pkt->pkt.encrypted = cdk_calloc (1, sizeof *pkt->pkt.encrypted);
-        if (!pkt->pkt.encrypted)
-            return CDK_Out_Of_Core;
-        use_mdc = (pkt->pkttype == CDK_PKT_ENCRYPTED_MDC) ? 1 : 0;
-        rc = read_encrypted( inp, pktlen, pkt->pkt.encrypted,
-                             is_partial, use_mdc );
-        break;
+      case CDK_PKT_ENCRYPTED_MDC:
+      case CDK_PKT_ENCRYPTED:
+          pkt->pkt.encrypted = cdk_calloc( 1, sizeof *pkt->pkt.encrypted );
+          if( !pkt->pkt.encrypted )
+              return CDK_Out_Of_Core;
+          use_mdc =( pkt->pkttype == CDK_PKT_ENCRYPTED_MDC) ? 1 : 0;
+          rc = read_encrypted( inp, pktlen, pkt->pkt.encrypted,
+                               is_partial, use_mdc  );
+          break;
 
-    case CDK_PKT_SYMKEY_ENC:
-        pkt->pkt.symkey_enc = cdk_calloc (1, sizeof *pkt->pkt.symkey_enc);
-        if (!pkt->pkt.symkey_enc)
-            return CDK_Out_Of_Core;
-        rc = read_symkey_enc (inp, pktlen, pkt->pkt.symkey_enc);
-        break;
+      case CDK_PKT_SYMKEY_ENC:
+          pkt->pkt.symkey_enc = cdk_calloc( 1, sizeof *pkt->pkt.symkey_enc );
+          if( !pkt->pkt.symkey_enc )
+              return CDK_Out_Of_Core;
+          rc = read_symkey_enc( inp, pktlen, pkt->pkt.symkey_enc );
+          break;
 
-    case CDK_PKT_PUBKEY_ENC:
-        pkt->pkt.pubkey_enc = cdk_calloc (1, sizeof *pkt->pkt.pubkey_enc);
-        if (!pkt->pkt.pubkey_enc)
-            return CDK_Out_Of_Core;
-        rc = read_pubkey_enc (inp, pktlen, pkt->pkt.pubkey_enc);
-        break;
+      case CDK_PKT_PUBKEY_ENC:
+          pkt->pkt.pubkey_enc = cdk_calloc( 1, sizeof *pkt->pkt.pubkey_enc );
+          if( !pkt->pkt.pubkey_enc )
+              return CDK_Out_Of_Core;
+          rc = read_pubkey_enc( inp, pktlen, pkt->pkt.pubkey_enc );
+          break;
 
-    case CDK_PKT_COMPRESSED:
-        pkt->pkt.compressed = cdk_calloc (1, sizeof *pkt->pkt.compressed);
-        if (!pkt->pkt.compressed)
-            return CDK_Out_Of_Core;
-        rc = read_compressed (inp, pktlen, pkt->pkt.compressed);
-        break;
+      case CDK_PKT_COMPRESSED:
+          pkt->pkt.compressed = cdk_calloc( 1, sizeof *pkt->pkt.compressed );
+          if( !pkt->pkt.compressed )
+              return CDK_Out_Of_Core;
+          rc = read_compressed( inp, pktlen, pkt->pkt.compressed );
+          break;
 
-    case CDK_PKT_MDC:
-        pkt->pkt.mdc = cdk_calloc (1, sizeof *pkt->pkt.mdc);
-        if (!pkt->pkt.mdc)
-            return CDK_Out_Of_Core;
-        rc = read_mdc (inp, pkt->pkt.mdc);
-        break;
+      case CDK_PKT_MDC:
+          pkt->pkt.mdc = cdk_calloc( 1, sizeof *pkt->pkt.mdc );
+          if( !pkt->pkt.mdc )
+              return CDK_Out_Of_Core;
+          rc = read_mdc( inp, pkt->pkt.mdc );
+          break;
 
-    default:
-        /* skip all packets we don't understand */
-        skip_packet( inp, pktlen );
-        break;
+      default:
+          /* skip all packets we don't understand */
+          skip_packet( inp, pktlen );
+          break;
     }
 
     return rc;
 }
+
