@@ -40,10 +40,8 @@ hash_mpibuf( cdk_pkt_pubkey_t pk, cdk_md_hd_t md )
     npkey = cdk_pk_get_npkey( pk->pubkey_algo );
     for( i = 0; i < npkey; i++ ) {
         a = pk->mpi[i];
-        if( pk->version == 4 ) {
-            cdk_md_putc( md, a->bits >> 8 );
-            cdk_md_putc( md, a->bits );
-        }
+        if( pk->version == 4 )
+            cdk_md_write( md, a->data, 2 );
         cdk_md_write( md, a->data + 2, a->bytes );
     }
 }
@@ -52,7 +50,7 @@ hash_mpibuf( cdk_pkt_pubkey_t pk, cdk_md_hd_t md )
 void
 _cdk_hash_pubkey( cdk_pkt_pubkey_t pk, cdk_md_hd_t md, int usefpr )
 {
-    byte buf[4];
+    byte buf[12];
     u16 n;
     int i, npkey;
 
@@ -69,26 +67,26 @@ _cdk_hash_pubkey( cdk_pkt_pubkey_t pk, cdk_md_hd_t md, int usefpr )
         n += pk->mpi[i]->bytes;
         n += 2;
     }
-  
-    cdk_md_putc( md, 0x99 );
-    cdk_md_putc( md, n >> 8 );
-    cdk_md_putc( md, n );
-    cdk_md_putc( md, pk->version );
-  
-    buf[0] = pk->timestamp >> 24;
-    buf[1] = pk->timestamp >> 16;
-    buf[2] = pk->timestamp >> 8;
-    buf[3] = pk->timestamp;
-    cdk_md_write( md, buf, 4 );
+
+    i = 0;
+    buf[i++] = 0x99;
+    buf[i++] = n >> 8;
+    buf[i++] = n;
+    buf[i++] = pk->version;
+    buf[i++] = pk->timestamp >> 24;
+    buf[i++] = pk->timestamp >> 16;
+    buf[i++] = pk->timestamp >> 8;
+    buf[i++] = pk->timestamp;
 
     if( pk->version < 4 ) {
         u16 a = 0;
         if( pk->expiredate )
             a = (u16)((pk->expiredate - pk->timestamp) / 86400L);
-        cdk_md_putc( md, a >> 8 );
-        cdk_md_putc( md, a );
+        buf[i++] = a >> 8;
+        buf[i++] = a;
     }
-    cdk_md_putc( md, pk->pubkey_algo );
+    buf[i++] = pk->pubkey_algo;
+    cdk_md_write( md, buf, i );
     hash_mpibuf( pk, md );
 }
 
@@ -129,43 +127,58 @@ _cdk_hash_userid( cdk_pkt_userid_t uid, int is_v4, cdk_md_hd_t md )
 void
 _cdk_hash_sig_data( cdk_pkt_signature_t sig, cdk_md_hd_t md )
 {
-    byte buf[4];
+    byte * buf;
     size_t n = 0;
+    int i = 0;
 
     if( !sig || !md )
         return;
 
+    buf = cdk_calloc( 1, 32 );
+    if( !buf )
+        return;
+    
     if( sig->version == 4 )
-        cdk_md_putc( md, sig->version );
-    cdk_md_putc( md, sig->sig_class );
+        buf[i++] = sig->version;
+    buf[i++] = sig->sig_class;
     if( sig->version < 4 ) {
-        buf[0] = sig->timestamp >> 24;
-        buf[1] = sig->timestamp >> 16;
-        buf[2] = sig->timestamp >> 8;
-        buf[3] = sig->timestamp;
-        cdk_md_write( md, buf, 4 );
+        buf[i++] = sig->timestamp >> 24;
+        buf[i++] = sig->timestamp >> 16;
+        buf[i++] = sig->timestamp >> 8;
+        buf[i++] = sig->timestamp;
     }
     else {
-        cdk_md_putc( md, sig->pubkey_algo );
-        cdk_md_putc( md, sig->digest_algo );
+        buf[i++] = sig->pubkey_algo;
+        buf[i++] = sig->digest_algo;
         if( sig->hashed ) {
-            _cdk_subpkt_hash( sig->hashed, &n, md );
+            byte * p = _cdk_subpkt_get_array( sig->hashed, 0, &n );
+            if( !p )
+                return;
+            buf = cdk_realloc( buf, i+n+2+1 );
+            if( !buf )
+                return;
+            buf[i++] = n >> 8;
+            buf[i++] = n;
+            memcpy( buf+i, p, n );
+            i += n;
+            cdk_free( p );
             sig->hashed_size = n;
             n = sig->hashed_size + 6;
 	}
         else {
-            cdk_md_putc( md, 0 );
-            cdk_md_putc( md, 0 );
+            buf[i++] = 0x00;
+            buf[i++] = 0x00;
+            sig->hashed_size = 6;
             n = 6;
 	}
-        cdk_md_putc( md, sig->version );
-        cdk_md_putc( md, 0xff );
-        buf[0] = n >> 24;
-        buf[1] = n >> 16;
-        buf[2] = n >> 8;
-        buf[3] = n;
-        cdk_md_write( md, buf, 4 );
+        buf[i++] = sig->version;
+        buf[i++] = 0xFF;
+        buf[i++] = n >> 24;
+        buf[i++] = n >> 16;
+        buf[i++] = n >> 8;
+        buf[i++] = n;
     }
+    cdk_md_write( md, buf, i );
 }
 
 
@@ -245,9 +258,9 @@ _cdk_pk_check_sig( cdk_keydb_hd_t hd, cdk_kbnode_t knode, cdk_kbnode_t snode )
     sig = snode->pkt->pkt.signature;
     digest_algo = sig->digest_algo;
 
-    md = cdk_md_open( digest_algo, 0 );
-    if( !md )
-        return CDK_Out_Of_Core;
+    rc = cdk_md_open( &md, digest_algo, 0 );
+    if( rc )
+        return rc;
 
     if( sig->sig_class == 0x20 ) { /* key revocation */
         cdk_kbnode_hash( knode, md, 0, 0, 0 );
@@ -314,7 +327,7 @@ cdk_error_t
 cdk_pk_check_sigs( cdk_kbnode_t knode, cdk_keydb_hd_t hd, int * r_status )
 {
     cdk_pkt_signature_t sig = NULL;
-    cdk_kbnode_t k;
+    cdk_kbnode_t node;
     u32 keyid = 0;
     int key_status = 0;
     int rc = 0;
@@ -322,24 +335,24 @@ cdk_pk_check_sigs( cdk_kbnode_t knode, cdk_keydb_hd_t hd, int * r_status )
     if( !knode || !r_status )
         return CDK_Inv_Value;
 
-    k = cdk_kbnode_find( knode, CDK_PKT_PUBLIC_KEY );
-    if( !k )
+    node = cdk_kbnode_find( knode, CDK_PKT_PUBLIC_KEY );
+    if( !node )
         return CDK_Error_No_Key;
-    if( k->pkt->pkt.public_key->is_revoked )
+    if( node->pkt->pkt.public_key->is_revoked )
         key_status |= CDK_KEY_REVOKED;
-    if( k->pkt->pkt.public_key->has_expired )
+    if( node->pkt->pkt.public_key->has_expired )
         key_status |= CDK_KEY_EXPIRED;
     if( key_status ) {
         *r_status = key_status;
         return CDK_General_Error;
     }
-    keyid = cdk_pk_get_keyid( k->pkt->pkt.public_key, NULL );
+    keyid = cdk_pk_get_keyid( node->pkt->pkt.public_key, NULL );
 
-    for( k = knode; k && k->pkt->pkttype; k = k->next ) {
-        if( k->pkt->pkttype != CDK_PKT_SIGNATURE )
+    for( node = knode; node; node = node->next ) {
+        if( node->pkt->pkttype != CDK_PKT_SIGNATURE )
             continue;
-        sig = k->pkt->pkt.signature;
-        rc = _cdk_pk_check_sig( hd, knode, k );
+        sig = node->pkt->pkt.signature;
+        rc = _cdk_pk_check_sig( hd, knode, node );
         if( rc && IS_UID_SIG( sig ) && rc == CDK_Error_No_Key ) {
             sig->flags.missing_key = 1;
             continue;
@@ -356,3 +369,4 @@ cdk_pk_check_sigs( cdk_kbnode_t knode, cdk_keydb_hd_t hd, int * r_status )
         *r_status = CDK_KEY_VALID;
     return rc;
 }
+
