@@ -1,5 +1,4 @@
-/* -*- Mode: C; c-file-style: "bsd" -*-
- * sign.c - Signing routines
+/* sign.c - Signing routines
  *        Copyright (C) 2002, 2003 Timo Schulz
  *
  * This file is part of OpenCDK.
@@ -69,7 +68,7 @@ _cdk_sig_hash_for( int pkalgo, int pktver )
 int
 _cdk_sig_create( cdk_pkt_pubkey_t pk, cdk_pkt_signature_t sig )
 {
-    cdk_subpkt_t node;
+    cdk_subpkt_t n, node;
     byte buf[8];
 
     if( !sig )
@@ -83,7 +82,7 @@ _cdk_sig_create( cdk_pkt_pubkey_t pk, cdk_pkt_signature_t sig )
         cdk_pk_get_keyid( pk, sig->keyid );
     }
     sig->timestamp = _cdk_timestamp( );
-    if( sig->version == 3 )
+    if( sig->version < 4 )
         return 0;
   
     sig->hashed = sig->unhashed = NULL;
@@ -91,9 +90,10 @@ _cdk_sig_create( cdk_pkt_pubkey_t pk, cdk_pkt_signature_t sig )
     _cdk_u32tobuf( sig->keyid[0], buf );
     _cdk_u32tobuf( sig->keyid[1], buf + 4 );
     node = cdk_subpkt_new( 8 );
-    if( node )
+    if( node ) {
         cdk_subpkt_init( node, CDK_SIGSUBPKT_ISSUER, buf, 8 );
-    sig->unhashed = node;
+        sig->unhashed = node;
+    }
   
     _cdk_u32tobuf( sig->timestamp, buf );
     node = cdk_subpkt_new( 4 );
@@ -111,6 +111,26 @@ _cdk_sig_create( cdk_pkt_pubkey_t pk, cdk_pkt_signature_t sig )
             cdk_subpkt_add( sig->hashed, node );
         }
     }
+    if( pk->uid ) {
+        char * p = cdk_utf8_encode( pk->uid->name );
+        node = cdk_subpkt_new( strlen( p ) + 1 );
+        if( node ) {
+            cdk_subpkt_init( node, CDK_SIGSUBPKT_SIGNERS_UID, p, strlen( p ) );
+            cdk_subpkt_add( sig->hashed, node );
+        }
+        cdk_free( p );
+        if( pk->uid->selfsig ) {
+            n = cdk_subpkt_find( pk->uid->selfsig->hashed,
+                                 CDK_SIGSUBPKT_PREF_KS );
+            if( n ) {
+                node = cdk_subpkt_new( n->size+1 );
+                if( node ) {
+                    cdk_subpkt_init( node, n->type, n->d, n->size );
+                    cdk_subpkt_add( sig->hashed, node );
+                }
+            }
+        }
+    }
     calc_subpkt_size( sig );
     return 0;
 }
@@ -120,7 +140,7 @@ int
 _cdk_sig_complete( cdk_pkt_signature_t sig, cdk_pkt_seckey_t sk,
                    cdk_md_hd_t md )
 {
-    byte digest[24];
+    byte * mdbuf;
 
     if( !sig || !sk || !md )
         return CDK_Inv_Value;
@@ -128,9 +148,8 @@ _cdk_sig_complete( cdk_pkt_signature_t sig, cdk_pkt_seckey_t sk,
     calc_subpkt_size( sig );
     _cdk_hash_sig_data( sig, md );
     cdk_md_final( md );
-    memcpy( digest, cdk_md_read( md, sig->digest_algo),
-            cdk_md_get_algo_dlen( sig->digest_algo ) );
-    return cdk_pk_sign( sk, sig, digest );
+    mdbuf = cdk_md_read( md, sig->digest_algo );
+    return cdk_pk_sign( sk, sig, mdbuf );
 }
 
 
@@ -139,18 +158,19 @@ write_literal( cdk_stream_t inp, cdk_stream_t out )
 {
     cdk_packet_t pkt;
     cdk_pkt_literal_t pt;
-    const char * s = _cdk_stream_get_fname( inp );
+    const char * s;
     int rc;
 
     if( !inp || !out )
         return CDK_Inv_Value;
 
+    s = _cdk_stream_get_fname( inp );
+    if( !s )
+        s = "_CONSOLE";
     pkt = cdk_calloc( 1, sizeof *pkt );
     if( !pkt )
         return CDK_Out_Of_Core;
     cdk_stream_seek( inp, 0 );
-    if( !s )
-        s = "_CONSOLE";
     pt = cdk_calloc( 1, sizeof *pt + strlen( s ) + 1 );
     if( !pt )
         return CDK_Out_Of_Core;
@@ -204,10 +224,8 @@ sig_get_version( cdk_ctx_t hd, cdk_keylist_t kl )
 
     if( hd && hd->opt.compat )
         return 3;
-  
     for( l = kl; l; l = l->next ) {
-        if( (l->type == CDK_PKT_PUBLIC_KEY && l->key.pk->version == 3)
-            || (l->type == CDK_PKT_SECRET_KEY && l->key.sk->version == 3))
+        if( l->version == 3 )
             return 3;
     }
     return 4;
@@ -258,14 +276,14 @@ cdk_stream_sign( cdk_ctx_t hd, cdk_stream_t inp, cdk_stream_t out,
         cdk_stream_set_armor_flag( out, detached? CDK_ARMOR_SIGNATURE : 0 );
   
     if( encryptflag ) {
-        cdk_stream_set_cache( out, 1 );
+        cdk_stream_switch_cache( out, 1 );
         rc = write_pubkey_enc_list( hd, out, remusr );
         if( rc ) {
             cdk_sklist_release( list );
             return rc;
         }
         cdk_stream_set_cipher_flag( out, hd->dek, hd->dek->use_mdc );
-        cdk_stream_set_cache( out, 0 );
+        cdk_stream_switch_cache( out, 0 );
     }
   
     cdk_stream_set_hash_flag( inp, digest_algo );
@@ -350,17 +368,17 @@ put_hash_line( cdk_stream_t out, int digest_algo, int is_v4 )
     const char * s = NULL;
 
     if( !is_v4 ) {
-        cdk_stream_putc( out, '\n' );
+        s = _cdk_armor_get_lineend( );
+        _cdk_stream_puts( out, s );
         return;
     }
 
     switch( digest_algo ) {
-    case CDK_MD_MD2    : s = "Hash: MD2\n\n"; break;
-    case CDK_MD_MD5    : s = "Hash: MD5\n\n"; break;
-    case CDK_MD_SHA1   : s = "Hash: SHA1\n\n"; break;
-    case CDK_MD_RMD160 : s = "Hash: RIPEMD160\n\n"; break;
-    case CDK_MD_SHA256 : s = "Hash: SHA256\n\n"; break;
-    default            : s = "Hash: SHA1\n\n"; break;
+      case CDK_MD_MD5    : s = "Hash: MD5\n\n"; break;
+      case CDK_MD_SHA1   : s = "Hash: SHA1\n\n"; break;
+      case CDK_MD_RMD160 : s = "Hash: RIPEMD160\n\n"; break;
+      case CDK_MD_SHA256 : s = "Hash: SHA256\n\n"; break;
+      default            : s = "Hash: SHA1\n\n"; break;
     }
     _cdk_stream_puts( out, s );
 }
@@ -369,10 +387,11 @@ put_hash_line( cdk_stream_t out, int digest_algo, int is_v4 )
 void
 _cdk_trim_string( char * s, int canon )
 {
-    while( s && *s &&(  s[strlen( s )-1] == '\t'
-                        || s[strlen( s )-1] == '\r'
-                        || s[strlen( s )-1] == '\n'
-                        || s[strlen( s )-1] == ' '))
+    while( s && *s
+           && (  s[strlen( s )-1] == '\t'
+                 || s[strlen( s )-1] == '\r'
+                 || s[strlen( s )-1] == '\n'
+                 || s[strlen( s )-1] == ' ') )
         s[strlen( s ) -1] = '\0';
     if( canon )
         strcat( s, "\r\n" );
@@ -398,15 +417,14 @@ stream_clearsign( cdk_ctx_t hd, cdk_stream_t inp, cdk_stream_t out,
 
     sk = list->key.sk;
     digest_algo = _cdk_sig_hash_for( sk->pubkey_algo, sk->version );
-    md = cdk_md_open( digest_algo, 0 );
-    if( !md ) {
+    rc = cdk_md_open( &md, digest_algo, 0 );
+    if( rc ) {
         cdk_sklist_release( list );
-        return CDK_Gcry_Error;
+        return rc;
     }
 
     s = _cdk_armor_get_lineend( );
-    strcpy( buf, "-----BEGIN PGP SIGNED MESSAGE-----" );
-    strcat( buf, s );
+    sprintf( buf, "-----BEGIN PGP SIGNED MESSAGE-----%s", s );
     _cdk_stream_puts( out, buf );
     put_hash_line( out, digest_algo, sk->version == 4 );
   
@@ -474,20 +492,13 @@ file_clearsign( cdk_ctx_t hd, cdk_strlist_t locusr,
         return CDK_Inv_Mode;
 
     rc = cdk_stream_open( file, &inp );
-    if( rc )
-        return rc;
-  
-    rc = cdk_stream_create( output, &out );
-    if( rc ) {
-        cdk_stream_close( inp );
-        return rc;
-    }
-
-    rc = stream_clearsign( hd, inp, out, locusr );
+    if( !rc )
+        rc = cdk_stream_create( output, &out );
+    if( !rc )
+        rc = stream_clearsign( hd, inp, out, locusr );
     
     cdk_stream_close( inp );
     cdk_stream_close( out );
-    
     return rc;
 }
 
