@@ -1,5 +1,4 @@
-/* -*- Mode: C; c-file-style: "bsd" -*-
- * stream.c - provides a STREAM object
+/* stream.c - provides a STREAM object
  *        Copyright (C) 2002, 2003 Timo Schulz
  *
  * This file is part of OpenCDK.
@@ -16,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with OpenCDK; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+ * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -188,7 +187,6 @@ cdk_stream_tmp_from_mem( const void * buf, size_t count )
 }
 
   
-
 cdk_stream_t
 _cdk_stream_fpopen( FILE * fp, unsigned write_mode )
 {
@@ -239,14 +237,14 @@ cdk_stream_control( cdk_stream_t s, int ctl, int val )
 
     if( val == -1 ) {
         switch( ctl ) {
-        case CDK_STREAMCTL_COMPRESSED: return s->flags.compressed;
+          case CDK_STREAMCTL_COMPRESSED: return s->flags.compressed;
         }
         return 0;
     }
     switch( ctl ) {
-    case CDK_STREAMCTL_DISABLE: s->flags.no_filter = val; break;
-    case CDK_STREAMCTL_COMPRESSED: s->flags.compressed = val; break;
-    default                   : return CDK_Inv_Mode;
+      case CDK_STREAMCTL_DISABLE   : s->flags.no_filter = val; break;
+      case CDK_STREAMCTL_COMPRESSED: s->flags.compressed = val; break;
+      default                      : return CDK_Inv_Mode;
     }
     return 0;
 }
@@ -255,7 +253,7 @@ cdk_stream_control( cdk_stream_t s, int ctl, int val )
 cdk_error_t
 cdk_stream_flush( cdk_stream_t s )
 {
-    int rc = 0;
+    cdk_error_t rc = 0;
   
     if( !s )
         return CDK_Inv_Value;
@@ -299,7 +297,7 @@ cdk_error_t
 cdk_stream_close( cdk_stream_t s )
 {
     struct stream_filter_s * f, * f2;
-    int rc = 0;
+    cdk_error_t rc = 0;
 
     if( !s )
         return CDK_Inv_Value;
@@ -444,13 +442,13 @@ filter_add( cdk_stream_t s, filter_fnct_t fnc, int type )
     f->tmp = NULL;
     f->type = type;
     switch( type ) {
-    case fARMOR    : f->opaque = &f->u.afx; break;
-    case fCIPHER   : f->opaque = &f->u.cfx; break;
-    case fLITERAL: f->opaque = &f->u.pfx; break;
-    case fCOMPRESS : f->opaque = &f->u.zfx; break;
-    case fHASH     : f->opaque = &f->u.mfx; break;
-    case fTEXT     : f->opaque = &f->u.tfx; break;
-    default        : f->opaque = NULL;
+      case fARMOR    : f->opaque = &f->u.afx; break;
+      case fCIPHER   : f->opaque = &f->u.cfx; break;
+      case fLITERAL: f->opaque = &f->u.pfx; break;
+      case fCOMPRESS : f->opaque = &f->u.zfx; break;
+      case fHASH     : f->opaque = &f->u.mfx; break;
+      case fTEXT     : f->opaque = &f->u.tfx; break;
+      default        : f->opaque = NULL;
     }
     return f;
 }
@@ -471,12 +469,12 @@ static filter_fnct_t
 stream_id_to_filter( int type )
 {
     switch( type ) {
-    case fARMOR    : return _cdk_filter_armor;
-    case fLITERAL: return _cdk_filter_literal;
-    case fTEXT     : return _cdk_filter_text;
-    case fCIPHER   : return _cdk_filter_cipher;
-    case fCOMPRESS : return _cdk_filter_compress;
-    default        : return NULL;
+      case fARMOR    : return _cdk_filter_armor;
+      case fLITERAL: return _cdk_filter_literal;
+      case fTEXT     : return _cdk_filter_text;
+      case fCIPHER   : return _cdk_filter_cipher;
+      case fCOMPRESS : return _cdk_filter_compress;
+      default        : return NULL;
     }
 }
 
@@ -503,10 +501,10 @@ cdk_stream_filter_disable( cdk_stream_t s, int type )
 }
 
 
-static int
+static cdk_error_t
 stream_fp_replace( cdk_stream_t s, FILE ** tmp )
 {
-    int rc;
+    cdk_error_t rc;
 
     assert( s );
   
@@ -522,11 +520,11 @@ stream_fp_replace( cdk_stream_t s, FILE ** tmp )
 /* This function is exactly like filter_read, except the fact that we can't
    use tmpfile () all the time. That's why we open the real file when there
    is no last filter. */
-static int
+static cdk_error_t
 stream_filter_write( cdk_stream_t s )
 {
     struct stream_filter_s * f;
-    int rc = 0;
+    cdk_error_t rc = 0;
 
     assert( s );
     
@@ -575,11 +573,11 @@ stream_filter_write( cdk_stream_t s )
    Create a tempfile and use it for the output of the filter. Then the
    original file handle will be closed and replace with the temp handle.
    The file pointer will be set to the begin and the game starts again. */
-static int
+static cdk_error_t
 stream_filter_read( cdk_stream_t s )
 {
     struct stream_filter_s * f;
-    int rc = 0;
+    cdk_error_t rc = 0;
 
     assert( s );
     
@@ -767,7 +765,7 @@ cdk_stream_tell( cdk_stream_t s )
 cdk_error_t
 cdk_stream_seek( cdk_stream_t s, long offset )
 {
-    int rc;
+    cdk_error_t rc;
 
     if( !s )
         return CDK_Inv_Value;    
@@ -780,10 +778,10 @@ cdk_stream_seek( cdk_stream_t s, long offset )
 }
 
 
-static int
+static cdk_error_t
 stream_flush( cdk_stream_t s )
 {
-    int rc;
+    cdk_error_t rc;
     
     assert( s );
     
@@ -844,7 +842,7 @@ cdk_stream_set_cipher_flag( cdk_stream_t s, cdk_dek_t dek, int use_mdc )
     dek->use_mdc = use_mdc;
     f->ctl = stream_get_mode( s );
     f->u.cfx.dek = dek;
-    f->u.cfx.mdc_method = use_mdc? GCRY_MD_SHA1 : 0;
+    f->u.cfx.mdc_method = use_mdc? CDK_MD_SHA1 : 0;
     if( s->blkmode ) {
         f->u.cfx.blkmode.on = 1;
         f->u.cfx.blkmode.size = s->blkmode;
@@ -905,8 +903,15 @@ cdk_stream_set_hash_flag( cdk_stream_t s, int algo )
 }
 
 
+/**
+ * cdk_stream_switch_cache:
+ * @s: the stream object
+ * @val: 1=on, 0=off
+ *
+ * Enable or disable the cache section of a stream object.
+ **/
 cdk_error_t
-cdk_stream_set_cache( cdk_stream_t s, int val )
+cdk_stream_switch_cache( cdk_stream_t s, int val )
 {
     if( !s )
         return CDK_Inv_Value;
@@ -941,7 +946,7 @@ cdk_stream_kick_off( cdk_stream_t inp, cdk_stream_t out )
 {
     byte buf[8192];
     int nread, nwritten;
-    int rc = 0;
+    cdk_error_t rc = 0;
 
     if( !inp || !out )
         return CDK_Inv_Value;
@@ -972,7 +977,8 @@ cdk_stream_mmap( cdk_stream_t s, byte ** ret_buf, size_t * ret_count )
 {
     const u32 max_filesize = 16777216;
     u32 len, oldpos;
-    int n, rc;
+    cdk_error_t rc;
+    int n;
     char * p;
 
     if( !s || !ret_buf || !ret_count )
@@ -1054,7 +1060,7 @@ _cdk_stream_puts( cdk_stream_t s, const char * buf )
 }
 
 
-int
+cdk_error_t
 _cdk_stream_set_blockmode( cdk_stream_t s, size_t nbytes )
 {
     if( !s )
