@@ -1,4 +1,3 @@
-/* -*- Mode: C; c-file-style: "bsd" -*- */
 /* stream.h - internal definiton for the STREAM object
  *        Copyright (C) 2002, 2003 Timo Schulz
  *
@@ -23,29 +22,29 @@
 #define CDK_STREAM_H
 
 enum {
-    fDUMMY     = 0,
-    fARMOR     = 1,
-    fCIPHER    = 2,
-    fPLAINTEXT = 3,
-    fCOMPRESS  = 4,
-    fHASH      = 5,
-    fTEXT      = 6
+    fDUMMY   = 0,
+    fARMOR   = 1,
+    fCIPHER  = 2,
+    fLITERAL = 3,
+    fCOMPRESS= 4,
+    fHASH    = 5,
+    fTEXT    = 6
 };
 
-typedef int (*FILTER_FUNC) (void * opaque, int ctl, FILE * in, FILE * out);
+typedef int (*filter_fnct_t) (void * opaque, int ctl, FILE * in, FILE * out);
 
 struct stream_filter_s {
     struct stream_filter_s * next;
-    FILTER_FUNC fnc;
+    filter_fnct_t fnct;
     void * opaque;
     FILE * tmp;
     union {
-        armor_filter_s afx;
-        cipher_filter_s cfx;
-        plaintext_filter_s pfx;
-        compress_filter_s zfx;
-        text_filter_s tfx;
-        md_filter_s mfx;
+        armor_filter_t afx;
+        cipher_filter_t cfx;
+        literal_filter_t pfx;
+        compress_filter_t zfx;
+        text_filter_t tfx;
+        md_filter_t mfx;
     } u;
     struct {
         unsigned enabled:1;
@@ -56,14 +55,7 @@ struct stream_filter_s {
 };
 
 
-struct stream_filter_table_s {
-    int id;
-    FILTER_FUNC encode;
-    FILTER_FUNC decode;
-};
-
-  
-struct cdk_stream_context_s {
+struct cdk_stream_s {
     struct stream_filter_s * filters;
     int fmode;
     int error;
@@ -75,6 +67,7 @@ struct cdk_stream_context_s {
         unsigned temp:1;
         unsigned reset:1;
         unsigned no_filter:1;
+        unsigned compressed:3;
     } flags;
     struct {
         unsigned char buf[8192];
