@@ -1,5 +1,5 @@
 /* types.h - Some type definitions
- *        Copyright (C) 2002 Timo Schulz
+ *        Copyright (C) 2002, 2003 Timo Schulz
  *
  * This file is part of OpenCDK.
  *
@@ -37,10 +37,16 @@
 
 #ifndef HAVE_U32_TYPEDEF
 # undef u32
-  typedef unsigned int u32;
+  typedef unsigned long u32;
 # define HAVE_U32_TYPEDEF
 #endif
 
+#ifndef HAVE_ULONG_TYPEDEF
+# undef ulong
+  typedef unsigned long ulong;
+# define HAVE_ULONG_TYPEDEF
+#endif
+
 #ifndef DIM
 # define DIM(v) (sizeof (v)/sizeof ((v)[0]))
 # define DIMof(type, member)   DIM(((type *)0)->member)
@@ -50,4 +56,9 @@
 struct cdk_verify_result_s;
 typedef struct cdk_verify_result_s *_cdk_verify_result_t;
 
+
+/* kludge until we support this in the lib */
+typedef GCRY_MPI gcry_mpi_t;
+typedef GCRY_SEXP gcry_sexp_t;
+
 #endif /* CDK_TYPES_H */
