@@ -1,5 +1,4 @@
-/* -*- Mode: C; c-file-style: "bsd" -*-
- * verify.c - Verify signatures
+/* verify.c - Verify signatures
  *        Copyright (C) 2001, 2002, 2003 Timo Schulz 
  *
  * This file is part of OpenCDK.
@@ -141,16 +140,16 @@ cdk_sig_get_ulong_attr( cdk_ctx_t hd, int idx, int what )
     assert( idx == 0 );
     res = hd->result.verify;
     switch( what ) {
-    case CDK_ATTR_CREATED: val = res->created; break;
-    case CDK_ATTR_EXPIRE : val = res->expires; break;
-    case CDK_ATTR_KEYID  : val = res->keyid[1]; break;
-    case CDK_ATTR_STATUS : val = res->sig_status; break;
-    case CDK_ATTR_ALGO_PK: val = res->pubkey_algo; break;
-    case CDK_ATTR_ALGO_MD: val = res->digest_algo; break;
-    case CDK_ATTR_VERSION: val = res->sig_ver; break;
-    case CDK_ATTR_LEN    : val = res->sig_len; break;
-    case CDK_ATTR_FLAGS  : val = res->sig_flags; break;
-    default              : val = 0; break;
+      case CDK_ATTR_CREATED: val = res->created; break;
+      case CDK_ATTR_EXPIRE : val = res->expires; break;
+      case CDK_ATTR_KEYID  : val = res->keyid[1]; break;
+      case CDK_ATTR_STATUS : val = res->sig_status; break;
+      case CDK_ATTR_ALGO_PK: val = res->pubkey_algo; break;
+      case CDK_ATTR_ALGO_MD: val = res->digest_algo; break;
+      case CDK_ATTR_VERSION: val = res->sig_ver; break;
+      case CDK_ATTR_LEN    : val = res->sig_len; break;
+      case CDK_ATTR_FLAGS  : val = res->sig_flags; break;
+      default              : val = 0; break;
     }
     
     return val;
@@ -178,10 +177,12 @@ cdk_sig_get_data_attr( cdk_ctx_t hd, int idx, int what )
     assert( idx == 0 );
     res = hd->result.verify;
     switch( what ) {
-    case CDK_ATTR_KEYID   : val = res->keyid; break;
-    case CDK_ATTR_NOTATION: val = res->notation; break;
-    case CDK_ATTR_MPI     : val = res->sig_data; break;
-    default               : val = NULL;
+      case CDK_ATTR_NAME    : val = res->user_id; break;
+      case CDK_ATTR_KEYID   : val = res->keyid; break;
+      case CDK_ATTR_NOTATION: val = res->notation; break;
+      case CDK_ATTR_URL     : val = res->policy_url; break;
+      case CDK_ATTR_MPI     : val = res->sig_data; break;
+      default               : val = NULL;
     }
 
     return val;
@@ -246,11 +247,9 @@ file_verify_clearsign( cdk_ctx_t hd, const char * file, const char * output )
     }
     if( !digest_algo )
         digest_algo = CDK_MD_MD5;
-    md = cdk_md_open( digest_algo, 0 );
-    if( !md ) {
-        rc = CDK_Inv_Algo;
+    rc = cdk_md_open( &md, digest_algo, 0 );
+    if( rc )
         goto leave;
-    }
 
     s = "-----BEGIN PGP SIGNATURE-----";
     while( !cdk_stream_eof( inp ) ) {
