@@ -64,7 +64,8 @@ enum
   oArmor = 'p',
   oCompat = 'g',
   oNoCompress = '0',
-  oDetachSign = 'j',  
+  oDetachSign = 'j',
+  oBlockMode = '+'
 };
 
 
@@ -73,17 +74,18 @@ static struct
   const char * name;
   int algid;
 } algo_table[] = {
-  {"3DES", CDK_CIPHER_3DES},
-  {"CAST5", CDK_CIPHER_CAST5},
+  {    "3DES", CDK_CIPHER_3DES},
+  {   "CAST5", CDK_CIPHER_CAST5},
   {"BLOWFISH", CDK_CIPHER_BLOWFISH},
-  {"TWOFISH", CDK_CIPHER_TWOFISH},
-  {"AES", CDK_CIPHER_AES},
-  {"AES192", CDK_CIPHER_AES192},
-  {"AES256", CDK_CIPHER_AES256},
-  {"MD5", CDK_MD_MD5},
-  {"SHA1", CDK_MD_SHA1},
+  { "TWOFISH", CDK_CIPHER_TWOFISH},
+  {     "AES", CDK_CIPHER_AES},
+  {  "AES192", CDK_CIPHER_AES192},
+  {  "AES256", CDK_CIPHER_AES256},
+  {   "dummy", -1},
+  {      "MD5", CDK_MD_MD5},
+  {     "SHA1", CDK_MD_SHA1},
   {"RIPEMD160", CDK_MD_RMD160},
-  {"SHA256", CDK_MD_SHA256},
+  {   "SHA256", CDK_MD_SHA256},
   {NULL},
 };
 
@@ -94,13 +96,15 @@ char * cipher_algo = "(none)";
 char * digest_algo = "(none)";
 char * ks_keyid = "(none)";
 char * compress_level = "(none)";
-int cipher_algid = 0;
-int digest_algid = 0;
-int use_outname = 0;
-
+static int cipher_algid = 0;
+static int digest_algid = 0;
+static int use_outname = 0;
+static int verbose = 0;
 
 struct poptOption arguments[] =
 {
+  {"block-mode", oBlockMode, POPT_ARG_NONE, 0, oBlockMode,
+   "force partial block mode", 0},
   {"show-prefs", cShowpref, POPT_ARG_STRING, &user, cShowpref,
    "show the preferences of a key", 0},
   {"detach-sign", oDetachSign, POPT_ARG_NONE, 0, oDetachSign,
@@ -155,6 +159,8 @@ struct poptOption arguments[] =
   {NULL}
 };
 
+static const char * get_sig_algo (int pubkey_algo);
+     
 
 static void
 out_of_core (void)
@@ -220,9 +226,9 @@ do_remove_ext (const char * filename)
 
 
 static char
-key_get_letter (int pubkey_algo)
+key_get_letter (int pkalgo)
 {
-  switch (pubkey_algo)
+  switch (pkalgo)
     {
     case CDK_PK_RSA:   return 'R';
     case CDK_PK_RSA_E: return 'r';
@@ -250,21 +256,18 @@ strdate_from_long (long timestamp)
 
 
 static char
-sig_get_letter (cdkPKT_signature * sig)
+sig_get_letter (cdk_pkt_signature_t sig)
 {
   if (sig->flags.missing_key)
     return '%';
   if (sig->flags.checked)
-    {
-      if (sig->flags.valid == 1)
-	return sig->flags.valid ? '!' : '-';
-    }
+    return sig->flags.valid == 1? '!' : '-';
   return '?';
 }
 
 
 static char
-sig_get_status (cdkPKT_signature * sig)
+sig_get_status (cdk_pkt_signature_t sig)
 {
   if (!sig->flags.exportable)
     return 'L';
@@ -294,7 +297,7 @@ algo_get_id (const char *algo, int what)
 
 
 static char *
-get_keyvalid_char (cdkPKT_public_key * pk)
+get_keyvalid_char (cdk_pkt_pubkey_t pk)
 {
   static char flags[2];
   memset (flags, 0, 2);
@@ -317,9 +320,9 @@ get_keyvalid_char (cdkPKT_public_key * pk)
 }
 
 static void
-do_show_key (CDK_KBNODE pk, int with_sigs)
+do_show_key (cdk_kbnode_t pk, cdk_keydb_hd_t hd, int with_sigs)
 {
-  CDK_KBNODE p, ctx = NULL;
+  cdk_kbnode_t p, ctx = NULL;
   CDK_PACKET * pkt;
   
   while ((p = cdk_kbnode_walk (pk, &ctx, 0)))
@@ -327,7 +330,7 @@ do_show_key (CDK_KBNODE pk, int with_sigs)
       pkt = cdk_kbnode_get_packet (p);
       if (pkt->pkttype == CDK_PKT_PUBLIC_KEY)
 	{
-	  cdkPKT_public_key * key = pkt->pkt.public_key;
+	  cdk_pkt_pubkey_t key = pkt->pkt.public_key;
 	  fprintf (stdout, "pub[%s]  %04d%c/%08lX %s",
                    get_keyvalid_char (key),
 		   cdk_pk_get_nbits (key),
@@ -339,10 +342,30 @@ do_show_key (CDK_KBNODE pk, int with_sigs)
           else
             fprintf (stdout, "  [expires: %s]\n",
                      strdate_from_long (key->expiredate));
+          if (key->revkeys)
+            {
+              cdk_desig_revoker_t ctx = NULL;
+              const unsigned char *p;
+              int algid, class, i;
+              char hexbuf[8];
+
+              while ((p = cdk_key_desig_revoker_walk
+                      (key->revkeys, &ctx, &class, &algid)))
+                {
+                  printf ("revkey    %s class 0x%X fingerprint ",
+                          get_sig_algo (algid), class);
+                  for (i = 0; i < 20; i++)
+                    {
+                      sprintf (hexbuf, "%X", p[i]);
+                      fputs (hexbuf, stdout);
+                    }
+                  fputc ('\n', stdout);
+                }
+            }
 	}
       else if (pkt->pkttype == CDK_PKT_PUBLIC_SUBKEY)
 	{
-	  cdkPKT_public_key * key = pkt->pkt.public_key;
+	  cdk_pkt_pubkey_t key = pkt->pkt.public_key;
 	  fprintf (stdout, "sub[%s]  %04d%c/%08lX %s",
                    get_keyvalid_char (key),
 		   cdk_pk_get_nbits (key),
@@ -357,7 +380,7 @@ do_show_key (CDK_KBNODE pk, int with_sigs)
 	}
       else if (pkt->pkttype == CDK_PKT_USER_ID)
 	{
-	  cdkPKT_user_id * id = pkt->pkt.user_id;
+	  cdk_pkt_userid_t id = pkt->pkt.user_id;
           char * uid = cdk_utf8_decode (id->name, id->len, 0);
           fprintf (stdout, "uid %s %s\n",
                    id->is_revoked ? "[revoked]" : "         ", uid);
@@ -365,21 +388,33 @@ do_show_key (CDK_KBNODE pk, int with_sigs)
 	}
       else if (with_sigs && pkt->pkttype == CDK_PKT_SIGNATURE)
 	{
-	  cdkPKT_signature * sig = pkt->pkt.signature;
-	  fprintf (stdout, " sig%c %c %08lX %s\n",
+	  cdk_pkt_signature_t sig = pkt->pkt.signature;
+          cdk_packet_t pkt;
+          cdk_kbnode_t pubk;
+
+	  fprintf (stdout, " sig%c %c %08lX %s",
 		   sig_get_letter (sig),
 		   sig_get_status (sig),
 		   (unsigned long)cdk_sig_get_keyid (sig, NULL),
 		   strdate_from_long (sig->timestamp));
+          if (!cdk_keydb_get_bykeyid (hd, sig->keyid, &pubk))
+            {
+              pkt = cdk_kbnode_find_packet (pubk, CDK_PKT_USER_ID);
+              printf (" %s\n", pkt?
+                      pkt->pkt.user_id->name : "[user-ID not found]");
+              cdk_kbnode_release (pubk);
+            }
+          else
+            printf (" [user-ID not found]\n");
 	}
     }
 }
 
 
 static int
-check_for_wildcards (CDK_STRLIST list)
+check_for_wildcards (cdk_strlist_t list)
 {
-  CDK_STRLIST ctx = NULL;
+  cdk_strlist_t ctx = NULL;
   const char * s = cdk_strlist_walk (list, &ctx);
   if (*s == '*')
     return 1;
@@ -388,69 +423,69 @@ check_for_wildcards (CDK_STRLIST list)
 
 
 int
-do_list_keys (CDK_KEYDB_HD hd, CDK_STRLIST keys)
+do_list_keys (cdk_keydb_hd_t hd, cdk_strlist_t keys)
 {
-  const char * s;
-  unsigned char fpr[20];
-  CDK_PACKET * pkt;
-  CDK_STREAM inp;
-  CDK_KBNODE pk = NULL;
-  CDK_STRLIST ctx = NULL;
+  unsigned char *fpr;
+  cdk_kbnode_t pk = NULL;
+  cdk_listkey_t list;
   int rc = 0, i;
 
   if (check_for_wildcards (keys))
     keys = NULL;
   if (keys)
     {
-      while ((s = cdk_strlist_walk (keys, &ctx)))
+      rc = cdk_listkey_start (&list, hd, NULL, keys);
+      if (rc)
         {
-	  rc = cdk_keydb_get_bypattern (hd, s, &pk);
-	  if (rc)
-	    fprintf (stdout, "get key `%s': %s\n", s, cdk_strerror (rc));
-	  if (!rc)
+          fprintf (stderr, "listkey start: %s\n", cdk_strerror (rc));
+          return rc;
+        }
+      while (!(rc = cdk_listkey_next (list, &pk)))
+        {
+          do_show_key (pk, hd, 0);
+          fpr = cdk_kbnode_get_attr (pk, CDK_PKT_PUBLIC_KEY, CDK_ATTR_FPR);
+          if (fpr)
             {
-              do_show_key (pk, 0);
-              pkt = cdk_kbnode_get_packet (pk);
               printf ("Fingerprint ");
-              cdk_pk_get_fingerprint (pkt->pkt.public_key, fpr);
+              i = (int)cdk_kbnode_get_attr (pk, CDK_PKT_PUBLIC_KEY, CDK_ATTR_VERSION);
+              printf ("v%d ", i);
               for (i = 0; i < 20/2; i++)
                 printf ("%02X%02X ", fpr[2*i], fpr[2*i+1]);
               printf ("\n");
             }
-              
+          
 	  cdk_kbnode_release (pk);
 	  pk = NULL;
 	}
+      cdk_listkey_close (list);
       return 0;
     }
   /* Show all keys */
   cdk_set_log_level (CDK_LOG_DEBUG);
-  
-  rc = cdk_keydb_open (hd, &inp);
+
+  rc = cdk_listkey_start (&list, hd, "*", NULL);
   if (rc)
     {
-      fprintf (stderr, "keydb open: %s\n", cdk_strerror (rc));
+      fprintf (stderr, "listkey start: %s\n", cdk_strerror (rc));
       return rc;
     }
 
-  while (!(rc = cdk_keydb_get_keyblock (inp, &pk)))
+  while (!(rc = cdk_listkey_next (list, &pk)))
     {
-      do_show_key (pk, 0);
+      do_show_key (pk, hd, 0);
       fputc ('\n', stdout);
       cdk_kbnode_release (pk);
       pk = NULL;
-      rc = cdk_stream_eof (inp);
-      if (rc)
-        break;
     }
+  cdk_listkey_close (list);
   return 0;
 }
 
 
 int
-do_export_keys (CDK_HD ctx, CDK_KEYDB_HD hd, CDK_STRLIST keys, int secret)
+do_export_keys (cdk_ctx_t ctx, cdk_keydb_hd_t hd, cdk_strlist_t keys, int secret)
 {
-  CDK_STREAM outp = NULL;
+  cdk_stream_t outp = NULL;
   int rc = 0;
 
   if (!use_outname)
@@ -479,25 +514,31 @@ do_export_keys (CDK_HD ctx, CDK_KEYDB_HD hd, CDK_STRLIST keys, int secret)
 
 
 int
-do_import_keys (CDK_KEYDB_HD hd, const char * keyfile)
+do_import_keys (cdk_keydb_hd_t pubdb, cdk_keydb_hd_t secdb,
+                const char * keyfile)
 {
-  CDK_KBNODE node = NULL, n, ctx = NULL;
-  CDK_STREAM s;
+  cdk_keydb_hd_t db;
+  cdk_kbnode_t node = NULL, n, ctx = NULL;
+  cdk_stream_t s;
   int rc, res[4];
 
-  if (!hd)
+  if (!pubdb)
     {
       fprintf (stderr, "no public keyring available.\n");
       return -1;
     }
   rc = cdk_stream_open (keyfile, &s);
-  if (!s)
+  if (rc)
     {
       fprintf (stderr, "open `%s' failed: %s\n", keyfile, cdk_strerror (rc));
       return -1;
     }
   if (cdk_armor_filter_use (s))
-    cdk_stream_set_armor_flag (s, 0);
+    {
+      if (verbose)
+        fprintf (stderr, "push armor filter\n");
+      cdk_stream_set_armor_flag (s, 0);
+    }
   rc = cdk_keydb_get_keyblock (s, &node);
   if (!node)
     {
@@ -505,7 +546,8 @@ do_import_keys (CDK_KEYDB_HD hd, const char * keyfile)
       fprintf (stderr, "get keyblock failed: %s\n", cdk_strerror (rc));
       return -1;
     }
-  rc = cdk_keydb_import (hd, node, res);
+  db = cdk_kbnode_find (node, CDK_PKT_SECRET_KEY)? secdb : pubdb;
+  rc = cdk_keydb_import (db, node, res);
   if (rc)
     fprintf (stderr, "import failed: %s\n", cdk_strerror (rc));
   else
@@ -534,10 +576,10 @@ do_import_keys (CDK_KEYDB_HD hd, const char * keyfile)
   
 
 int
-do_list_sigs (CDK_KEYDB_HD hd, CDK_STRLIST keys)
+do_list_sigs (cdk_keydb_hd_t hd, cdk_strlist_t keys)
 {
-  CDK_KBNODE pk = NULL;
-  CDK_STRLIST ctx = NULL;
+  cdk_kbnode_t pk = NULL;
+  cdk_strlist_t ctx = NULL;
   const char * s;
   int kstat = 0;
   int rc = 0;
@@ -549,10 +591,10 @@ do_list_sigs (CDK_KEYDB_HD hd, CDK_STRLIST keys)
 	  rc = cdk_keydb_get_bypattern (hd, s, &pk);
 	  if (rc)
 	    fprintf (stdout, "get key `%s': %s\n", s, cdk_strerror (rc));
-	  rc = cdk_key_check_sigs (pk, hd, &kstat);
+	  rc = cdk_pk_check_sigs (pk, hd, &kstat);
 	  if (rc)
 	    fprintf (stdout, "check sigs `%s': %s\n", s, cdk_strerror (rc));
-	  do_show_key (pk, 1);
+	  do_show_key (pk, hd, 1);
 	}
     }
 
@@ -584,8 +626,8 @@ ks_fetch_key (const char * kid)
 {
   unsigned long keyid;
   unsigned char kidbuf[4];
-  CDK_KBNODE pk = NULL;
-  CDK_PACKET * pkt;
+  const char * s;
+  cdk_kbnode_t pk = NULL;
   int rc;
   
   if (!strncmp (kid, "0x", 2))
@@ -612,9 +654,9 @@ ks_fetch_key (const char * kid)
       printf ("%s\n", cdk_strerror (rc));
   else
       printf ("OK\n");
-  pkt = cdk_kbnode_find_packet (pk, CDK_PKT_USER_ID);
-  if (pkt)
-    printf ("%s\n", pkt->pkt.user_id->name);
+  s = cdk_kbnode_get_attr (pk, CDK_PKT_USER_ID, CDK_ATTR_NAME);
+  if (s)
+    printf ("%s\n", s);
 
   cdk_kbnode_release (pk);
   return 0;
@@ -685,9 +727,10 @@ print_mpibuf (const unsigned char * buf, int pubkey_algo)
       
 
 void
-print_verify_stats (CDK_HD hd, int extflag)
+print_verify_stats (cdk_ctx_t hd, int extflag)
 {
   const unsigned long * keyid;
+  const char * s;
   unsigned long created;
   int sigstat, pkalgo;
 
@@ -697,11 +740,19 @@ print_verify_stats (CDK_HD hd, int extflag)
       keyid = cdk_sig_get_data_attr (hd, 0, CDK_ATTR_KEYID);
       created = cdk_sig_get_ulong_attr (hd, 0, CDK_ATTR_CREATED);
       pkalgo = cdk_sig_get_ulong_attr (hd, 0, CDK_ATTR_ALGO_PK);
-      printf ("(%s) keyid %08lX%08lX algo %s created %s\n",
-              get_sig_status (sigstat), keyid[0], keyid[1],
-              get_sig_algo (pkalgo), get_sig_time (created));
+      s = cdk_sig_get_data_attr (hd, 0, CDK_ATTR_NAME);
+      printf ("%s keyid 0x%08lX algo %s created %s\n\t\"%s\"\n",
+              get_sig_status (sigstat), keyid[1],
+              get_sig_algo (pkalgo), get_sig_time (created),
+              s? s : "");
       if (extflag)
         print_mpibuf (cdk_sig_get_data_attr (hd, 0, CDK_ATTR_MPI), pkalgo);
+      s = cdk_sig_get_data_attr (hd, 0, CDK_ATTR_NOTATION);
+      if (s)
+        printf ("Notation: %s\n", s);
+      s = cdk_sig_get_data_attr (hd, 0, CDK_ATTR_URL);
+      if (s)
+        printf ("Policy-URL: %s\n", s);
     }
 }
 
@@ -711,7 +762,7 @@ genkey (void)
 {
   int c, algo = 0, rc =0;
   unsigned char mdbuf[3] = {CDK_MD_MD5, CDK_MD_RMD160, CDK_MD_SHA1};
-  CDK_KEYGEN_CTX hd;
+  cdk_keygen_ctx_t hd;
   
   printf ("(1) DSA (sign only)\n");
   printf ("(2) DSA and ElGamal\n");
@@ -744,24 +795,29 @@ genkey (void)
     }
   cdk_keygen_set_name (hd, "Otto Bismarck");
   cdk_keygen_set_mdc_feature (hd, 1);
+  cdk_keygen_set_keyserver_flags (hd, 1, "http://horowitz.surfnet.nl");
   cdk_keygen_set_expire_date (hd, 0, 864000);
   cdk_keygen_set_prefs (hd, CDK_PREFTYPE_HASH, mdbuf, 3);
-  cdk_keygen_set_passphrase (hd, "abc");
+  //cdk_keygen_set_passphrase (hd, "abc");
   rc = cdk_keygen_start (hd);
+  printf ("rc_start=%d\n", rc);
   if (!rc)
-    rc = cdk_keygen_save (hd, "pub.test", "sec.test");
+    {
+      rc = cdk_keygen_save (hd, "pub.test", "sec.test");
+      printf ("rc_save=%d\n", rc);
+    }
   cdk_keygen_free (hd);
   return rc;
 }
 
 
 static void
-show_one_array (cdkPKT_user_id * id, int type)
+show_one_array (cdk_pkt_userid_t id, int type)
 {
   size_t n = 0, i = 0;
   unsigned char * p;
 
-  p = cdk_prefs_get_array (id, type, &n);
+  p = cdk_userid_pref_get_array (id, type, &n);
   if (p)
     {
       for (i = 0; i < n; i++)
@@ -780,9 +836,9 @@ show_one_array (cdkPKT_user_id * id, int type)
 
 
 static void
-show_prefs (const char * name, CDK_KEYDB_HD db)
+show_prefs (const char * name, cdk_keydb_hd_t db)
 {
-  CDK_KBNODE knode = NULL;
+  cdk_kbnode_t knode = NULL;
   CDK_PACKET * pkt;
   int rc;
   
@@ -818,16 +874,18 @@ file_error (const char * fil, const char * fnc, int rc)
 int
 main (int argc, char ** argv)
 {
-  CDK_HD hd;
-  CDK_STRLIST remusr = NULL, klist = NULL;
-  CDK_KEYDB_HD sec_db, pub_db;
+  cdk_ctx_t hd;
+  cdk_strlist_t remusr = NULL, klist = NULL;
+  cdk_keydb_hd_t sec_db, pub_db;
   poptContext opt_ctx;
   char * output = NULL, * s;
   char u[128];
-  int c = 0, verbose = 0, i = 0, sigmode = 0, zip_level = 6;
+  int c = 0, i = 0, sigmode = 0, zip_level = 6;
+  int blockmode = 0;
   int rc = 0, nfiles = 0;
 
   s = "/home/twoaday/.gnupg/pubring.cdk";
+  //s = "pub.test";
   /*s = "pubring.gpg";*/
   rc = cdk_keydb_new (&pub_db, CDK_DBTYPE_PK_KEYRING, s, strlen (s));
   if (rc)
@@ -836,7 +894,8 @@ main (int argc, char ** argv)
       return 0;
     }
   s = "sec-with-pwd.gpg";
-  /*s = "secring.gpg";*/
+  //s = "secring.gpg";
+  //s = "sec.test";
   rc = cdk_keydb_new (&sec_db, CDK_DBTYPE_SK_KEYRING, s, strlen (s));
   if (rc)
     {
@@ -857,6 +916,12 @@ main (int argc, char ** argv)
     {
       switch (c)
 	{
+        case '+':
+          if (verbose)
+            printf ("enable block-mode\n");
+          blockmode++;
+          break;
+          
 	case 'v':
 	  verbose++;
 	  break;
@@ -893,9 +958,13 @@ main (int argc, char ** argv)
 	case 'V':
 	  fprintf (stderr, "Minimal Privacy Guard (minpg) %s\n\n", OPENCDK_VERSION);
 	  fprintf (stderr, "Supported algorithms:\n");
-	  fprintf (stderr, "Cipher, Digest: ");
-	  for (i = 0; algo_table[i].name; i++)
+	  fprintf (stderr, "Cipher: ");
+	  for (i = 0; algo_table[i].name && algo_table[i].algid != -1; i++)
 	    fprintf (stderr, "%s ", algo_table[i].name);
+          i++;
+          fprintf (stderr, "\nDigest: ");
+          for (; algo_table[i].name; i++)
+            fprintf (stderr, "%s ", algo_table[i].name);
 	  fprintf (stderr, "\n");
 	  return 0;
 
@@ -922,7 +991,8 @@ main (int argc, char ** argv)
 	  break;
 
         case 'y':
-          cdk_set_log_level (CDK_LOG_DEBUG);
+          if (verbose > 1)
+            cdk_set_log_level (CDK_LOG_DEBUG);
           cdk_handle_set_keydb (hd, pub_db);
           rc = cdk_file_verify (hd, file, NULL);
           if (rc)
@@ -932,11 +1002,12 @@ main (int argc, char ** argv)
           break;
           
 	case 'u':
-          /*cdk_set_log_level (CDK_LOG_DEBUG);*/
+          if (verbose > 1)
+            cdk_set_log_level (CDK_LOG_DEBUG);
 	  output = cdk_calloc (1, strlen (file) + 1);
           if (!output)
             out_of_core ();
-	  if (strstr (file, ".asc"))
+	  if (strstr (file, ".asc") || strstr (file, ".gpg"))
 	    strncpy (output, file, strlen (file) - 4);
 	  else
 	    strcpy (output, file);
@@ -948,7 +1019,8 @@ main (int argc, char ** argv)
 	  break;
 
 	case 'l':
-          /*cdk_set_log_level (CDK_LOG_DEBUG);*/
+          if (verbose > 1)
+            cdk_set_log_level (CDK_LOG_DEBUG);
 	  cdk_strlist_add (&klist, user);
 	  do_list_sigs (pub_db, klist);
 	  break;
@@ -975,7 +1047,10 @@ main (int argc, char ** argv)
 	  break;
 
 	case 'c':
-          /*cdk_set_log_level (CDK_LOG_DEBUG);*/
+          if (blockmode)
+            cdk_handle_set_blockmode (hd, 1);
+          if (verbose > 1)
+            cdk_set_log_level (CDK_LOG_DEBUG);
 	  output = do_add_ext (file, ".gpg");
 	  if (verbose)
 	    fprintf (stdout, "symencrypt `%s' -> %s\n", file, output);
@@ -1008,7 +1083,9 @@ main (int argc, char ** argv)
 	  break;
 
         case 'i':
-          do_import_keys (pub_db, file);
+          if (verbose > 1)
+            cdk_set_log_level (CDK_LOG_DEBUG);
+          do_import_keys (pub_db, sec_db, file);
           break;
 
         case 'f':
@@ -1028,7 +1105,10 @@ main (int argc, char ** argv)
 	      fgets (u, sizeof u, stdin);
 	      cdk_strlist_add (&remusr, user);
 	    }
-          /*cdk_set_log_level (CDK_LOG_DEBUG);*/
+          if (verbose > 1)
+            cdk_set_log_level (CDK_LOG_DEBUG);
+          if (blockmode)
+            cdk_handle_set_blockmode (hd, 1);
 	  output = do_add_ext (file, ".gpg");
 	  if (verbose)
 	    fprintf (stdout, "encrypt `%s' -> %s\n", file, output);
@@ -1042,7 +1122,8 @@ main (int argc, char ** argv)
 
 	case 'd':
         case '$':
-          cdk_set_log_level (CDK_LOG_DEBUG);
+          if (verbose > 1)
+            cdk_set_log_level (CDK_LOG_DEBUG);
 	  output = do_remove_ext (file);
 	  if (verbose)
 	    fprintf (stdout, "decrypt `%s' -> %s\n", file, output);
