@@ -24,6 +24,7 @@
 #include <errno.h>
 #include <time.h>
 #include <stdarg.h>
+#include <stdlib.h>
 #include <assert.h>
 
 #include "opencdk.h"
@@ -38,8 +39,8 @@ int
 stream_test_filters (int literal, int compress, int armor, int encrypt)
 {
   char buf[128];
-  CDK_STREAM s;
-  CDK_DEK dek;
+  cdk_stream_t s;
+  cdk_dek_t dek;
   int rc, i;
 
   rc = cdk_stream_new ("test.asc", &s);
@@ -59,7 +60,7 @@ stream_test_filters (int literal, int compress, int armor, int encrypt)
           cdk_stream_close (s);
           return rc;
         }
-      cdk_dek_set_cipher( dek, GCRY_CIPHER_TWOFISH );
+      cdk_dek_set_cipher( dek, CDK_CIPHER_TWOFISH );
       cdk_stream_set_cipher_flag (s, dek, 0);
     }
   if (compress)
@@ -67,7 +68,7 @@ stream_test_filters (int literal, int compress, int armor, int encrypt)
   if (literal)
     cdk_stream_set_literal_flag (s, 0, "test.asc");
 
-  cdk_stream_set_cache (s, 1);
+  cdk_stream_switch_cache (s, 1);
   /* write a marker packet we won't filter! */
   buf[0] = (0x80 | (10<<2));
   buf[1] = 3;
@@ -75,7 +76,7 @@ stream_test_filters (int literal, int compress, int armor, int encrypt)
   buf[3] = 0x47;
   buf[4] = 0x50;
   cdk_stream_write (s, buf, 5);
-  cdk_stream_set_cache (s, 0);
+  cdk_stream_switch_cache (s, 0);
 
   for (i = 0; i < 512; i++)
     {
@@ -131,7 +132,7 @@ read_ascii_data (const char *file, int *r_nbytes)
 int
 stream_test_filters_tmp_ascii (void)
 {
-  CDK_STREAM s;
+  cdk_stream_t s;
   char *pk;
   size_t nbytes;
   int c, rc;
@@ -171,7 +172,7 @@ stream_test_filters_tmp_ascii (void)
 int
 stream_test_filters_tmp_both (void)
 {
-  CDK_STREAM s;
+  cdk_stream_t s;
   char *p;
   size_t nbytes;
   int rc, c;
@@ -207,7 +208,7 @@ int
 stream_encrypt (char *file)
 {
   int rc;
-  CDK_HD hd;
+  cdk_ctx_t hd;
 
   cdk_handle_new (&hd);
   rc = cdk_file_encrypt (hd, NULL, file, "out.gpg");
@@ -222,9 +223,9 @@ int
 stream_encrypt_pk (char *user, char *file)
 {
   int rc;
-  CDK_HD hd;
-  CDK_STRLIST recp = NULL;
-  CDK_KEYDB_HD db;
+  cdk_ctx_t hd;
+  cdk_strlist_t recp = NULL;
+  cdk_keydb_hd_t db;
   
   cdk_handle_new (&hd);
   cdk_keydb_new (&db, CDK_DBTYPE_PK_KEYRING, "ts.gpg", 6);
@@ -243,8 +244,8 @@ stream_encrypt_pk (char *user, char *file)
 int
 stream_trustdb (char *db)
 {
-  CDK_STREAM s;
-  cdkPKT_user_id *u;
+  cdk_stream_t s;
+  cdk_pkt_userid_t u;
   int rc, valid;
 
   rc = cdk_stream_open (db, &s);
@@ -300,12 +301,12 @@ get_sig_algo (int pubkey_algo)
 {
   switch (pubkey_algo)
     {
-    case GCRY_PK_RSA_E:
-    case GCRY_PK_RSA_S:
-    case GCRY_PK_RSA  : return "RSA";
-    case GCRY_PK_ELG  :
-    case GCRY_PK_ELG_E: return "ELG";
-    case GCRY_PK_DSA  : return "DSA";
+    case CDK_PK_RSA_E:
+    case CDK_PK_RSA_S:
+    case CDK_PK_RSA  : return "RSA";
+    case CDK_PK_ELG  :
+    case CDK_PK_ELG_E: return "ELG";
+    case CDK_PK_DSA  : return "DSA";
     default           : return "???";
     }
   return NULL;
@@ -317,8 +318,8 @@ stream_decrypt (char *file, int with_pass)
 {
   int rc, sigstat, created, pkalgo;
   const unsigned long * keyid;
-  CDK_HD hd;
-  CDK_KEYDB_HD db, p_db;
+  cdk_ctx_t hd;
+  cdk_keydb_hd_t db, p_db;
 
   if (!with_pass)
     cdk_keydb_new (&db, CDK_DBTYPE_SK_KEYRING, "sec.gpg", 7);
@@ -349,8 +350,8 @@ stream_decrypt (char *file, int with_pass)
 int
 stream_hash (char * file, char * algo)
 {
-  md_filter_s * mfx;
-  CDK_STREAM s;
+  md_filter_t * mfx;
+  cdk_stream_t s;
   int rc;
 
   rc = cdk_stream_open (file, &s);
@@ -365,12 +366,12 @@ stream_hash (char * file, char * algo)
   printf ("hash %s...\n", file);
   if (mfx && mfx->md)
     {
-      unsigned char *md = gcry_md_read (mfx->md, 0);
-      size_t dlen = gcry_md_get_algo_dlen (mfx->digest_algo), i;
+      unsigned char *md = cdk_md_read (mfx->md, 0);
+      size_t dlen = cdk_md_get_algo_dlen (mfx->digest_algo), i;
       for (i = 0; i < dlen; i++)
         printf ("%02X ", md[i]);
       printf ("\n");
-      gcry_md_close (mfx->md);
+      cdk_md_close (mfx->md);
     }
   cdk_stream_close (s);
   return 0;
@@ -379,7 +380,7 @@ stream_hash (char * file, char * algo)
 int
 stream_test_armor (char *file)
 {
-  CDK_STREAM s;
+  cdk_stream_t s;
   int rc;
     
   rc = cdk_stream_open (file, &s);
@@ -397,8 +398,8 @@ stream_test_armor (char *file)
 int
 stream_test_keyblock (char *file)
 {
-  CDK_STREAM s;
-  CDK_KBNODE knode;
+  cdk_stream_t s;
+  cdk_kbnode_t knode;
   int rc;
 
   rc = cdk_stream_open (file, &s);
@@ -421,8 +422,8 @@ stream_test_keyblock (char *file)
 static int
 stream_verify (char *file)
 {
-  CDK_HD hd;
-  CDK_KEYDB_HD db;
+  cdk_ctx_t hd;
+  cdk_keydb_hd_t db;
   int rc;
   
   cdk_handle_new (&hd);
@@ -439,17 +440,19 @@ stream_verify (char *file)
 static int
 stream_clearsign (char *file)
 {
-  CDK_HD hd;
-  CDK_STRLIST locusr = NULL;
-  CDK_KEYDB_HD db;
+  cdk_ctx_t hd;
+  cdk_strlist_t locusr = NULL;
+  cdk_keydb_hd_t db;
   int rc ;
   
   cdk_handle_new (&hd);
   cdk_keydb_new (&db, CDK_DBTYPE_SK_KEYRING, "sec.gpg", 7);
   cdk_strlist_add (&locusr, "twoaday@daredevil");
-      //rc = cdk_file_clearsign (hd, locusr, file, "out.asc");
+/*
+  rc = cdk_file_clearsign (hd, locusr, file, "out.asc");
   if (rc)
     printf ("clearsign_file %s: `%s'\n", file, cdk_strerror (rc));
+*/
   cdk_handle_free (hd);
   cdk_strlist_free (locusr);
   return 0;
@@ -459,7 +462,7 @@ stream_clearsign (char *file)
 int
 stream_test_textmode (void)
 {
-  CDK_STREAM s;
+  cdk_stream_t s;
   int rc;
 
   rc = cdk_stream_new ("literal.txm", &s);
@@ -480,8 +483,8 @@ stream_test_textmode (void)
 int
 test_stream_kbnode (const char * file)
 {
-  CDK_KBNODE node = NULL, n, ctx = NULL;
-  CDK_STREAM inp;
+  cdk_kbnode_t node = NULL, n, ctx = NULL;
+  cdk_stream_t inp;
   unsigned char buf[8192];
   int rc, nread;
 
@@ -545,7 +548,7 @@ int
 main (int argc, char **argv)
 {
   cdk_set_log_level (CDK_LOG_DEBUG);
-  return test_stream_kbnode (argc>1? argv[1] : "foo");
+  /*return test_stream_kbnode (argc>1? argv[1] : "foo");*/
   /*stream_test_filters (1, 1, 1, 1);*/
   /*stream_test_filters_tmp_ascii ();*/
   /*stream_test_filters_tmp_both ();*/
@@ -575,9 +578,9 @@ main (int argc, char **argv)
 int
 test_sign (int argc, char ** argv)
 {
-    CDK_HD hd;
-    CDK_KEYDB_HD db;
-    CDK_STRLIST locusr = NULL;
+    cdk_ctx_t hd;
+    cdk_keydb_hd_t db;
+    cdk_strlist_t locusr = NULL;
     char * s = "/home/twoaday/cvs/gnupg.org/opencdk/tests/sec-with-pwd.gpg";
     const char * test = "Hello World!";
     unsigned char * out = NULL;
@@ -609,9 +612,9 @@ test_sign (int argc, char ** argv)
 int
 test_encrypt (int argc, char ** argv)
 {
-    CDK_HD hd;
-    CDK_KEYDB_HD db;
-    CDK_STRLIST remusr = NULL;
+    cdk_ctx_t hd;
+    cdk_keydb_hd_t db;
+    cdk_strlist_t remusr = NULL;
     char * s = "/home/twoaday/.gnupg/pubring.gpg";
     const char *test = "Hello World!";
     unsigned char * out = NULL;
@@ -644,8 +647,8 @@ test_encrypt (int argc, char ** argv)
 int
 test_verify (int argc, char ** argv)
 {
-    CDK_HD hd;
-    CDK_KEYDB_HD db;
+    cdk_ctx_t hd;
+    cdk_keydb_hd_t db;
     int rc;
     char * s = "/home/twoaday/.gnupg/pubring.cdk";
     const char * st =
@@ -683,8 +686,8 @@ test_verify (int argc, char ** argv)
 int
 test_import (int argc, char ** argv)
 {
-    CDK_HD hd;
-    CDK_KEYDB_HD db;
+    cdk_ctx_t hd;
+    cdk_keydb_hd_t db;
     int rc;
     char * s = "/home/twoaday/.gnupg/pubring.cdk";
     const char * pk =
@@ -725,8 +728,8 @@ test_import (int argc, char ** argv)
 int
 test_decrypt (int argc, char ** argv)
 {
-    CDK_HD hd;
-    CDK_KEYDB_HD db;
+    cdk_ctx_t hd;
+    cdk_keydb_hd_t db;
     int rc;
     char * s = "/home/twoaday/cvs/gnupg.org/opencdk/tests/sec-with-pwd.gpg";
     const char * ct =
@@ -766,14 +769,72 @@ test_decrypt (int argc, char ** argv)
 
 
 int
-main (int argc, char ** argv)
+test_keydb_routines (void)
 {
-    /*test_encrypt (argc, argv);*/
-    test_decrypt (argc, argv);
-    /*test_sign (argc, argv);*/
-    /*test_verify (argc, argv);*/
-    /*test_import (argc, argv);*/
-    return 0;
+    cdk_keydb_hd_t db;
+    cdk_kbnode_t node;
+    int rc;
+
+    rc = cdk_keydb_new (&db, CDK_DBTYPE_PK_KEYRING, "pubring.gpg", 11);
+    if (!rc)
+        rc = cdk_keydb_search_start (db, CDK_DBSEARCH_NEXT, NULL);
+        /*rc = cdk_keydb_search_start (db, CDK_DBSEARCH_AUTO,
+          "0xBF3DF9B4");*/
+    printf ("rc=%d\n", rc);
+    while (!rc) {
+        rc = cdk_keydb_search (db, &node);
+        if (!rc) {
+            printf ("%s\n",
+                    cdk_kbnode_get_attr (node, CDK_PKT_USER_ID, CDK_ATTR_NAME)
+                    );
+            cdk_kbnode_release (node);
+        }
+    }
+    if (rc)
+        printf ("keydb error `%s'\n", cdk_strerror (rc));
+    cdk_keydb_free (db);
+    return rc;
 }
 
+#if 0
+int
+main (int argc, char ** argv)
+{
+  /*test_encrypt (argc, argv);*/
+  /*test_decrypt (argc, argv);*/
+  /*test_sign (argc, argv);*/
+  /*test_verify (argc, argv);*/
+  /*test_import (argc, argv);*/
+  /*test_keydb_routines ();*/
+  return 0;
+}
+#endif
+
+#include "../src/random.h"
+int
+main (int argc, char ** argv)
+{
+  unsigned char * p, buf[128];
+  
+  cdk_set_log_level (CDK_LOG_DEBUG);
+  p = cdk_random_bytes (32, CDK_RANDOM_STRONG);
+  if (p)
+    {
+      int i;
+      for (i = 0; i < 32; i++)
+        printf ("%02X", p[i]);
+      cdk_free (p);
+    }
+  printf ("\n");
+  cdk_randomize (buf, 128, CDK_RANDOM_STRONG);
+  {
+    int i;
+    for (i = 0; i < 128; i++)
+      printf ("%02X", buf[i]);
+  }
+  printf ("\n");
+  return 0;
+}
+
+  
 
